/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "shell-dbus.h"

#include <string.h>
#ifdef G_OS_UNIX
#  include <gio/gunixfdlist.h>
#endif

#ifdef G_ENABLE_DEBUG
#define g_marshal_value_peek_boolean(v)  g_value_get_boolean (v)
#define g_marshal_value_peek_char(v)     g_value_get_schar (v)
#define g_marshal_value_peek_uchar(v)    g_value_get_uchar (v)
#define g_marshal_value_peek_int(v)      g_value_get_int (v)
#define g_marshal_value_peek_uint(v)     g_value_get_uint (v)
#define g_marshal_value_peek_long(v)     g_value_get_long (v)
#define g_marshal_value_peek_ulong(v)    g_value_get_ulong (v)
#define g_marshal_value_peek_int64(v)    g_value_get_int64 (v)
#define g_marshal_value_peek_uint64(v)   g_value_get_uint64 (v)
#define g_marshal_value_peek_enum(v)     g_value_get_enum (v)
#define g_marshal_value_peek_flags(v)    g_value_get_flags (v)
#define g_marshal_value_peek_float(v)    g_value_get_float (v)
#define g_marshal_value_peek_double(v)   g_value_get_double (v)
#define g_marshal_value_peek_string(v)   (char*) g_value_get_string (v)
#define g_marshal_value_peek_param(v)    g_value_get_param (v)
#define g_marshal_value_peek_boxed(v)    g_value_get_boxed (v)
#define g_marshal_value_peek_pointer(v)  g_value_get_pointer (v)
#define g_marshal_value_peek_object(v)   g_value_get_object (v)
#define g_marshal_value_peek_variant(v)  g_value_get_variant (v)
#else /* !G_ENABLE_DEBUG */
/* WARNING: This code accesses GValues directly, which is UNSUPPORTED API.
 *          Do not access GValues directly in your code. Instead, use the
 *          g_value_get_*() functions
 */
#define g_marshal_value_peek_boolean(v)  (v)->data[0].v_int
#define g_marshal_value_peek_char(v)     (v)->data[0].v_int
#define g_marshal_value_peek_uchar(v)    (v)->data[0].v_uint
#define g_marshal_value_peek_int(v)      (v)->data[0].v_int
#define g_marshal_value_peek_uint(v)     (v)->data[0].v_uint
#define g_marshal_value_peek_long(v)     (v)->data[0].v_long
#define g_marshal_value_peek_ulong(v)    (v)->data[0].v_ulong
#define g_marshal_value_peek_int64(v)    (v)->data[0].v_int64
#define g_marshal_value_peek_uint64(v)   (v)->data[0].v_uint64
#define g_marshal_value_peek_enum(v)     (v)->data[0].v_long
#define g_marshal_value_peek_flags(v)    (v)->data[0].v_ulong
#define g_marshal_value_peek_float(v)    (v)->data[0].v_float
#define g_marshal_value_peek_double(v)   (v)->data[0].v_double
#define g_marshal_value_peek_string(v)   (v)->data[0].v_pointer
#define g_marshal_value_peek_param(v)    (v)->data[0].v_pointer
#define g_marshal_value_peek_boxed(v)    (v)->data[0].v_pointer
#define g_marshal_value_peek_pointer(v)  (v)->data[0].v_pointer
#define g_marshal_value_peek_object(v)   (v)->data[0].v_pointer
#define g_marshal_value_peek_variant(v)  (v)->data[0].v_pointer
#endif /* !G_ENABLE_DEBUG */

typedef struct
{
  GDBusArgInfo parent_struct;
  gboolean use_gvariant;
} _ExtendedGDBusArgInfo;

typedef struct
{
  GDBusMethodInfo parent_struct;
  const gchar *signal_name;
  gboolean pass_fdlist;
} _ExtendedGDBusMethodInfo;

typedef struct
{
  GDBusSignalInfo parent_struct;
  const gchar *signal_name;
} _ExtendedGDBusSignalInfo;

typedef struct
{
  GDBusPropertyInfo parent_struct;
  const gchar *hyphen_name;
  guint use_gvariant : 1;
  guint emits_changed_signal : 1;
} _ExtendedGDBusPropertyInfo;

typedef struct
{
  GDBusInterfaceInfo parent_struct;
  const gchar *hyphen_name;
} _ExtendedGDBusInterfaceInfo;

typedef struct
{
  const _ExtendedGDBusPropertyInfo *info;
  guint prop_id;
  GValue orig_value; /* the value before the change */
} ChangedProperty;

static void
_changed_property_free (ChangedProperty *data)
{
  g_value_unset (&data->orig_value);
  g_free (data);
}

static gboolean
_g_strv_equal0 (gchar **a, gchar **b)
{
  gboolean ret = FALSE;
  guint n;
  if (a == NULL && b == NULL)
    {
      ret = TRUE;
      goto out;
    }
  if (a == NULL || b == NULL)
    goto out;
  if (g_strv_length (a) != g_strv_length (b))
    goto out;
  for (n = 0; a[n] != NULL; n++)
    if (g_strcmp0 (a[n], b[n]) != 0)
      goto out;
  ret = TRUE;
out:
  return ret;
}

static gboolean
_g_variant_equal0 (GVariant *a, GVariant *b)
{
  gboolean ret = FALSE;
  if (a == NULL && b == NULL)
    {
      ret = TRUE;
      goto out;
    }
  if (a == NULL || b == NULL)
    goto out;
  ret = g_variant_equal (a, b);
out:
  return ret;
}

G_GNUC_UNUSED static gboolean
_g_value_equal (const GValue *a, const GValue *b)
{
  gboolean ret = FALSE;
  g_assert (G_VALUE_TYPE (a) == G_VALUE_TYPE (b));
  switch (G_VALUE_TYPE (a))
    {
      case G_TYPE_BOOLEAN:
        ret = (g_value_get_boolean (a) == g_value_get_boolean (b));
        break;
      case G_TYPE_UCHAR:
        ret = (g_value_get_uchar (a) == g_value_get_uchar (b));
        break;
      case G_TYPE_INT:
        ret = (g_value_get_int (a) == g_value_get_int (b));
        break;
      case G_TYPE_UINT:
        ret = (g_value_get_uint (a) == g_value_get_uint (b));
        break;
      case G_TYPE_INT64:
        ret = (g_value_get_int64 (a) == g_value_get_int64 (b));
        break;
      case G_TYPE_UINT64:
        ret = (g_value_get_uint64 (a) == g_value_get_uint64 (b));
        break;
      case G_TYPE_DOUBLE:
        {
          /* Avoid -Wfloat-equal warnings by doing a direct bit compare */
          gdouble da = g_value_get_double (a);
          gdouble db = g_value_get_double (b);
          ret = memcmp (&da, &db, sizeof (gdouble)) == 0;
        }
        break;
      case G_TYPE_STRING:
        ret = (g_strcmp0 (g_value_get_string (a), g_value_get_string (b)) == 0);
        break;
      case G_TYPE_VARIANT:
        ret = _g_variant_equal0 (g_value_get_variant (a), g_value_get_variant (b));
        break;
      default:
        if (G_VALUE_TYPE (a) == G_TYPE_STRV)
          ret = _g_strv_equal0 (g_value_get_boxed (a), g_value_get_boxed (b));
        else
          g_critical ("_g_value_equal() does not handle type %s", g_type_name (G_VALUE_TYPE (a)));
        break;
    }
  return ret;
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectStringFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        const gchar *arg_name,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectStringFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectStringFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_string (param_values + 2),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_BOOLEAN (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectBooleanFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        gboolean arg_locked,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectBooleanFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectBooleanFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_boolean (param_values + 2),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectIntFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        gint arg_accountType,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectIntFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectIntFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_int (param_values + 2),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectStringStringFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        const gchar *arg_password,
        const gchar *arg_hint,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectStringStringFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 4);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectStringStringFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_string (param_values + 2),
              g_marshal_value_peek_string (param_values + 3),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 2);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT64 (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectInt64Func)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        gint64 arg_id,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectInt64Func callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectInt64Func)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_int64 (param_values + 2),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING_INT (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectStringStringIntFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        const gchar *arg_name,
        const gchar *arg_fullname,
        gint arg_accountType,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectStringStringIntFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 5);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectStringStringIntFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_string (param_values + 2),
              g_marshal_value_peek_string (param_values + 3),
              g_marshal_value_peek_int (param_values + 4),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT64_BOOLEAN (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectInt64BooleanFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        gint64 arg_id,
        gboolean arg_removeFiles,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectInt64BooleanFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 4);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectInt64BooleanFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_int64 (param_values + 2),
              g_marshal_value_peek_boolean (param_values + 3),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_UINT (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectUintFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        guint arg_cookie,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectUintFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectUintFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_uint (param_values + 2),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_BOOLEAN (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectStringBooleanFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        const gchar *arg_message,
        gboolean arg_fatal,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectStringBooleanFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 4);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectStringBooleanFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_string (param_values + 2),
              g_marshal_value_peek_boolean (param_values + 3),
              data2);

  g_value_set_boolean (return_value, v_return);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_UINT_STRING_UINT (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectStringUintStringUintFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        const gchar *arg_app_id,
        guint arg_toplevel_xid,
        const gchar *arg_reason,
        guint arg_flags,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectStringUintStringUintFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 6);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectStringUintStringUintFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_string (param_values + 2),
              g_marshal_value_peek_uint (param_values + 3),
              g_marshal_value_peek_string (param_values + 4),
              g_marshal_value_peek_uint (param_values + 5),
              data2);

  g_value_set_boolean (return_value, v_return);
}

/* ------------------------------------------------------------------------
 * Code for interface org.freedesktop.Accounts.User
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:OrgFreedesktopAccountsUser
 * @title: OrgFreedesktopAccountsUser
 * @short_description: Generated C code for the org.freedesktop.Accounts.User D-Bus interface
 *
 * This section contains code for working with the <link linkend="gdbus-interface-org-freedesktop-Accounts-User.top_of_page">org.freedesktop.Accounts.User</link> D-Bus interface in C.
 */

enum
{
  _ORG_FREEDESKTOP_ACCOUNTS_USER_CHANGED,
};

static unsigned _ORG_FREEDESKTOP_ACCOUNTS_USER_SIGNALS[1] = { 0 };

/* ---- Introspection data for org.freedesktop.Accounts.User ---- */

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_user_name_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_user_name_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_user_name_IN_ARG_name.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_user_name_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_user_name_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_user_name_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_user_name =
{
  {
    -1,
    (gchar *) "SetUserName",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_user_name_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_user_name_annotation_info_pointers
  },
  "handle-set-user-name",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_real_name_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_real_name_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_real_name_IN_ARG_name.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_real_name_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_real_name_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_real_name_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_real_name =
{
  {
    -1,
    (gchar *) "SetRealName",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_real_name_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_real_name_annotation_info_pointers
  },
  "handle-set-real-name",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_email_IN_ARG_email =
{
  {
    -1,
    (gchar *) "email",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_email_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_email_IN_ARG_email.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_email_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_email_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_email_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_email =
{
  {
    -1,
    (gchar *) "SetEmail",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_email_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_email_annotation_info_pointers
  },
  "handle-set-email",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_language_IN_ARG_language =
{
  {
    -1,
    (gchar *) "language",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_language_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_language_IN_ARG_language.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_language_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_language_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_language_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_language =
{
  {
    -1,
    (gchar *) "SetLanguage",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_language_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_language_annotation_info_pointers
  },
  "handle-set-language",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_xsession_IN_ARG_x_session =
{
  {
    -1,
    (gchar *) "x_session",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_xsession_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_xsession_IN_ARG_x_session.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_xsession_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_xsession_annotation_info_1 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.CSymbol",
  (gchar *) "user_set_x_session",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_xsession_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_xsession_annotation_info_0,
  &_org_freedesktop_accounts_user_method_set_xsession_annotation_info_1,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_xsession =
{
  {
    -1,
    (gchar *) "SetXSession",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_xsession_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_xsession_annotation_info_pointers
  },
  "handle-set-xsession",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_location_IN_ARG_location =
{
  {
    -1,
    (gchar *) "location",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_location_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_location_IN_ARG_location.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_location_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_location_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_location_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_location =
{
  {
    -1,
    (gchar *) "SetLocation",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_location_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_location_annotation_info_pointers
  },
  "handle-set-location",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_home_directory_IN_ARG_homedir =
{
  {
    -1,
    (gchar *) "homedir",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_home_directory_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_home_directory_IN_ARG_homedir.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_home_directory_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_home_directory_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_home_directory_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_home_directory =
{
  {
    -1,
    (gchar *) "SetHomeDirectory",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_home_directory_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_home_directory_annotation_info_pointers
  },
  "handle-set-home-directory",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_shell_IN_ARG_shell =
{
  {
    -1,
    (gchar *) "shell",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_shell_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_shell_IN_ARG_shell.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_shell_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_shell_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_shell_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_shell =
{
  {
    -1,
    (gchar *) "SetShell",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_shell_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_shell_annotation_info_pointers
  },
  "handle-set-shell",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_icon_file_IN_ARG_filename =
{
  {
    -1,
    (gchar *) "filename",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_icon_file_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_icon_file_IN_ARG_filename.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_icon_file_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_icon_file_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_icon_file_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_icon_file =
{
  {
    -1,
    (gchar *) "SetIconFile",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_icon_file_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_icon_file_annotation_info_pointers
  },
  "handle-set-icon-file",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_locked_IN_ARG_locked =
{
  {
    -1,
    (gchar *) "locked",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_locked_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_locked_IN_ARG_locked.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_locked_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_locked_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_locked_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_locked =
{
  {
    -1,
    (gchar *) "SetLocked",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_locked_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_locked_annotation_info_pointers
  },
  "handle-set-locked",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_account_type_IN_ARG_accountType =
{
  {
    -1,
    (gchar *) "accountType",
    (gchar *) "i",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_account_type_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_account_type_IN_ARG_accountType.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_account_type_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_account_type_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_account_type_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_account_type =
{
  {
    -1,
    (gchar *) "SetAccountType",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_account_type_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_account_type_annotation_info_pointers
  },
  "handle-set-account-type",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_password_mode_IN_ARG_mode =
{
  {
    -1,
    (gchar *) "mode",
    (gchar *) "i",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_password_mode_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_password_mode_IN_ARG_mode.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_password_mode_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_password_mode_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_password_mode_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_password_mode =
{
  {
    -1,
    (gchar *) "SetPasswordMode",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_password_mode_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_password_mode_annotation_info_pointers
  },
  "handle-set-password-mode",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_password_IN_ARG_password =
{
  {
    -1,
    (gchar *) "password",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_password_IN_ARG_hint =
{
  {
    -1,
    (gchar *) "hint",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_password_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_password_IN_ARG_password.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_password_IN_ARG_hint.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_password_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_password_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_password_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_password =
{
  {
    -1,
    (gchar *) "SetPassword",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_password_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_password_annotation_info_pointers
  },
  "handle-set-password",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_password_hint_IN_ARG_hint =
{
  {
    -1,
    (gchar *) "hint",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_password_hint_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_password_hint_IN_ARG_hint.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_password_hint_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_password_hint_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_password_hint_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_password_hint =
{
  {
    -1,
    (gchar *) "SetPasswordHint",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_password_hint_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_password_hint_annotation_info_pointers
  },
  "handle-set-password-hint",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_user_method_info_set_automatic_login_IN_ARG_enabled =
{
  {
    -1,
    (gchar *) "enabled",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_user_method_info_set_automatic_login_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_automatic_login_IN_ARG_enabled.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_user_method_set_automatic_login_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_user_method_set_automatic_login_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_set_automatic_login_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_user_method_info_set_automatic_login =
{
  {
    -1,
    (gchar *) "SetAutomaticLogin",
    (GDBusArgInfo **) &_org_freedesktop_accounts_user_method_info_set_automatic_login_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_user_method_set_automatic_login_annotation_info_pointers
  },
  "handle-set-automatic-login",
  FALSE
};

static const GDBusMethodInfo * const _org_freedesktop_accounts_user_method_info_pointers[] =
{
  &_org_freedesktop_accounts_user_method_info_set_user_name.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_real_name.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_email.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_language.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_xsession.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_location.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_home_directory.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_shell.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_icon_file.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_locked.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_account_type.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_password_mode.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_password.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_password_hint.parent_struct,
  &_org_freedesktop_accounts_user_method_info_set_automatic_login.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _org_freedesktop_accounts_user_signal_info_changed =
{
  {
    -1,
    (gchar *) "Changed",
    NULL,
    NULL
  },
  "changed"
};

static const GDBusSignalInfo * const _org_freedesktop_accounts_user_signal_info_pointers[] =
{
  &_org_freedesktop_accounts_user_signal_info_changed.parent_struct,
  NULL
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_uid =
{
  {
    -1,
    (gchar *) "Uid",
    (gchar *) "t",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "uid",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_user_name =
{
  {
    -1,
    (gchar *) "UserName",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "user-name",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_real_name =
{
  {
    -1,
    (gchar *) "RealName",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "real-name",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_account_type =
{
  {
    -1,
    (gchar *) "AccountType",
    (gchar *) "i",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "account-type",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_home_directory =
{
  {
    -1,
    (gchar *) "HomeDirectory",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "home-directory",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_shell =
{
  {
    -1,
    (gchar *) "Shell",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "shell",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_email =
{
  {
    -1,
    (gchar *) "Email",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "email",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_language =
{
  {
    -1,
    (gchar *) "Language",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "language",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_xsession =
{
  {
    -1,
    (gchar *) "XSession",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "xsession",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_location =
{
  {
    -1,
    (gchar *) "Location",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "location",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_login_frequency =
{
  {
    -1,
    (gchar *) "LoginFrequency",
    (gchar *) "t",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "login-frequency",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_login_time =
{
  {
    -1,
    (gchar *) "LoginTime",
    (gchar *) "x",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "login-time",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_login_history =
{
  {
    -1,
    (gchar *) "LoginHistory",
    (gchar *) "a(xxa{sv})",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "login-history",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_icon_file =
{
  {
    -1,
    (gchar *) "IconFile",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "icon-file",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_locked =
{
  {
    -1,
    (gchar *) "Locked",
    (gchar *) "b",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "locked",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_password_mode =
{
  {
    -1,
    (gchar *) "PasswordMode",
    (gchar *) "i",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "password-mode",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_password_hint =
{
  {
    -1,
    (gchar *) "PasswordHint",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "password-hint",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_automatic_login =
{
  {
    -1,
    (gchar *) "AutomaticLogin",
    (gchar *) "b",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "automatic-login",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_system_account =
{
  {
    -1,
    (gchar *) "SystemAccount",
    (gchar *) "b",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "system-account",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_user_property_info_local_account =
{
  {
    -1,
    (gchar *) "LocalAccount",
    (gchar *) "b",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "local-account",
  FALSE,
  TRUE
};

static const GDBusPropertyInfo * const _org_freedesktop_accounts_user_property_info_pointers[] =
{
  &_org_freedesktop_accounts_user_property_info_uid.parent_struct,
  &_org_freedesktop_accounts_user_property_info_user_name.parent_struct,
  &_org_freedesktop_accounts_user_property_info_real_name.parent_struct,
  &_org_freedesktop_accounts_user_property_info_account_type.parent_struct,
  &_org_freedesktop_accounts_user_property_info_home_directory.parent_struct,
  &_org_freedesktop_accounts_user_property_info_shell.parent_struct,
  &_org_freedesktop_accounts_user_property_info_email.parent_struct,
  &_org_freedesktop_accounts_user_property_info_language.parent_struct,
  &_org_freedesktop_accounts_user_property_info_xsession.parent_struct,
  &_org_freedesktop_accounts_user_property_info_location.parent_struct,
  &_org_freedesktop_accounts_user_property_info_login_frequency.parent_struct,
  &_org_freedesktop_accounts_user_property_info_login_time.parent_struct,
  &_org_freedesktop_accounts_user_property_info_login_history.parent_struct,
  &_org_freedesktop_accounts_user_property_info_icon_file.parent_struct,
  &_org_freedesktop_accounts_user_property_info_locked.parent_struct,
  &_org_freedesktop_accounts_user_property_info_password_mode.parent_struct,
  &_org_freedesktop_accounts_user_property_info_password_hint.parent_struct,
  &_org_freedesktop_accounts_user_property_info_automatic_login.parent_struct,
  &_org_freedesktop_accounts_user_property_info_system_account.parent_struct,
  &_org_freedesktop_accounts_user_property_info_local_account.parent_struct,
  NULL
};

static const _ExtendedGDBusInterfaceInfo _org_freedesktop_accounts_user_interface_info =
{
  {
    -1,
    (gchar *) "org.freedesktop.Accounts.User",
    (GDBusMethodInfo **) &_org_freedesktop_accounts_user_method_info_pointers,
    (GDBusSignalInfo **) &_org_freedesktop_accounts_user_signal_info_pointers,
    (GDBusPropertyInfo **) &_org_freedesktop_accounts_user_property_info_pointers,
    NULL
  },
  "org-freedesktop-accounts-user",
};


/**
 * org_freedesktop_accounts_user_interface_info:
 *
 * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-Accounts-User.top_of_page">org.freedesktop.Accounts.User</link> D-Bus interface.
 *
 * Returns: (transfer none): A #GDBusInterfaceInfo. Do not free.
 */
GDBusInterfaceInfo *
org_freedesktop_accounts_user_interface_info (void)
{
  return (GDBusInterfaceInfo *) &_org_freedesktop_accounts_user_interface_info.parent_struct;
}

/**
 * org_freedesktop_accounts_user_override_properties:
 * @klass: The class structure for a #GObject derived class.
 * @property_id_begin: The property id to assign to the first overridden property.
 *
 * Overrides all #GObject properties in the #OrgFreedesktopAccountsUser interface for a concrete class.
 * The properties are overridden in the order they are defined.
 *
 * Returns: The last property id.
 */
guint
org_freedesktop_accounts_user_override_properties (GObjectClass *klass, guint property_id_begin)
{
  g_object_class_override_property (klass, property_id_begin++, "uid");
  g_object_class_override_property (klass, property_id_begin++, "user-name");
  g_object_class_override_property (klass, property_id_begin++, "real-name");
  g_object_class_override_property (klass, property_id_begin++, "account-type");
  g_object_class_override_property (klass, property_id_begin++, "home-directory");
  g_object_class_override_property (klass, property_id_begin++, "shell");
  g_object_class_override_property (klass, property_id_begin++, "email");
  g_object_class_override_property (klass, property_id_begin++, "language");
  g_object_class_override_property (klass, property_id_begin++, "xsession");
  g_object_class_override_property (klass, property_id_begin++, "location");
  g_object_class_override_property (klass, property_id_begin++, "login-frequency");
  g_object_class_override_property (klass, property_id_begin++, "login-time");
  g_object_class_override_property (klass, property_id_begin++, "login-history");
  g_object_class_override_property (klass, property_id_begin++, "icon-file");
  g_object_class_override_property (klass, property_id_begin++, "locked");
  g_object_class_override_property (klass, property_id_begin++, "password-mode");
  g_object_class_override_property (klass, property_id_begin++, "password-hint");
  g_object_class_override_property (klass, property_id_begin++, "automatic-login");
  g_object_class_override_property (klass, property_id_begin++, "system-account");
  g_object_class_override_property (klass, property_id_begin++, "local-account");
  return property_id_begin - 1;
}


inline static void
org_freedesktop_accounts_user_signal_marshal_changed (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__VOID (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_user_name (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_real_name (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_email (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_language (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_xsession (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_location (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_home_directory (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_shell (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_icon_file (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_locked (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_BOOLEAN (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_account_type (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_password_mode (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_password (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_password_hint (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_user_method_marshal_set_automatic_login (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_BOOLEAN (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}


/**
 * OrgFreedesktopAccountsUser:
 *
 * Abstract interface type for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-Accounts-User.top_of_page">org.freedesktop.Accounts.User</link>.
 */

/**
 * OrgFreedesktopAccountsUserIface:
 * @parent_iface: The parent interface.
 * @handle_set_account_type: Handler for the #OrgFreedesktopAccountsUser::handle-set-account-type signal.
 * @handle_set_automatic_login: Handler for the #OrgFreedesktopAccountsUser::handle-set-automatic-login signal.
 * @handle_set_email: Handler for the #OrgFreedesktopAccountsUser::handle-set-email signal.
 * @handle_set_home_directory: Handler for the #OrgFreedesktopAccountsUser::handle-set-home-directory signal.
 * @handle_set_icon_file: Handler for the #OrgFreedesktopAccountsUser::handle-set-icon-file signal.
 * @handle_set_language: Handler for the #OrgFreedesktopAccountsUser::handle-set-language signal.
 * @handle_set_location: Handler for the #OrgFreedesktopAccountsUser::handle-set-location signal.
 * @handle_set_locked: Handler for the #OrgFreedesktopAccountsUser::handle-set-locked signal.
 * @handle_set_password: Handler for the #OrgFreedesktopAccountsUser::handle-set-password signal.
 * @handle_set_password_hint: Handler for the #OrgFreedesktopAccountsUser::handle-set-password-hint signal.
 * @handle_set_password_mode: Handler for the #OrgFreedesktopAccountsUser::handle-set-password-mode signal.
 * @handle_set_real_name: Handler for the #OrgFreedesktopAccountsUser::handle-set-real-name signal.
 * @handle_set_shell: Handler for the #OrgFreedesktopAccountsUser::handle-set-shell signal.
 * @handle_set_user_name: Handler for the #OrgFreedesktopAccountsUser::handle-set-user-name signal.
 * @handle_set_xsession: Handler for the #OrgFreedesktopAccountsUser::handle-set-xsession signal.
 * @get_account_type: Getter for the #OrgFreedesktopAccountsUser:account-type property.
 * @get_automatic_login: Getter for the #OrgFreedesktopAccountsUser:automatic-login property.
 * @get_email: Getter for the #OrgFreedesktopAccountsUser:email property.
 * @get_home_directory: Getter for the #OrgFreedesktopAccountsUser:home-directory property.
 * @get_icon_file: Getter for the #OrgFreedesktopAccountsUser:icon-file property.
 * @get_language: Getter for the #OrgFreedesktopAccountsUser:language property.
 * @get_local_account: Getter for the #OrgFreedesktopAccountsUser:local-account property.
 * @get_location: Getter for the #OrgFreedesktopAccountsUser:location property.
 * @get_locked: Getter for the #OrgFreedesktopAccountsUser:locked property.
 * @get_login_frequency: Getter for the #OrgFreedesktopAccountsUser:login-frequency property.
 * @get_login_history: Getter for the #OrgFreedesktopAccountsUser:login-history property.
 * @get_login_time: Getter for the #OrgFreedesktopAccountsUser:login-time property.
 * @get_password_hint: Getter for the #OrgFreedesktopAccountsUser:password-hint property.
 * @get_password_mode: Getter for the #OrgFreedesktopAccountsUser:password-mode property.
 * @get_real_name: Getter for the #OrgFreedesktopAccountsUser:real-name property.
 * @get_shell: Getter for the #OrgFreedesktopAccountsUser:shell property.
 * @get_system_account: Getter for the #OrgFreedesktopAccountsUser:system-account property.
 * @get_uid: Getter for the #OrgFreedesktopAccountsUser:uid property.
 * @get_user_name: Getter for the #OrgFreedesktopAccountsUser:user-name property.
 * @get_xsession: Getter for the #OrgFreedesktopAccountsUser:xsession property.
 * @changed: Handler for the #OrgFreedesktopAccountsUser::changed signal.
 *
 * Virtual table for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-Accounts-User.top_of_page">org.freedesktop.Accounts.User</link>.
 */

typedef OrgFreedesktopAccountsUserIface OrgFreedesktopAccountsUserInterface;
G_DEFINE_INTERFACE (OrgFreedesktopAccountsUser, org_freedesktop_accounts_user, G_TYPE_OBJECT)

static void
org_freedesktop_accounts_user_default_init (OrgFreedesktopAccountsUserIface *iface)
{
  /* GObject signals for incoming D-Bus method calls: */
  /**
   * OrgFreedesktopAccountsUser::handle-set-user-name:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetUserName">SetUserName()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_user_name() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-user-name",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_user_name),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_user_name,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccountsUser::handle-set-real-name:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetRealName">SetRealName()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_real_name() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-real-name",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_real_name),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_real_name,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccountsUser::handle-set-email:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_email: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetEmail">SetEmail()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_email() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-email",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_email),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_email,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccountsUser::handle-set-language:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_language: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetLanguage">SetLanguage()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_language() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-language",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_language),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_language,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccountsUser::handle-set-xsession:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_x_session: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetXSession">SetXSession()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_xsession() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-xsession",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_xsession),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_xsession,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccountsUser::handle-set-location:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_location: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetLocation">SetLocation()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_location() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-location",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_location),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_location,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccountsUser::handle-set-home-directory:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_homedir: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetHomeDirectory">SetHomeDirectory()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_home_directory() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-home-directory",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_home_directory),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_home_directory,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccountsUser::handle-set-shell:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_shell: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetShell">SetShell()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_shell() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-shell",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_shell),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_shell,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccountsUser::handle-set-icon-file:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_filename: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetIconFile">SetIconFile()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_icon_file() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-icon-file",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_icon_file),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_icon_file,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccountsUser::handle-set-locked:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_locked: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetLocked">SetLocked()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_locked() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-locked",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_locked),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_locked,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_BOOLEAN);

  /**
   * OrgFreedesktopAccountsUser::handle-set-account-type:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_accountType: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetAccountType">SetAccountType()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_account_type() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-account-type",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_account_type),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_account_type,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_INT);

  /**
   * OrgFreedesktopAccountsUser::handle-set-password-mode:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_mode: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetPasswordMode">SetPasswordMode()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_password_mode() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-password-mode",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_password_mode),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_password_mode,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_INT);

  /**
   * OrgFreedesktopAccountsUser::handle-set-password:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_password: Argument passed by remote caller.
   * @arg_hint: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetPassword">SetPassword()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_password() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-password",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_password),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_password,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccountsUser::handle-set-password-hint:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_hint: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetPasswordHint">SetPasswordHint()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_password_hint() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-password-hint",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_password_hint),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_password_hint,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccountsUser::handle-set-automatic-login:
   * @object: A #OrgFreedesktopAccountsUser.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_enabled: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetAutomaticLogin">SetAutomaticLogin()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_user_complete_set_automatic_login() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-automatic-login",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, handle_set_automatic_login),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_user_method_marshal_set_automatic_login,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_BOOLEAN);

  /* GObject signals for received D-Bus signals: */
  /**
   * OrgFreedesktopAccountsUser::changed:
   * @object: A #OrgFreedesktopAccountsUser.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-org-freedesktop-Accounts-User.Changed">"Changed"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _ORG_FREEDESKTOP_ACCOUNTS_USER_SIGNALS[_ORG_FREEDESKTOP_ACCOUNTS_USER_CHANGED] =
    g_signal_new ("changed",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (OrgFreedesktopAccountsUserIface, changed),
      NULL,
      NULL,
      org_freedesktop_accounts_user_signal_marshal_changed,
      G_TYPE_NONE,
      0);

  /* GObject properties for D-Bus properties: */
  /**
   * OrgFreedesktopAccountsUser:uid:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.Uid">"Uid"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_uint64 ("uid", "Uid", "Uid", 0, G_MAXUINT64, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:user-name:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.UserName">"UserName"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("user-name", "UserName", "UserName", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:real-name:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.RealName">"RealName"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("real-name", "RealName", "RealName", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:account-type:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.AccountType">"AccountType"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_int ("account-type", "AccountType", "AccountType", G_MININT32, G_MAXINT32, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:home-directory:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.HomeDirectory">"HomeDirectory"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("home-directory", "HomeDirectory", "HomeDirectory", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:shell:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.Shell">"Shell"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("shell", "Shell", "Shell", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:email:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.Email">"Email"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("email", "Email", "Email", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:language:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.Language">"Language"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("language", "Language", "Language", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:xsession:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.XSession">"XSession"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("xsession", "XSession", "XSession", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:location:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.Location">"Location"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("location", "Location", "Location", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:login-frequency:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.LoginFrequency">"LoginFrequency"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_uint64 ("login-frequency", "LoginFrequency", "LoginFrequency", 0, G_MAXUINT64, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:login-time:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.LoginTime">"LoginTime"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_int64 ("login-time", "LoginTime", "LoginTime", G_MININT64, G_MAXINT64, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:login-history:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.LoginHistory">"LoginHistory"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_variant ("login-history", "LoginHistory", "LoginHistory", G_VARIANT_TYPE ("a(xxa{sv})"), NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:icon-file:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.IconFile">"IconFile"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("icon-file", "IconFile", "IconFile", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:locked:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.Locked">"Locked"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_boolean ("locked", "Locked", "Locked", FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:password-mode:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.PasswordMode">"PasswordMode"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_int ("password-mode", "PasswordMode", "PasswordMode", G_MININT32, G_MAXINT32, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:password-hint:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.PasswordHint">"PasswordHint"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("password-hint", "PasswordHint", "PasswordHint", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:automatic-login:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.AutomaticLogin">"AutomaticLogin"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_boolean ("automatic-login", "AutomaticLogin", "AutomaticLogin", FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:system-account:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.SystemAccount">"SystemAccount"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_boolean ("system-account", "SystemAccount", "SystemAccount", FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgFreedesktopAccountsUser:local-account:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts-User.LocalAccount">"LocalAccount"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_boolean ("local-account", "LocalAccount", "LocalAccount", FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
}

/**
 * org_freedesktop_accounts_user_get_uid: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Uid">"Uid"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
guint64 
org_freedesktop_accounts_user_get_uid (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), 0);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_uid (object);
}

/**
 * org_freedesktop_accounts_user_set_uid: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Uid">"Uid"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_uid (OrgFreedesktopAccountsUser *object, guint64 value)
{
  g_object_set (G_OBJECT (object), "uid", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_user_name: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.UserName">"UserName"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use org_freedesktop_accounts_user_dup_user_name() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
org_freedesktop_accounts_user_get_user_name (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), NULL);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_user_name (object);
}

/**
 * org_freedesktop_accounts_user_dup_user_name: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets a copy of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.UserName">"UserName"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
org_freedesktop_accounts_user_dup_user_name (OrgFreedesktopAccountsUser *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "user-name", &value, NULL);
  return value;
}

/**
 * org_freedesktop_accounts_user_set_user_name: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.UserName">"UserName"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_user_name (OrgFreedesktopAccountsUser *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "user-name", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_real_name: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.RealName">"RealName"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use org_freedesktop_accounts_user_dup_real_name() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
org_freedesktop_accounts_user_get_real_name (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), NULL);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_real_name (object);
}

/**
 * org_freedesktop_accounts_user_dup_real_name: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets a copy of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.RealName">"RealName"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
org_freedesktop_accounts_user_dup_real_name (OrgFreedesktopAccountsUser *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "real-name", &value, NULL);
  return value;
}

/**
 * org_freedesktop_accounts_user_set_real_name: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.RealName">"RealName"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_real_name (OrgFreedesktopAccountsUser *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "real-name", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_account_type: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.AccountType">"AccountType"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
gint 
org_freedesktop_accounts_user_get_account_type (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), 0);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_account_type (object);
}

/**
 * org_freedesktop_accounts_user_set_account_type: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.AccountType">"AccountType"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_account_type (OrgFreedesktopAccountsUser *object, gint value)
{
  g_object_set (G_OBJECT (object), "account-type", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_home_directory: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.HomeDirectory">"HomeDirectory"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use org_freedesktop_accounts_user_dup_home_directory() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
org_freedesktop_accounts_user_get_home_directory (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), NULL);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_home_directory (object);
}

/**
 * org_freedesktop_accounts_user_dup_home_directory: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets a copy of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.HomeDirectory">"HomeDirectory"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
org_freedesktop_accounts_user_dup_home_directory (OrgFreedesktopAccountsUser *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "home-directory", &value, NULL);
  return value;
}

/**
 * org_freedesktop_accounts_user_set_home_directory: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.HomeDirectory">"HomeDirectory"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_home_directory (OrgFreedesktopAccountsUser *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "home-directory", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_shell: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Shell">"Shell"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use org_freedesktop_accounts_user_dup_shell() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
org_freedesktop_accounts_user_get_shell (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), NULL);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_shell (object);
}

/**
 * org_freedesktop_accounts_user_dup_shell: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets a copy of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Shell">"Shell"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
org_freedesktop_accounts_user_dup_shell (OrgFreedesktopAccountsUser *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "shell", &value, NULL);
  return value;
}

/**
 * org_freedesktop_accounts_user_set_shell: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Shell">"Shell"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_shell (OrgFreedesktopAccountsUser *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "shell", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_email: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Email">"Email"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use org_freedesktop_accounts_user_dup_email() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
org_freedesktop_accounts_user_get_email (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), NULL);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_email (object);
}

/**
 * org_freedesktop_accounts_user_dup_email: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets a copy of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Email">"Email"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
org_freedesktop_accounts_user_dup_email (OrgFreedesktopAccountsUser *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "email", &value, NULL);
  return value;
}

/**
 * org_freedesktop_accounts_user_set_email: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Email">"Email"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_email (OrgFreedesktopAccountsUser *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "email", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_language: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Language">"Language"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use org_freedesktop_accounts_user_dup_language() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
org_freedesktop_accounts_user_get_language (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), NULL);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_language (object);
}

/**
 * org_freedesktop_accounts_user_dup_language: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets a copy of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Language">"Language"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
org_freedesktop_accounts_user_dup_language (OrgFreedesktopAccountsUser *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "language", &value, NULL);
  return value;
}

/**
 * org_freedesktop_accounts_user_set_language: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Language">"Language"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_language (OrgFreedesktopAccountsUser *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "language", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_xsession: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.XSession">"XSession"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use org_freedesktop_accounts_user_dup_xsession() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
org_freedesktop_accounts_user_get_xsession (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), NULL);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_xsession (object);
}

/**
 * org_freedesktop_accounts_user_dup_xsession: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets a copy of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.XSession">"XSession"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
org_freedesktop_accounts_user_dup_xsession (OrgFreedesktopAccountsUser *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "xsession", &value, NULL);
  return value;
}

/**
 * org_freedesktop_accounts_user_set_xsession: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.XSession">"XSession"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_xsession (OrgFreedesktopAccountsUser *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "xsession", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_location: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Location">"Location"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use org_freedesktop_accounts_user_dup_location() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
org_freedesktop_accounts_user_get_location (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), NULL);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_location (object);
}

/**
 * org_freedesktop_accounts_user_dup_location: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets a copy of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Location">"Location"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
org_freedesktop_accounts_user_dup_location (OrgFreedesktopAccountsUser *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "location", &value, NULL);
  return value;
}

/**
 * org_freedesktop_accounts_user_set_location: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Location">"Location"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_location (OrgFreedesktopAccountsUser *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "location", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_login_frequency: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.LoginFrequency">"LoginFrequency"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
guint64 
org_freedesktop_accounts_user_get_login_frequency (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), 0);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_login_frequency (object);
}

/**
 * org_freedesktop_accounts_user_set_login_frequency: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.LoginFrequency">"LoginFrequency"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_login_frequency (OrgFreedesktopAccountsUser *object, guint64 value)
{
  g_object_set (G_OBJECT (object), "login-frequency", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_login_time: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.LoginTime">"LoginTime"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
gint64 
org_freedesktop_accounts_user_get_login_time (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), 0);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_login_time (object);
}

/**
 * org_freedesktop_accounts_user_set_login_time: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.LoginTime">"LoginTime"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_login_time (OrgFreedesktopAccountsUser *object, gint64 value)
{
  g_object_set (G_OBJECT (object), "login-time", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_login_history: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.LoginHistory">"LoginHistory"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use org_freedesktop_accounts_user_dup_login_history() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
GVariant *
org_freedesktop_accounts_user_get_login_history (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), NULL);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_login_history (object);
}

/**
 * org_freedesktop_accounts_user_dup_login_history: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets a copy of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.LoginHistory">"LoginHistory"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_variant_unref().
 */
GVariant *
org_freedesktop_accounts_user_dup_login_history (OrgFreedesktopAccountsUser *object)
{
  GVariant *value;
  g_object_get (G_OBJECT (object), "login-history", &value, NULL);
  return value;
}

/**
 * org_freedesktop_accounts_user_set_login_history: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.LoginHistory">"LoginHistory"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_login_history (OrgFreedesktopAccountsUser *object, GVariant *value)
{
  g_object_set (G_OBJECT (object), "login-history", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_icon_file: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.IconFile">"IconFile"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use org_freedesktop_accounts_user_dup_icon_file() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
org_freedesktop_accounts_user_get_icon_file (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), NULL);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_icon_file (object);
}

/**
 * org_freedesktop_accounts_user_dup_icon_file: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets a copy of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.IconFile">"IconFile"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
org_freedesktop_accounts_user_dup_icon_file (OrgFreedesktopAccountsUser *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "icon-file", &value, NULL);
  return value;
}

/**
 * org_freedesktop_accounts_user_set_icon_file: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.IconFile">"IconFile"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_icon_file (OrgFreedesktopAccountsUser *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "icon-file", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_locked: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Locked">"Locked"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
gboolean 
org_freedesktop_accounts_user_get_locked (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), FALSE);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_locked (object);
}

/**
 * org_freedesktop_accounts_user_set_locked: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.Locked">"Locked"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_locked (OrgFreedesktopAccountsUser *object, gboolean value)
{
  g_object_set (G_OBJECT (object), "locked", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_password_mode: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.PasswordMode">"PasswordMode"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
gint 
org_freedesktop_accounts_user_get_password_mode (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), 0);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_password_mode (object);
}

/**
 * org_freedesktop_accounts_user_set_password_mode: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.PasswordMode">"PasswordMode"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_password_mode (OrgFreedesktopAccountsUser *object, gint value)
{
  g_object_set (G_OBJECT (object), "password-mode", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_password_hint: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.PasswordHint">"PasswordHint"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use org_freedesktop_accounts_user_dup_password_hint() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
org_freedesktop_accounts_user_get_password_hint (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), NULL);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_password_hint (object);
}

/**
 * org_freedesktop_accounts_user_dup_password_hint: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets a copy of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.PasswordHint">"PasswordHint"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
org_freedesktop_accounts_user_dup_password_hint (OrgFreedesktopAccountsUser *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "password-hint", &value, NULL);
  return value;
}

/**
 * org_freedesktop_accounts_user_set_password_hint: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.PasswordHint">"PasswordHint"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_password_hint (OrgFreedesktopAccountsUser *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "password-hint", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_automatic_login: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.AutomaticLogin">"AutomaticLogin"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
gboolean 
org_freedesktop_accounts_user_get_automatic_login (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), FALSE);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_automatic_login (object);
}

/**
 * org_freedesktop_accounts_user_set_automatic_login: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.AutomaticLogin">"AutomaticLogin"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_automatic_login (OrgFreedesktopAccountsUser *object, gboolean value)
{
  g_object_set (G_OBJECT (object), "automatic-login", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_system_account: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.SystemAccount">"SystemAccount"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
gboolean 
org_freedesktop_accounts_user_get_system_account (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), FALSE);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_system_account (object);
}

/**
 * org_freedesktop_accounts_user_set_system_account: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.SystemAccount">"SystemAccount"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_system_account (OrgFreedesktopAccountsUser *object, gboolean value)
{
  g_object_set (G_OBJECT (object), "system-account", value, NULL);
}

/**
 * org_freedesktop_accounts_user_get_local_account: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts-User.LocalAccount">"LocalAccount"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
gboolean 
org_freedesktop_accounts_user_get_local_account (OrgFreedesktopAccountsUser *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS_USER (object), FALSE);

  return ORG_FREEDESKTOP_ACCOUNTS_USER_GET_IFACE (object)->get_local_account (object);
}

/**
 * org_freedesktop_accounts_user_set_local_account: (skip)
 * @object: A #OrgFreedesktopAccountsUser.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts-User.LocalAccount">"LocalAccount"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_user_set_local_account (OrgFreedesktopAccountsUser *object, gboolean value)
{
  g_object_set (G_OBJECT (object), "local-account", value, NULL);
}

/**
 * org_freedesktop_accounts_user_emit_changed:
 * @object: A #OrgFreedesktopAccountsUser.
 *
 * Emits the <link linkend="gdbus-signal-org-freedesktop-Accounts-User.Changed">"Changed"</link> D-Bus signal.
 */
void
org_freedesktop_accounts_user_emit_changed (
    OrgFreedesktopAccountsUser *object)
{
  g_signal_emit (object, _ORG_FREEDESKTOP_ACCOUNTS_USER_SIGNALS[_ORG_FREEDESKTOP_ACCOUNTS_USER_CHANGED], 0);
}

/**
 * org_freedesktop_accounts_user_call_set_user_name:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetUserName">SetUserName()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_user_name_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_user_name_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_user_name (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetUserName",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_user_name_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_user_name().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_user_name().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_user_name_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_user_name_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetUserName">SetUserName()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_user_name() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_user_name_sync (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_name,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetUserName",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_real_name:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetRealName">SetRealName()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_real_name_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_real_name_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_real_name (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetRealName",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_real_name_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_real_name().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_real_name().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_real_name_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_real_name_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetRealName">SetRealName()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_real_name() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_real_name_sync (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_name,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetRealName",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_email:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_email: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetEmail">SetEmail()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_email_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_email_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_email (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_email,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetEmail",
    g_variant_new ("(s)",
                   arg_email),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_email_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_email().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_email().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_email_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_email_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_email: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetEmail">SetEmail()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_email() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_email_sync (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_email,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetEmail",
    g_variant_new ("(s)",
                   arg_email),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_language:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_language: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetLanguage">SetLanguage()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_language_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_language_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_language (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_language,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetLanguage",
    g_variant_new ("(s)",
                   arg_language),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_language_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_language().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_language().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_language_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_language_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_language: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetLanguage">SetLanguage()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_language() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_language_sync (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_language,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetLanguage",
    g_variant_new ("(s)",
                   arg_language),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_xsession:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_x_session: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetXSession">SetXSession()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_xsession_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_xsession_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_xsession (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_x_session,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetXSession",
    g_variant_new ("(s)",
                   arg_x_session),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_xsession_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_xsession().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_xsession().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_xsession_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_xsession_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_x_session: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetXSession">SetXSession()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_xsession() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_xsession_sync (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_x_session,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetXSession",
    g_variant_new ("(s)",
                   arg_x_session),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_location:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_location: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetLocation">SetLocation()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_location_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_location_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_location (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_location,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetLocation",
    g_variant_new ("(s)",
                   arg_location),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_location_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_location().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_location().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_location_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_location_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_location: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetLocation">SetLocation()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_location() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_location_sync (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_location,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetLocation",
    g_variant_new ("(s)",
                   arg_location),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_home_directory:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_homedir: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetHomeDirectory">SetHomeDirectory()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_home_directory_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_home_directory_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_home_directory (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_homedir,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetHomeDirectory",
    g_variant_new ("(s)",
                   arg_homedir),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_home_directory_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_home_directory().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_home_directory().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_home_directory_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_home_directory_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_homedir: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetHomeDirectory">SetHomeDirectory()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_home_directory() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_home_directory_sync (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_homedir,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetHomeDirectory",
    g_variant_new ("(s)",
                   arg_homedir),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_shell:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_shell: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetShell">SetShell()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_shell_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_shell_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_shell (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_shell,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetShell",
    g_variant_new ("(s)",
                   arg_shell),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_shell_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_shell().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_shell().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_shell_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_shell_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_shell: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetShell">SetShell()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_shell() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_shell_sync (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_shell,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetShell",
    g_variant_new ("(s)",
                   arg_shell),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_icon_file:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_filename: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetIconFile">SetIconFile()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_icon_file_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_icon_file_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_icon_file (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_filename,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetIconFile",
    g_variant_new ("(s)",
                   arg_filename),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_icon_file_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_icon_file().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_icon_file().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_icon_file_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_icon_file_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_filename: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetIconFile">SetIconFile()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_icon_file() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_icon_file_sync (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_filename,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetIconFile",
    g_variant_new ("(s)",
                   arg_filename),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_locked:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_locked: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetLocked">SetLocked()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_locked_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_locked_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_locked (
    OrgFreedesktopAccountsUser *proxy,
    gboolean arg_locked,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetLocked",
    g_variant_new ("(b)",
                   arg_locked),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_locked_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_locked().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_locked().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_locked_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_locked_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_locked: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetLocked">SetLocked()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_locked() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_locked_sync (
    OrgFreedesktopAccountsUser *proxy,
    gboolean arg_locked,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetLocked",
    g_variant_new ("(b)",
                   arg_locked),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_account_type:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_accountType: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetAccountType">SetAccountType()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_account_type_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_account_type_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_account_type (
    OrgFreedesktopAccountsUser *proxy,
    gint arg_accountType,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetAccountType",
    g_variant_new ("(i)",
                   arg_accountType),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_account_type_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_account_type().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_account_type().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_account_type_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_account_type_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_accountType: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetAccountType">SetAccountType()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_account_type() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_account_type_sync (
    OrgFreedesktopAccountsUser *proxy,
    gint arg_accountType,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetAccountType",
    g_variant_new ("(i)",
                   arg_accountType),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_password_mode:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_mode: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetPasswordMode">SetPasswordMode()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_password_mode_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_password_mode_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_password_mode (
    OrgFreedesktopAccountsUser *proxy,
    gint arg_mode,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetPasswordMode",
    g_variant_new ("(i)",
                   arg_mode),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_password_mode_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_password_mode().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_password_mode().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_password_mode_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_password_mode_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_mode: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetPasswordMode">SetPasswordMode()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_password_mode() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_password_mode_sync (
    OrgFreedesktopAccountsUser *proxy,
    gint arg_mode,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetPasswordMode",
    g_variant_new ("(i)",
                   arg_mode),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_password:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_password: Argument to pass with the method invocation.
 * @arg_hint: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetPassword">SetPassword()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_password_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_password_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_password (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_password,
    const gchar *arg_hint,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetPassword",
    g_variant_new ("(ss)",
                   arg_password,
                   arg_hint),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_password_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_password().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_password().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_password_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_password_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_password: Argument to pass with the method invocation.
 * @arg_hint: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetPassword">SetPassword()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_password() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_password_sync (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_password,
    const gchar *arg_hint,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetPassword",
    g_variant_new ("(ss)",
                   arg_password,
                   arg_hint),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_password_hint:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_hint: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetPasswordHint">SetPasswordHint()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_password_hint_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_password_hint_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_password_hint (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_hint,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetPasswordHint",
    g_variant_new ("(s)",
                   arg_hint),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_password_hint_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_password_hint().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_password_hint().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_password_hint_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_password_hint_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_hint: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetPasswordHint">SetPasswordHint()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_password_hint() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_password_hint_sync (
    OrgFreedesktopAccountsUser *proxy,
    const gchar *arg_hint,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetPasswordHint",
    g_variant_new ("(s)",
                   arg_hint),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_automatic_login:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_enabled: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetAutomaticLogin">SetAutomaticLogin()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_call_set_automatic_login_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_call_set_automatic_login_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_user_call_set_automatic_login (
    OrgFreedesktopAccountsUser *proxy,
    gboolean arg_enabled,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetAutomaticLogin",
    g_variant_new ("(b)",
                   arg_enabled),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_user_call_set_automatic_login_finish:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_call_set_automatic_login().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_user_call_set_automatic_login().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_automatic_login_finish (
    OrgFreedesktopAccountsUser *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_call_set_automatic_login_sync:
 * @proxy: A #OrgFreedesktopAccountsUserProxy.
 * @arg_enabled: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetAutomaticLogin">SetAutomaticLogin()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_call_set_automatic_login() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_user_call_set_automatic_login_sync (
    OrgFreedesktopAccountsUser *proxy,
    gboolean arg_enabled,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetAutomaticLogin",
    g_variant_new ("(b)",
                   arg_enabled),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_user_complete_set_user_name:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetUserName">SetUserName()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_user_name (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_real_name:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetRealName">SetRealName()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_real_name (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_email:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetEmail">SetEmail()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_email (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_language:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetLanguage">SetLanguage()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_language (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_xsession:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetXSession">SetXSession()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_xsession (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_location:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetLocation">SetLocation()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_location (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_home_directory:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetHomeDirectory">SetHomeDirectory()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_home_directory (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_shell:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetShell">SetShell()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_shell (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_icon_file:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetIconFile">SetIconFile()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_icon_file (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_locked:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetLocked">SetLocked()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_locked (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_account_type:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetAccountType">SetAccountType()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_account_type (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_password_mode:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetPasswordMode">SetPasswordMode()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_password_mode (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_password:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetPassword">SetPassword()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_password (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_password_hint:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetPasswordHint">SetPasswordHint()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_password_hint (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_user_complete_set_automatic_login:
 * @object: A #OrgFreedesktopAccountsUser.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts-User.SetAutomaticLogin">SetAutomaticLogin()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_user_complete_set_automatic_login (
    OrgFreedesktopAccountsUser *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/* ------------------------------------------------------------------------ */

/**
 * OrgFreedesktopAccountsUserProxy:
 *
 * The #OrgFreedesktopAccountsUserProxy structure contains only private data and should only be accessed using the provided API.
 */

/**
 * OrgFreedesktopAccountsUserProxyClass:
 * @parent_class: The parent class.
 *
 * Class structure for #OrgFreedesktopAccountsUserProxy.
 */

struct _OrgFreedesktopAccountsUserProxyPrivate
{
  GData *qdata;
};

static void org_freedesktop_accounts_user_proxy_iface_init (OrgFreedesktopAccountsUserIface *iface);

#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (OrgFreedesktopAccountsUserProxy, org_freedesktop_accounts_user_proxy, G_TYPE_DBUS_PROXY,
                         G_ADD_PRIVATE (OrgFreedesktopAccountsUserProxy)
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER, org_freedesktop_accounts_user_proxy_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (OrgFreedesktopAccountsUserProxy, org_freedesktop_accounts_user_proxy, G_TYPE_DBUS_PROXY,
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER, org_freedesktop_accounts_user_proxy_iface_init))

#endif
static void
org_freedesktop_accounts_user_proxy_finalize (GObject *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  g_datalist_clear (&proxy->priv->qdata);
  G_OBJECT_CLASS (org_freedesktop_accounts_user_proxy_parent_class)->finalize (object);
}

static void
org_freedesktop_accounts_user_proxy_get_property (GObject      *object,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  const _ExtendedGDBusPropertyInfo *info;
  GVariant *variant;
  g_assert (prop_id != 0 && prop_id - 1 < 20);
  info = (const _ExtendedGDBusPropertyInfo *) _org_freedesktop_accounts_user_property_info_pointers[prop_id - 1];
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (object), info->parent_struct.name);
  if (info->use_gvariant)
    {
      g_value_set_variant (value, variant);
    }
  else
    {
      if (variant != NULL)
        g_dbus_gvariant_to_gvalue (variant, value);
    }
  if (variant != NULL)
    g_variant_unref (variant);
}

static void
org_freedesktop_accounts_user_proxy_set_property_cb (GDBusProxy *proxy,
  GAsyncResult *res,
  gpointer      user_data)
{
  const _ExtendedGDBusPropertyInfo *info = user_data;
  GError *error;
  GVariant *_ret;
  error = NULL;
  _ret = g_dbus_proxy_call_finish (proxy, res, &error);
  if (!_ret)
    {
      g_warning ("Error setting property '%s' on interface org.freedesktop.Accounts.User: %s (%s, %d)",
                 info->parent_struct.name, 
                 error->message, g_quark_to_string (error->domain), error->code);
      g_error_free (error);
    }
  else
    {
      g_variant_unref (_ret);
    }
}

static void
org_freedesktop_accounts_user_proxy_set_property (GObject      *object,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  const _ExtendedGDBusPropertyInfo *info;
  GVariant *variant;
  g_assert (prop_id != 0 && prop_id - 1 < 20);
  info = (const _ExtendedGDBusPropertyInfo *) _org_freedesktop_accounts_user_property_info_pointers[prop_id - 1];
  variant = g_dbus_gvalue_to_gvariant (value, G_VARIANT_TYPE (info->parent_struct.signature));
  g_dbus_proxy_call (G_DBUS_PROXY (object),
    "org.freedesktop.DBus.Properties.Set",
    g_variant_new ("(ssv)", "org.freedesktop.Accounts.User", info->parent_struct.name, variant),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    NULL, (GAsyncReadyCallback) org_freedesktop_accounts_user_proxy_set_property_cb, (GDBusPropertyInfo *) &info->parent_struct);
  g_variant_unref (variant);
}

static void
org_freedesktop_accounts_user_proxy_g_signal (GDBusProxy *proxy,
  const gchar *sender_name G_GNUC_UNUSED,
  const gchar *signal_name,
  GVariant *parameters)
{
  _ExtendedGDBusSignalInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  gsize n;
  guint signal_id;
  info = (_ExtendedGDBusSignalInfo *) g_dbus_interface_info_lookup_signal ((GDBusInterfaceInfo *) &_org_freedesktop_accounts_user_interface_info.parent_struct, signal_name);
  if (info == NULL)
    return;
  num_params = g_variant_n_children (parameters);
  paramv = g_new0 (GValue, num_params + 1);
  g_value_init (&paramv[0], TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER);
  g_value_set_object (&paramv[0], proxy);
  g_variant_iter_init (&iter, parameters);
  n = 1;
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.args[n - 1];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER);
  g_signal_emitv (paramv, signal_id, 0, NULL);
  for (n = 0; n < num_params + 1; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static void
org_freedesktop_accounts_user_proxy_g_properties_changed (GDBusProxy *_proxy,
  GVariant *changed_properties,
  const gchar *const *invalidated_properties)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (_proxy);
  guint n;
  const gchar *key;
  GVariantIter *iter;
  _ExtendedGDBusPropertyInfo *info;
  g_variant_get (changed_properties, "a{sv}", &iter);
  while (g_variant_iter_next (iter, "{&sv}", &key, NULL))
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_freedesktop_accounts_user_interface_info.parent_struct, key);
      g_datalist_remove_data (&proxy->priv->qdata, key);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
  g_variant_iter_free (iter);
  for (n = 0; invalidated_properties[n] != NULL; n++)
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_freedesktop_accounts_user_interface_info.parent_struct, invalidated_properties[n]);
      g_datalist_remove_data (&proxy->priv->qdata, invalidated_properties[n]);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
}

static guint64 
org_freedesktop_accounts_user_proxy_get_uid (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  guint64 value = 0;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "Uid");
  if (variant != NULL)
    {
      value = g_variant_get_uint64 (variant);
      g_variant_unref (variant);
    }
  return value;
}

static const gchar *
org_freedesktop_accounts_user_proxy_get_user_name (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "UserName");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static const gchar *
org_freedesktop_accounts_user_proxy_get_real_name (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "RealName");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static gint 
org_freedesktop_accounts_user_proxy_get_account_type (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  gint value = 0;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "AccountType");
  if (variant != NULL)
    {
      value = g_variant_get_int32 (variant);
      g_variant_unref (variant);
    }
  return value;
}

static const gchar *
org_freedesktop_accounts_user_proxy_get_home_directory (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "HomeDirectory");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static const gchar *
org_freedesktop_accounts_user_proxy_get_shell (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "Shell");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static const gchar *
org_freedesktop_accounts_user_proxy_get_email (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "Email");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static const gchar *
org_freedesktop_accounts_user_proxy_get_language (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "Language");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static const gchar *
org_freedesktop_accounts_user_proxy_get_xsession (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "XSession");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static const gchar *
org_freedesktop_accounts_user_proxy_get_location (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "Location");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static guint64 
org_freedesktop_accounts_user_proxy_get_login_frequency (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  guint64 value = 0;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "LoginFrequency");
  if (variant != NULL)
    {
      value = g_variant_get_uint64 (variant);
      g_variant_unref (variant);
    }
  return value;
}

static gint64 
org_freedesktop_accounts_user_proxy_get_login_time (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  gint64 value = 0;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "LoginTime");
  if (variant != NULL)
    {
      value = g_variant_get_int64 (variant);
      g_variant_unref (variant);
    }
  return value;
}

static GVariant *
org_freedesktop_accounts_user_proxy_get_login_history (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  GVariant *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "LoginHistory");
  value = variant;
  if (variant != NULL)
    g_variant_unref (variant);
  return value;
}

static const gchar *
org_freedesktop_accounts_user_proxy_get_icon_file (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "IconFile");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static gboolean 
org_freedesktop_accounts_user_proxy_get_locked (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  gboolean value = FALSE;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "Locked");
  if (variant != NULL)
    {
      value = g_variant_get_boolean (variant);
      g_variant_unref (variant);
    }
  return value;
}

static gint 
org_freedesktop_accounts_user_proxy_get_password_mode (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  gint value = 0;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "PasswordMode");
  if (variant != NULL)
    {
      value = g_variant_get_int32 (variant);
      g_variant_unref (variant);
    }
  return value;
}

static const gchar *
org_freedesktop_accounts_user_proxy_get_password_hint (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "PasswordHint");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static gboolean 
org_freedesktop_accounts_user_proxy_get_automatic_login (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  gboolean value = FALSE;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "AutomaticLogin");
  if (variant != NULL)
    {
      value = g_variant_get_boolean (variant);
      g_variant_unref (variant);
    }
  return value;
}

static gboolean 
org_freedesktop_accounts_user_proxy_get_system_account (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  gboolean value = FALSE;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "SystemAccount");
  if (variant != NULL)
    {
      value = g_variant_get_boolean (variant);
      g_variant_unref (variant);
    }
  return value;
}

static gboolean 
org_freedesktop_accounts_user_proxy_get_local_account (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY (object);
  GVariant *variant;
  gboolean value = FALSE;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "LocalAccount");
  if (variant != NULL)
    {
      value = g_variant_get_boolean (variant);
      g_variant_unref (variant);
    }
  return value;
}

static void
org_freedesktop_accounts_user_proxy_init (OrgFreedesktopAccountsUserProxy *proxy)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  proxy->priv = org_freedesktop_accounts_user_proxy_get_instance_private (proxy);
#else
  proxy->priv = G_TYPE_INSTANCE_GET_PRIVATE (proxy, TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY, OrgFreedesktopAccountsUserProxyPrivate);
#endif

  g_dbus_proxy_set_interface_info (G_DBUS_PROXY (proxy), org_freedesktop_accounts_user_interface_info ());
}

static void
org_freedesktop_accounts_user_proxy_class_init (OrgFreedesktopAccountsUserProxyClass *klass)
{
  GObjectClass *gobject_class;
  GDBusProxyClass *proxy_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize     = org_freedesktop_accounts_user_proxy_finalize;
  gobject_class->get_property = org_freedesktop_accounts_user_proxy_get_property;
  gobject_class->set_property = org_freedesktop_accounts_user_proxy_set_property;

  proxy_class = G_DBUS_PROXY_CLASS (klass);
  proxy_class->g_signal = org_freedesktop_accounts_user_proxy_g_signal;
  proxy_class->g_properties_changed = org_freedesktop_accounts_user_proxy_g_properties_changed;

  org_freedesktop_accounts_user_override_properties (gobject_class, 1);

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (OrgFreedesktopAccountsUserProxyPrivate));
#endif
}

static void
org_freedesktop_accounts_user_proxy_iface_init (OrgFreedesktopAccountsUserIface *iface)
{
  iface->get_uid = org_freedesktop_accounts_user_proxy_get_uid;
  iface->get_user_name = org_freedesktop_accounts_user_proxy_get_user_name;
  iface->get_real_name = org_freedesktop_accounts_user_proxy_get_real_name;
  iface->get_account_type = org_freedesktop_accounts_user_proxy_get_account_type;
  iface->get_home_directory = org_freedesktop_accounts_user_proxy_get_home_directory;
  iface->get_shell = org_freedesktop_accounts_user_proxy_get_shell;
  iface->get_email = org_freedesktop_accounts_user_proxy_get_email;
  iface->get_language = org_freedesktop_accounts_user_proxy_get_language;
  iface->get_xsession = org_freedesktop_accounts_user_proxy_get_xsession;
  iface->get_location = org_freedesktop_accounts_user_proxy_get_location;
  iface->get_login_frequency = org_freedesktop_accounts_user_proxy_get_login_frequency;
  iface->get_login_time = org_freedesktop_accounts_user_proxy_get_login_time;
  iface->get_login_history = org_freedesktop_accounts_user_proxy_get_login_history;
  iface->get_icon_file = org_freedesktop_accounts_user_proxy_get_icon_file;
  iface->get_locked = org_freedesktop_accounts_user_proxy_get_locked;
  iface->get_password_mode = org_freedesktop_accounts_user_proxy_get_password_mode;
  iface->get_password_hint = org_freedesktop_accounts_user_proxy_get_password_hint;
  iface->get_automatic_login = org_freedesktop_accounts_user_proxy_get_automatic_login;
  iface->get_system_account = org_freedesktop_accounts_user_proxy_get_system_account;
  iface->get_local_account = org_freedesktop_accounts_user_proxy_get_local_account;
}

/**
 * org_freedesktop_accounts_user_proxy_new:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-Accounts-User.top_of_page">org.freedesktop.Accounts.User</link>. See g_dbus_proxy_new() for more details.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_proxy_new_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_proxy_new_sync() for the synchronous, blocking version of this constructor.
 */
void
org_freedesktop_accounts_user_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.freedesktop.Accounts.User", NULL);
}

/**
 * org_freedesktop_accounts_user_proxy_new_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_proxy_new().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with org_freedesktop_accounts_user_proxy_new().
 *
 * Returns: (transfer full) (type OrgFreedesktopAccountsUserProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgFreedesktopAccountsUser *
org_freedesktop_accounts_user_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return ORG_FREEDESKTOP_ACCOUNTS_USER (ret);
  else
    return NULL;
}

/**
 * org_freedesktop_accounts_user_proxy_new_sync:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Synchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-Accounts-User.top_of_page">org.freedesktop.Accounts.User</link>. See g_dbus_proxy_new_sync() for more details.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_proxy_new() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type OrgFreedesktopAccountsUserProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgFreedesktopAccountsUser *
org_freedesktop_accounts_user_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.freedesktop.Accounts.User", NULL);
  if (ret != NULL)
    return ORG_FREEDESKTOP_ACCOUNTS_USER (ret);
  else
    return NULL;
}


/**
 * org_freedesktop_accounts_user_proxy_new_for_bus:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Like org_freedesktop_accounts_user_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_user_proxy_new_for_bus_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_user_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
 */
void
org_freedesktop_accounts_user_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.freedesktop.Accounts.User", NULL);
}

/**
 * org_freedesktop_accounts_user_proxy_new_for_bus_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_user_proxy_new_for_bus().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with org_freedesktop_accounts_user_proxy_new_for_bus().
 *
 * Returns: (transfer full) (type OrgFreedesktopAccountsUserProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgFreedesktopAccountsUser *
org_freedesktop_accounts_user_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return ORG_FREEDESKTOP_ACCOUNTS_USER (ret);
  else
    return NULL;
}

/**
 * org_freedesktop_accounts_user_proxy_new_for_bus_sync:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Like org_freedesktop_accounts_user_proxy_new_sync() but takes a #GBusType instead of a #GDBusConnection.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_user_proxy_new_for_bus() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type OrgFreedesktopAccountsUserProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgFreedesktopAccountsUser *
org_freedesktop_accounts_user_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.freedesktop.Accounts.User", NULL);
  if (ret != NULL)
    return ORG_FREEDESKTOP_ACCOUNTS_USER (ret);
  else
    return NULL;
}


/* ------------------------------------------------------------------------ */

/**
 * OrgFreedesktopAccountsUserSkeleton:
 *
 * The #OrgFreedesktopAccountsUserSkeleton structure contains only private data and should only be accessed using the provided API.
 */

/**
 * OrgFreedesktopAccountsUserSkeletonClass:
 * @parent_class: The parent class.
 *
 * Class structure for #OrgFreedesktopAccountsUserSkeleton.
 */

struct _OrgFreedesktopAccountsUserSkeletonPrivate
{
  GValue *properties;
  GList *changed_properties;
  GSource *changed_properties_idle_source;
  GMainContext *context;
  GMutex lock;
};

static void
_org_freedesktop_accounts_user_skeleton_handle_method_call (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name,
  const gchar *method_name,
  GVariant *parameters,
  GDBusMethodInvocation *invocation,
  gpointer user_data)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (user_data);
  _ExtendedGDBusMethodInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  guint num_extra;
  gsize n;
  guint signal_id;
  GValue return_value = G_VALUE_INIT;
  info = (_ExtendedGDBusMethodInfo *) g_dbus_method_invocation_get_method_info (invocation);
  g_assert (info != NULL);
  num_params = g_variant_n_children (parameters);
  num_extra = info->pass_fdlist ? 3 : 2;  paramv = g_new0 (GValue, num_params + num_extra);
  n = 0;
  g_value_init (&paramv[n], TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER);
  g_value_set_object (&paramv[n++], skeleton);
  g_value_init (&paramv[n], G_TYPE_DBUS_METHOD_INVOCATION);
  g_value_set_object (&paramv[n++], invocation);
  if (info->pass_fdlist)
    {
#ifdef G_OS_UNIX
      g_value_init (&paramv[n], G_TYPE_UNIX_FD_LIST);
      g_value_set_object (&paramv[n++], g_dbus_message_get_unix_fd_list (g_dbus_method_invocation_get_message (invocation)));
#else
      g_assert_not_reached ();
#endif
    }
  g_variant_iter_init (&iter, parameters);
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.in_args[n - num_extra];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER);
  g_value_init (&return_value, G_TYPE_BOOLEAN);
  g_signal_emitv (paramv, signal_id, 0, &return_value);
  if (!g_value_get_boolean (&return_value))
    g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD, "Method %s is not implemented on interface %s", method_name, interface_name);
  g_value_unset (&return_value);
  for (n = 0; n < num_params + num_extra; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static GVariant *
_org_freedesktop_accounts_user_skeleton_handle_get_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GError **error,
  gpointer user_data)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  GVariant *ret;
  ret = NULL;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_freedesktop_accounts_user_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      g_value_init (&value, pspec->value_type);
      g_object_get_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      ret = g_dbus_gvalue_to_gvariant (&value, G_VARIANT_TYPE (info->parent_struct.signature));
      g_value_unset (&value);
    }
  return ret;
}

static gboolean
_org_freedesktop_accounts_user_skeleton_handle_set_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GVariant *variant,
  GError **error,
  gpointer user_data)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  gboolean ret;
  ret = FALSE;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_freedesktop_accounts_user_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      if (info->use_gvariant)
        g_value_set_variant (&value, variant);
      else
        g_dbus_gvariant_to_gvalue (variant, &value);
      g_object_set_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      g_value_unset (&value);
      ret = TRUE;
    }
  return ret;
}

static const GDBusInterfaceVTable _org_freedesktop_accounts_user_skeleton_vtable =
{
  _org_freedesktop_accounts_user_skeleton_handle_method_call,
  _org_freedesktop_accounts_user_skeleton_handle_get_property,
  _org_freedesktop_accounts_user_skeleton_handle_set_property,
  {NULL}
};

static GDBusInterfaceInfo *
org_freedesktop_accounts_user_skeleton_dbus_interface_get_info (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return org_freedesktop_accounts_user_interface_info ();
}

static GDBusInterfaceVTable *
org_freedesktop_accounts_user_skeleton_dbus_interface_get_vtable (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return (GDBusInterfaceVTable *) &_org_freedesktop_accounts_user_skeleton_vtable;
}

static GVariant *
org_freedesktop_accounts_user_skeleton_dbus_interface_get_properties (GDBusInterfaceSkeleton *_skeleton)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (_skeleton);

  GVariantBuilder builder;
  guint n;
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
#else
  g_variant_builder_init(&builder, G_VARIANT_TYPE ("a{sv}"));
#endif
  if (_org_freedesktop_accounts_user_interface_info.parent_struct.properties == NULL)
    goto out;
  for (n = 0; _org_freedesktop_accounts_user_interface_info.parent_struct.properties[n] != NULL; n++)
    {
      GDBusPropertyInfo *info = _org_freedesktop_accounts_user_interface_info.parent_struct.properties[n];
      if (info->flags & G_DBUS_PROPERTY_INFO_FLAGS_READABLE)
        {
          GVariant *value;
          value = _org_freedesktop_accounts_user_skeleton_handle_get_property (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)), NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.freedesktop.Accounts.User", info->name, NULL, skeleton);
          if (value != NULL)
            {
              g_variant_take_ref (value);
              g_variant_builder_add (&builder, "{sv}", info->name, value);
              g_variant_unref (value);
            }
        }
    }
out:
  return g_variant_builder_end (&builder);
}

static gboolean _org_freedesktop_accounts_user_emit_changed (gpointer user_data);

static void
org_freedesktop_accounts_user_skeleton_dbus_interface_flush (GDBusInterfaceSkeleton *_skeleton)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (_skeleton);
  gboolean emit_changed = FALSE;

  g_mutex_lock (&skeleton->priv->lock);
  if (skeleton->priv->changed_properties_idle_source != NULL)
    {
      g_source_destroy (skeleton->priv->changed_properties_idle_source);
      skeleton->priv->changed_properties_idle_source = NULL;
      emit_changed = TRUE;
    }
  g_mutex_unlock (&skeleton->priv->lock);

  if (emit_changed)
    _org_freedesktop_accounts_user_emit_changed (skeleton);
}

static void
_org_freedesktop_accounts_user_on_signal_changed (
    OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("()"));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.freedesktop.Accounts.User", "Changed",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void org_freedesktop_accounts_user_skeleton_iface_init (OrgFreedesktopAccountsUserIface *iface);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (OrgFreedesktopAccountsUserSkeleton, org_freedesktop_accounts_user_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_ADD_PRIVATE (OrgFreedesktopAccountsUserSkeleton)
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER, org_freedesktop_accounts_user_skeleton_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (OrgFreedesktopAccountsUserSkeleton, org_freedesktop_accounts_user_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER, org_freedesktop_accounts_user_skeleton_iface_init))

#endif
static void
org_freedesktop_accounts_user_skeleton_finalize (GObject *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  guint n;
  for (n = 0; n < 20; n++)
    g_value_unset (&skeleton->priv->properties[n]);
  g_free (skeleton->priv->properties);
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
  if (skeleton->priv->changed_properties_idle_source != NULL)
    g_source_destroy (skeleton->priv->changed_properties_idle_source);
  g_main_context_unref (skeleton->priv->context);
  g_mutex_clear (&skeleton->priv->lock);
  G_OBJECT_CLASS (org_freedesktop_accounts_user_skeleton_parent_class)->finalize (object);
}

static void
org_freedesktop_accounts_user_skeleton_get_property (GObject      *object,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  g_assert (prop_id != 0 && prop_id - 1 < 20);
  g_mutex_lock (&skeleton->priv->lock);
  g_value_copy (&skeleton->priv->properties[prop_id - 1], value);
  g_mutex_unlock (&skeleton->priv->lock);
}

static gboolean
_org_freedesktop_accounts_user_emit_changed (gpointer user_data)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (user_data);
  GList *l;
  GVariantBuilder builder;
  GVariantBuilder invalidated_builder;
  guint num_changes;

  g_mutex_lock (&skeleton->priv->lock);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
  g_variant_builder_init_static (&invalidated_builder, G_VARIANT_TYPE ("as"));
#else
  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
  g_variant_builder_init (&invalidated_builder, G_VARIANT_TYPE ("as"));
#endif
  for (l = skeleton->priv->changed_properties, num_changes = 0; l != NULL; l = l->next)
    {
      ChangedProperty *cp = l->data;
      GVariant *variant;
      const GValue *cur_value;

      cur_value = &skeleton->priv->properties[cp->prop_id - 1];
      if (!_g_value_equal (cur_value, &cp->orig_value))
        {
          variant = g_dbus_gvalue_to_gvariant (cur_value, G_VARIANT_TYPE (cp->info->parent_struct.signature));
          g_variant_builder_add (&builder, "{sv}", cp->info->parent_struct.name, variant);
          g_variant_unref (variant);
          num_changes++;
        }
    }
  if (num_changes > 0)
    {
      GList *connections, *ll;
      GVariant *signal_variant;
      signal_variant = g_variant_ref_sink (g_variant_new ("(sa{sv}as)", "org.freedesktop.Accounts.User",
                                           &builder, &invalidated_builder));
      connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));
      for (ll = connections; ll != NULL; ll = ll->next)
        {
          GDBusConnection *connection = ll->data;

          g_dbus_connection_emit_signal (connection,
                                         NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)),
                                         "org.freedesktop.DBus.Properties",
                                         "PropertiesChanged",
                                         signal_variant,
                                         NULL);
        }
      g_variant_unref (signal_variant);
      g_list_free_full (connections, g_object_unref);
    }
  else
    {
      g_variant_builder_clear (&builder);
      g_variant_builder_clear (&invalidated_builder);
    }
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
  skeleton->priv->changed_properties = NULL;
  skeleton->priv->changed_properties_idle_source = NULL;
  g_mutex_unlock (&skeleton->priv->lock);
  return FALSE;
}

static void
_org_freedesktop_accounts_user_schedule_emit_changed (OrgFreedesktopAccountsUserSkeleton *skeleton, const _ExtendedGDBusPropertyInfo *info, guint prop_id, const GValue *orig_value)
{
  ChangedProperty *cp;
  GList *l;
  cp = NULL;
  for (l = skeleton->priv->changed_properties; l != NULL; l = l->next)
    {
      ChangedProperty *i_cp = l->data;
      if (i_cp->info == info)
        {
          cp = i_cp;
          break;
        }
    }
  if (cp == NULL)
    {
      cp = g_new0 (ChangedProperty, 1);
      cp->prop_id = prop_id;
      cp->info = info;
      skeleton->priv->changed_properties = g_list_prepend (skeleton->priv->changed_properties, cp);
      g_value_init (&cp->orig_value, G_VALUE_TYPE (orig_value));
      g_value_copy (orig_value, &cp->orig_value);
    }
}

static void
org_freedesktop_accounts_user_skeleton_notify (GObject      *object,
  GParamSpec *pspec G_GNUC_UNUSED)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  g_mutex_lock (&skeleton->priv->lock);
  if (skeleton->priv->changed_properties != NULL &&
      skeleton->priv->changed_properties_idle_source == NULL)
    {
      skeleton->priv->changed_properties_idle_source = g_idle_source_new ();
      g_source_set_priority (skeleton->priv->changed_properties_idle_source, G_PRIORITY_DEFAULT);
      g_source_set_callback (skeleton->priv->changed_properties_idle_source, _org_freedesktop_accounts_user_emit_changed, g_object_ref (skeleton), (GDestroyNotify) g_object_unref);
      g_source_set_name (skeleton->priv->changed_properties_idle_source, "[generated] _org_freedesktop_accounts_user_emit_changed");
      g_source_attach (skeleton->priv->changed_properties_idle_source, skeleton->priv->context);
      g_source_unref (skeleton->priv->changed_properties_idle_source);
    }
  g_mutex_unlock (&skeleton->priv->lock);
}

static void
org_freedesktop_accounts_user_skeleton_set_property (GObject      *object,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec)
{
  const _ExtendedGDBusPropertyInfo *info;
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  g_assert (prop_id != 0 && prop_id - 1 < 20);
  info = (const _ExtendedGDBusPropertyInfo *) _org_freedesktop_accounts_user_property_info_pointers[prop_id - 1];
  g_mutex_lock (&skeleton->priv->lock);
  g_object_freeze_notify (object);
  if (!_g_value_equal (value, &skeleton->priv->properties[prop_id - 1]))
    {
      if (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)) != NULL &&
          info->emits_changed_signal)
        _org_freedesktop_accounts_user_schedule_emit_changed (skeleton, info, prop_id, &skeleton->priv->properties[prop_id - 1]);
      g_value_copy (value, &skeleton->priv->properties[prop_id - 1]);
      g_object_notify_by_pspec (object, pspec);
    }
  g_mutex_unlock (&skeleton->priv->lock);
  g_object_thaw_notify (object);
}

static void
org_freedesktop_accounts_user_skeleton_init (OrgFreedesktopAccountsUserSkeleton *skeleton)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  skeleton->priv = org_freedesktop_accounts_user_skeleton_get_instance_private (skeleton);
#else
  skeleton->priv = G_TYPE_INSTANCE_GET_PRIVATE (skeleton, TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON, OrgFreedesktopAccountsUserSkeletonPrivate);
#endif

  g_mutex_init (&skeleton->priv->lock);
  skeleton->priv->context = g_main_context_ref_thread_default ();
  skeleton->priv->properties = g_new0 (GValue, 20);
  g_value_init (&skeleton->priv->properties[0], G_TYPE_UINT64);
  g_value_init (&skeleton->priv->properties[1], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[2], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[3], G_TYPE_INT);
  g_value_init (&skeleton->priv->properties[4], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[5], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[6], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[7], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[8], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[9], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[10], G_TYPE_UINT64);
  g_value_init (&skeleton->priv->properties[11], G_TYPE_INT64);
  g_value_init (&skeleton->priv->properties[12], G_TYPE_VARIANT);
  g_value_init (&skeleton->priv->properties[13], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[14], G_TYPE_BOOLEAN);
  g_value_init (&skeleton->priv->properties[15], G_TYPE_INT);
  g_value_init (&skeleton->priv->properties[16], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[17], G_TYPE_BOOLEAN);
  g_value_init (&skeleton->priv->properties[18], G_TYPE_BOOLEAN);
  g_value_init (&skeleton->priv->properties[19], G_TYPE_BOOLEAN);
}

static guint64 
org_freedesktop_accounts_user_skeleton_get_uid (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  guint64 value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_uint64 (&(skeleton->priv->properties[0]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static const gchar *
org_freedesktop_accounts_user_skeleton_get_user_name (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[1]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static const gchar *
org_freedesktop_accounts_user_skeleton_get_real_name (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[2]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static gint 
org_freedesktop_accounts_user_skeleton_get_account_type (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  gint value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_int (&(skeleton->priv->properties[3]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static const gchar *
org_freedesktop_accounts_user_skeleton_get_home_directory (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[4]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static const gchar *
org_freedesktop_accounts_user_skeleton_get_shell (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[5]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static const gchar *
org_freedesktop_accounts_user_skeleton_get_email (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[6]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static const gchar *
org_freedesktop_accounts_user_skeleton_get_language (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[7]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static const gchar *
org_freedesktop_accounts_user_skeleton_get_xsession (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[8]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static const gchar *
org_freedesktop_accounts_user_skeleton_get_location (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[9]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static guint64 
org_freedesktop_accounts_user_skeleton_get_login_frequency (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  guint64 value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_uint64 (&(skeleton->priv->properties[10]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static gint64 
org_freedesktop_accounts_user_skeleton_get_login_time (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  gint64 value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_int64 (&(skeleton->priv->properties[11]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static GVariant *
org_freedesktop_accounts_user_skeleton_get_login_history (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  GVariant *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_variant (&(skeleton->priv->properties[12]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static const gchar *
org_freedesktop_accounts_user_skeleton_get_icon_file (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[13]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static gboolean 
org_freedesktop_accounts_user_skeleton_get_locked (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  gboolean value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_boolean (&(skeleton->priv->properties[14]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static gint 
org_freedesktop_accounts_user_skeleton_get_password_mode (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  gint value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_int (&(skeleton->priv->properties[15]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static const gchar *
org_freedesktop_accounts_user_skeleton_get_password_hint (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[16]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static gboolean 
org_freedesktop_accounts_user_skeleton_get_automatic_login (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  gboolean value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_boolean (&(skeleton->priv->properties[17]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static gboolean 
org_freedesktop_accounts_user_skeleton_get_system_account (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  gboolean value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_boolean (&(skeleton->priv->properties[18]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static gboolean 
org_freedesktop_accounts_user_skeleton_get_local_account (OrgFreedesktopAccountsUser *object)
{
  OrgFreedesktopAccountsUserSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON (object);
  gboolean value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_boolean (&(skeleton->priv->properties[19]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static void
org_freedesktop_accounts_user_skeleton_class_init (OrgFreedesktopAccountsUserSkeletonClass *klass)
{
  GObjectClass *gobject_class;
  GDBusInterfaceSkeletonClass *skeleton_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize = org_freedesktop_accounts_user_skeleton_finalize;
  gobject_class->get_property = org_freedesktop_accounts_user_skeleton_get_property;
  gobject_class->set_property = org_freedesktop_accounts_user_skeleton_set_property;
  gobject_class->notify       = org_freedesktop_accounts_user_skeleton_notify;


  org_freedesktop_accounts_user_override_properties (gobject_class, 1);

  skeleton_class = G_DBUS_INTERFACE_SKELETON_CLASS (klass);
  skeleton_class->get_info = org_freedesktop_accounts_user_skeleton_dbus_interface_get_info;
  skeleton_class->get_properties = org_freedesktop_accounts_user_skeleton_dbus_interface_get_properties;
  skeleton_class->flush = org_freedesktop_accounts_user_skeleton_dbus_interface_flush;
  skeleton_class->get_vtable = org_freedesktop_accounts_user_skeleton_dbus_interface_get_vtable;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (OrgFreedesktopAccountsUserSkeletonPrivate));
#endif
}

static void
org_freedesktop_accounts_user_skeleton_iface_init (OrgFreedesktopAccountsUserIface *iface)
{
  iface->changed = _org_freedesktop_accounts_user_on_signal_changed;
  iface->get_uid = org_freedesktop_accounts_user_skeleton_get_uid;
  iface->get_user_name = org_freedesktop_accounts_user_skeleton_get_user_name;
  iface->get_real_name = org_freedesktop_accounts_user_skeleton_get_real_name;
  iface->get_account_type = org_freedesktop_accounts_user_skeleton_get_account_type;
  iface->get_home_directory = org_freedesktop_accounts_user_skeleton_get_home_directory;
  iface->get_shell = org_freedesktop_accounts_user_skeleton_get_shell;
  iface->get_email = org_freedesktop_accounts_user_skeleton_get_email;
  iface->get_language = org_freedesktop_accounts_user_skeleton_get_language;
  iface->get_xsession = org_freedesktop_accounts_user_skeleton_get_xsession;
  iface->get_location = org_freedesktop_accounts_user_skeleton_get_location;
  iface->get_login_frequency = org_freedesktop_accounts_user_skeleton_get_login_frequency;
  iface->get_login_time = org_freedesktop_accounts_user_skeleton_get_login_time;
  iface->get_login_history = org_freedesktop_accounts_user_skeleton_get_login_history;
  iface->get_icon_file = org_freedesktop_accounts_user_skeleton_get_icon_file;
  iface->get_locked = org_freedesktop_accounts_user_skeleton_get_locked;
  iface->get_password_mode = org_freedesktop_accounts_user_skeleton_get_password_mode;
  iface->get_password_hint = org_freedesktop_accounts_user_skeleton_get_password_hint;
  iface->get_automatic_login = org_freedesktop_accounts_user_skeleton_get_automatic_login;
  iface->get_system_account = org_freedesktop_accounts_user_skeleton_get_system_account;
  iface->get_local_account = org_freedesktop_accounts_user_skeleton_get_local_account;
}

/**
 * org_freedesktop_accounts_user_skeleton_new:
 *
 * Creates a skeleton object for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-Accounts-User.top_of_page">org.freedesktop.Accounts.User</link>.
 *
 * Returns: (transfer full) (type OrgFreedesktopAccountsUserSkeleton): The skeleton object.
 */
OrgFreedesktopAccountsUser *
org_freedesktop_accounts_user_skeleton_new (void)
{
  return ORG_FREEDESKTOP_ACCOUNTS_USER (g_object_new (TYPE_ORG_FREEDESKTOP_ACCOUNTS_USER_SKELETON, NULL));
}

/* ------------------------------------------------------------------------
 * Code for interface org.freedesktop.Accounts
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:OrgFreedesktopAccounts
 * @title: OrgFreedesktopAccounts
 * @short_description: Generated C code for the org.freedesktop.Accounts D-Bus interface
 *
 * This section contains code for working with the <link linkend="gdbus-interface-org-freedesktop-Accounts.top_of_page">org.freedesktop.Accounts</link> D-Bus interface in C.
 */

enum
{
  _ORG_FREEDESKTOP_ACCOUNTS_USER_ADDED,
  _ORG_FREEDESKTOP_ACCOUNTS_USER_DELETED,
};

static unsigned _ORG_FREEDESKTOP_ACCOUNTS_SIGNALS[2] = { 0 };

/* ---- Introspection data for org.freedesktop.Accounts ---- */

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_list_cached_users_OUT_ARG_users =
{
  {
    -1,
    (gchar *) "users",
    (gchar *) "ao",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_method_info_list_cached_users_OUT_ARG_pointers[] =
{
  &_org_freedesktop_accounts_method_info_list_cached_users_OUT_ARG_users.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_method_list_cached_users_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_method_list_cached_users_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_method_list_cached_users_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_method_info_list_cached_users =
{
  {
    -1,
    (gchar *) "ListCachedUsers",
    NULL,
    (GDBusArgInfo **) &_org_freedesktop_accounts_method_info_list_cached_users_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_method_list_cached_users_annotation_info_pointers
  },
  "handle-list-cached-users",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_find_user_by_id_IN_ARG_id =
{
  {
    -1,
    (gchar *) "id",
    (gchar *) "x",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_method_info_find_user_by_id_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_method_info_find_user_by_id_IN_ARG_id.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_find_user_by_id_OUT_ARG_user =
{
  {
    -1,
    (gchar *) "user",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_method_info_find_user_by_id_OUT_ARG_pointers[] =
{
  &_org_freedesktop_accounts_method_info_find_user_by_id_OUT_ARG_user.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_method_find_user_by_id_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_method_find_user_by_id_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_method_find_user_by_id_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_method_info_find_user_by_id =
{
  {
    -1,
    (gchar *) "FindUserById",
    (GDBusArgInfo **) &_org_freedesktop_accounts_method_info_find_user_by_id_IN_ARG_pointers,
    (GDBusArgInfo **) &_org_freedesktop_accounts_method_info_find_user_by_id_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_method_find_user_by_id_annotation_info_pointers
  },
  "handle-find-user-by-id",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_find_user_by_name_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_method_info_find_user_by_name_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_method_info_find_user_by_name_IN_ARG_name.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_find_user_by_name_OUT_ARG_user =
{
  {
    -1,
    (gchar *) "user",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_method_info_find_user_by_name_OUT_ARG_pointers[] =
{
  &_org_freedesktop_accounts_method_info_find_user_by_name_OUT_ARG_user.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_method_find_user_by_name_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_method_find_user_by_name_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_method_find_user_by_name_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_method_info_find_user_by_name =
{
  {
    -1,
    (gchar *) "FindUserByName",
    (GDBusArgInfo **) &_org_freedesktop_accounts_method_info_find_user_by_name_IN_ARG_pointers,
    (GDBusArgInfo **) &_org_freedesktop_accounts_method_info_find_user_by_name_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_method_find_user_by_name_annotation_info_pointers
  },
  "handle-find-user-by-name",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_create_user_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_create_user_IN_ARG_fullname =
{
  {
    -1,
    (gchar *) "fullname",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_create_user_IN_ARG_accountType =
{
  {
    -1,
    (gchar *) "accountType",
    (gchar *) "i",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_method_info_create_user_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_method_info_create_user_IN_ARG_name.parent_struct,
  &_org_freedesktop_accounts_method_info_create_user_IN_ARG_fullname.parent_struct,
  &_org_freedesktop_accounts_method_info_create_user_IN_ARG_accountType.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_create_user_OUT_ARG_user =
{
  {
    -1,
    (gchar *) "user",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_method_info_create_user_OUT_ARG_pointers[] =
{
  &_org_freedesktop_accounts_method_info_create_user_OUT_ARG_user.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_method_create_user_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_method_create_user_annotation_info_1 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_method_create_user_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_method_create_user_annotation_info_0,
  &_org_freedesktop_accounts_method_create_user_annotation_info_1,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_method_info_create_user =
{
  {
    -1,
    (gchar *) "CreateUser",
    (GDBusArgInfo **) &_org_freedesktop_accounts_method_info_create_user_IN_ARG_pointers,
    (GDBusArgInfo **) &_org_freedesktop_accounts_method_info_create_user_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_method_create_user_annotation_info_pointers
  },
  "handle-create-user",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_cache_user_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_method_info_cache_user_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_method_info_cache_user_IN_ARG_name.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_cache_user_OUT_ARG_user =
{
  {
    -1,
    (gchar *) "user",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_method_info_cache_user_OUT_ARG_pointers[] =
{
  &_org_freedesktop_accounts_method_info_cache_user_OUT_ARG_user.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_method_cache_user_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_method_cache_user_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_method_cache_user_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_method_info_cache_user =
{
  {
    -1,
    (gchar *) "CacheUser",
    (GDBusArgInfo **) &_org_freedesktop_accounts_method_info_cache_user_IN_ARG_pointers,
    (GDBusArgInfo **) &_org_freedesktop_accounts_method_info_cache_user_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_method_cache_user_annotation_info_pointers
  },
  "handle-cache-user",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_uncache_user_IN_ARG_name =
{
  {
    -1,
    (gchar *) "name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_method_info_uncache_user_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_method_info_uncache_user_IN_ARG_name.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_method_uncache_user_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_method_uncache_user_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_method_uncache_user_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_method_info_uncache_user =
{
  {
    -1,
    (gchar *) "UncacheUser",
    (GDBusArgInfo **) &_org_freedesktop_accounts_method_info_uncache_user_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_method_uncache_user_annotation_info_pointers
  },
  "handle-uncache-user",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_delete_user_IN_ARG_id =
{
  {
    -1,
    (gchar *) "id",
    (gchar *) "x",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_method_info_delete_user_IN_ARG_removeFiles =
{
  {
    -1,
    (gchar *) "removeFiles",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_method_info_delete_user_IN_ARG_pointers[] =
{
  &_org_freedesktop_accounts_method_info_delete_user_IN_ARG_id.parent_struct,
  &_org_freedesktop_accounts_method_info_delete_user_IN_ARG_removeFiles.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_freedesktop_accounts_method_delete_user_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_freedesktop_accounts_method_delete_user_annotation_info_pointers[] =
{
  &_org_freedesktop_accounts_method_delete_user_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_accounts_method_info_delete_user =
{
  {
    -1,
    (gchar *) "DeleteUser",
    (GDBusArgInfo **) &_org_freedesktop_accounts_method_info_delete_user_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_freedesktop_accounts_method_delete_user_annotation_info_pointers
  },
  "handle-delete-user",
  FALSE
};

static const GDBusMethodInfo * const _org_freedesktop_accounts_method_info_pointers[] =
{
  &_org_freedesktop_accounts_method_info_list_cached_users.parent_struct,
  &_org_freedesktop_accounts_method_info_find_user_by_id.parent_struct,
  &_org_freedesktop_accounts_method_info_find_user_by_name.parent_struct,
  &_org_freedesktop_accounts_method_info_create_user.parent_struct,
  &_org_freedesktop_accounts_method_info_cache_user.parent_struct,
  &_org_freedesktop_accounts_method_info_uncache_user.parent_struct,
  &_org_freedesktop_accounts_method_info_delete_user.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_signal_info_user_added_ARG_user =
{
  {
    -1,
    (gchar *) "user",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_signal_info_user_added_ARG_pointers[] =
{
  &_org_freedesktop_accounts_signal_info_user_added_ARG_user.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _org_freedesktop_accounts_signal_info_user_added =
{
  {
    -1,
    (gchar *) "UserAdded",
    (GDBusArgInfo **) &_org_freedesktop_accounts_signal_info_user_added_ARG_pointers,
    NULL
  },
  "user-added"
};

static const _ExtendedGDBusArgInfo _org_freedesktop_accounts_signal_info_user_deleted_ARG_user =
{
  {
    -1,
    (gchar *) "user",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_accounts_signal_info_user_deleted_ARG_pointers[] =
{
  &_org_freedesktop_accounts_signal_info_user_deleted_ARG_user.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _org_freedesktop_accounts_signal_info_user_deleted =
{
  {
    -1,
    (gchar *) "UserDeleted",
    (GDBusArgInfo **) &_org_freedesktop_accounts_signal_info_user_deleted_ARG_pointers,
    NULL
  },
  "user-deleted"
};

static const GDBusSignalInfo * const _org_freedesktop_accounts_signal_info_pointers[] =
{
  &_org_freedesktop_accounts_signal_info_user_added.parent_struct,
  &_org_freedesktop_accounts_signal_info_user_deleted.parent_struct,
  NULL
};

static const _ExtendedGDBusPropertyInfo _org_freedesktop_accounts_property_info_daemon_version =
{
  {
    -1,
    (gchar *) "DaemonVersion",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "daemon-version",
  FALSE,
  TRUE
};

static const GDBusPropertyInfo * const _org_freedesktop_accounts_property_info_pointers[] =
{
  &_org_freedesktop_accounts_property_info_daemon_version.parent_struct,
  NULL
};

static const _ExtendedGDBusInterfaceInfo _org_freedesktop_accounts_interface_info =
{
  {
    -1,
    (gchar *) "org.freedesktop.Accounts",
    (GDBusMethodInfo **) &_org_freedesktop_accounts_method_info_pointers,
    (GDBusSignalInfo **) &_org_freedesktop_accounts_signal_info_pointers,
    (GDBusPropertyInfo **) &_org_freedesktop_accounts_property_info_pointers,
    NULL
  },
  "org-freedesktop-accounts",
};


/**
 * org_freedesktop_accounts_interface_info:
 *
 * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-Accounts.top_of_page">org.freedesktop.Accounts</link> D-Bus interface.
 *
 * Returns: (transfer none): A #GDBusInterfaceInfo. Do not free.
 */
GDBusInterfaceInfo *
org_freedesktop_accounts_interface_info (void)
{
  return (GDBusInterfaceInfo *) &_org_freedesktop_accounts_interface_info.parent_struct;
}

/**
 * org_freedesktop_accounts_override_properties:
 * @klass: The class structure for a #GObject derived class.
 * @property_id_begin: The property id to assign to the first overridden property.
 *
 * Overrides all #GObject properties in the #OrgFreedesktopAccounts interface for a concrete class.
 * The properties are overridden in the order they are defined.
 *
 * Returns: The last property id.
 */
guint
org_freedesktop_accounts_override_properties (GObjectClass *klass, guint property_id_begin)
{
  g_object_class_override_property (klass, property_id_begin++, "daemon-version");
  return property_id_begin - 1;
}


inline static void
org_freedesktop_accounts_signal_marshal_user_added (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_signal_marshal_user_deleted (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_method_marshal_list_cached_users (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_method_marshal_find_user_by_id (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT64 (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_method_marshal_find_user_by_name (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_method_marshal_create_user (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING_INT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_method_marshal_cache_user (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_method_marshal_uncache_user (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_accounts_method_marshal_delete_user (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT64_BOOLEAN (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}


/**
 * OrgFreedesktopAccounts:
 *
 * Abstract interface type for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-Accounts.top_of_page">org.freedesktop.Accounts</link>.
 */

/**
 * OrgFreedesktopAccountsIface:
 * @parent_iface: The parent interface.
 * @handle_cache_user: Handler for the #OrgFreedesktopAccounts::handle-cache-user signal.
 * @handle_create_user: Handler for the #OrgFreedesktopAccounts::handle-create-user signal.
 * @handle_delete_user: Handler for the #OrgFreedesktopAccounts::handle-delete-user signal.
 * @handle_find_user_by_id: Handler for the #OrgFreedesktopAccounts::handle-find-user-by-id signal.
 * @handle_find_user_by_name: Handler for the #OrgFreedesktopAccounts::handle-find-user-by-name signal.
 * @handle_list_cached_users: Handler for the #OrgFreedesktopAccounts::handle-list-cached-users signal.
 * @handle_uncache_user: Handler for the #OrgFreedesktopAccounts::handle-uncache-user signal.
 * @get_daemon_version: Getter for the #OrgFreedesktopAccounts:daemon-version property.
 * @user_added: Handler for the #OrgFreedesktopAccounts::user-added signal.
 * @user_deleted: Handler for the #OrgFreedesktopAccounts::user-deleted signal.
 *
 * Virtual table for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-Accounts.top_of_page">org.freedesktop.Accounts</link>.
 */

typedef OrgFreedesktopAccountsIface OrgFreedesktopAccountsInterface;
G_DEFINE_INTERFACE (OrgFreedesktopAccounts, org_freedesktop_accounts, G_TYPE_OBJECT)

static void
org_freedesktop_accounts_default_init (OrgFreedesktopAccountsIface *iface)
{
  /* GObject signals for incoming D-Bus method calls: */
  /**
   * OrgFreedesktopAccounts::handle-list-cached-users:
   * @object: A #OrgFreedesktopAccounts.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts.ListCachedUsers">ListCachedUsers()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_complete_list_cached_users() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-list-cached-users",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsIface, handle_list_cached_users),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_method_marshal_list_cached_users,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * OrgFreedesktopAccounts::handle-find-user-by-id:
   * @object: A #OrgFreedesktopAccounts.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_id: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts.FindUserById">FindUserById()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_complete_find_user_by_id() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-find-user-by-id",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsIface, handle_find_user_by_id),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_method_marshal_find_user_by_id,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_INT64);

  /**
   * OrgFreedesktopAccounts::handle-find-user-by-name:
   * @object: A #OrgFreedesktopAccounts.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts.FindUserByName">FindUserByName()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_complete_find_user_by_name() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-find-user-by-name",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsIface, handle_find_user_by_name),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_method_marshal_find_user_by_name,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccounts::handle-create-user:
   * @object: A #OrgFreedesktopAccounts.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   * @arg_fullname: Argument passed by remote caller.
   * @arg_accountType: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts.CreateUser">CreateUser()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_complete_create_user() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-create-user",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsIface, handle_create_user),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_method_marshal_create_user,
    G_TYPE_BOOLEAN,
    4,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);

  /**
   * OrgFreedesktopAccounts::handle-cache-user:
   * @object: A #OrgFreedesktopAccounts.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts.CacheUser">CacheUser()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_complete_cache_user() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-cache-user",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsIface, handle_cache_user),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_method_marshal_cache_user,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccounts::handle-uncache-user:
   * @object: A #OrgFreedesktopAccounts.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_name: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts.UncacheUser">UncacheUser()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_complete_uncache_user() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-uncache-user",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsIface, handle_uncache_user),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_method_marshal_uncache_user,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccounts::handle-delete-user:
   * @object: A #OrgFreedesktopAccounts.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_id: Argument passed by remote caller.
   * @arg_removeFiles: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-Accounts.DeleteUser">DeleteUser()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_accounts_complete_delete_user() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-delete-user",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopAccountsIface, handle_delete_user),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_accounts_method_marshal_delete_user,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_INT64, G_TYPE_BOOLEAN);

  /* GObject signals for received D-Bus signals: */
  /**
   * OrgFreedesktopAccounts::user-added:
   * @object: A #OrgFreedesktopAccounts.
   * @arg_user: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-org-freedesktop-Accounts.UserAdded">"UserAdded"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _ORG_FREEDESKTOP_ACCOUNTS_SIGNALS[_ORG_FREEDESKTOP_ACCOUNTS_USER_ADDED] =
    g_signal_new ("user-added",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (OrgFreedesktopAccountsIface, user_added),
      NULL,
      NULL,
      org_freedesktop_accounts_signal_marshal_user_added,
      G_TYPE_NONE,
      1, G_TYPE_STRING);

  /**
   * OrgFreedesktopAccounts::user-deleted:
   * @object: A #OrgFreedesktopAccounts.
   * @arg_user: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-org-freedesktop-Accounts.UserDeleted">"UserDeleted"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _ORG_FREEDESKTOP_ACCOUNTS_SIGNALS[_ORG_FREEDESKTOP_ACCOUNTS_USER_DELETED] =
    g_signal_new ("user-deleted",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (OrgFreedesktopAccountsIface, user_deleted),
      NULL,
      NULL,
      org_freedesktop_accounts_signal_marshal_user_deleted,
      G_TYPE_NONE,
      1, G_TYPE_STRING);

  /* GObject properties for D-Bus properties: */
  /**
   * OrgFreedesktopAccounts:daemon-version:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-Accounts.DaemonVersion">"DaemonVersion"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("daemon-version", "DaemonVersion", "DaemonVersion", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
}

/**
 * org_freedesktop_accounts_get_daemon_version: (skip)
 * @object: A #OrgFreedesktopAccounts.
 *
 * Gets the value of the <link linkend="gdbus-property-org-freedesktop-Accounts.DaemonVersion">"DaemonVersion"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use org_freedesktop_accounts_dup_daemon_version() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
org_freedesktop_accounts_get_daemon_version (OrgFreedesktopAccounts *object)
{
  g_return_val_if_fail (IS_ORG_FREEDESKTOP_ACCOUNTS (object), NULL);

  return ORG_FREEDESKTOP_ACCOUNTS_GET_IFACE (object)->get_daemon_version (object);
}

/**
 * org_freedesktop_accounts_dup_daemon_version: (skip)
 * @object: A #OrgFreedesktopAccounts.
 *
 * Gets a copy of the <link linkend="gdbus-property-org-freedesktop-Accounts.DaemonVersion">"DaemonVersion"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
org_freedesktop_accounts_dup_daemon_version (OrgFreedesktopAccounts *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "daemon-version", &value, NULL);
  return value;
}

/**
 * org_freedesktop_accounts_set_daemon_version: (skip)
 * @object: A #OrgFreedesktopAccounts.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-freedesktop-Accounts.DaemonVersion">"DaemonVersion"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_freedesktop_accounts_set_daemon_version (OrgFreedesktopAccounts *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "daemon-version", value, NULL);
}

/**
 * org_freedesktop_accounts_emit_user_added:
 * @object: A #OrgFreedesktopAccounts.
 * @arg_user: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-org-freedesktop-Accounts.UserAdded">"UserAdded"</link> D-Bus signal.
 */
void
org_freedesktop_accounts_emit_user_added (
    OrgFreedesktopAccounts *object,
    const gchar *arg_user)
{
  g_signal_emit (object, _ORG_FREEDESKTOP_ACCOUNTS_SIGNALS[_ORG_FREEDESKTOP_ACCOUNTS_USER_ADDED], 0, arg_user);
}

/**
 * org_freedesktop_accounts_emit_user_deleted:
 * @object: A #OrgFreedesktopAccounts.
 * @arg_user: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-org-freedesktop-Accounts.UserDeleted">"UserDeleted"</link> D-Bus signal.
 */
void
org_freedesktop_accounts_emit_user_deleted (
    OrgFreedesktopAccounts *object,
    const gchar *arg_user)
{
  g_signal_emit (object, _ORG_FREEDESKTOP_ACCOUNTS_SIGNALS[_ORG_FREEDESKTOP_ACCOUNTS_USER_DELETED], 0, arg_user);
}

/**
 * org_freedesktop_accounts_call_list_cached_users:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.ListCachedUsers">ListCachedUsers()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_call_list_cached_users_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_call_list_cached_users_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_call_list_cached_users (
    OrgFreedesktopAccounts *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "ListCachedUsers",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_call_list_cached_users_finish:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @out_users: (out) (optional) (array zero-terminated=1): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_call_list_cached_users().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_call_list_cached_users().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_list_cached_users_finish (
    OrgFreedesktopAccounts *proxy,
    gchar ***out_users,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(^ao)",
                 out_users);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_call_list_cached_users_sync:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @out_users: (out) (optional) (array zero-terminated=1): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.ListCachedUsers">ListCachedUsers()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_call_list_cached_users() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_list_cached_users_sync (
    OrgFreedesktopAccounts *proxy,
    gchar ***out_users,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "ListCachedUsers",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(^ao)",
                 out_users);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_call_find_user_by_id:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @arg_id: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.FindUserById">FindUserById()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_call_find_user_by_id_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_call_find_user_by_id_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_call_find_user_by_id (
    OrgFreedesktopAccounts *proxy,
    gint64 arg_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "FindUserById",
    g_variant_new ("(x)",
                   arg_id),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_call_find_user_by_id_finish:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @out_user: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_call_find_user_by_id().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_call_find_user_by_id().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_find_user_by_id_finish (
    OrgFreedesktopAccounts *proxy,
    gchar **out_user,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(o)",
                 out_user);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_call_find_user_by_id_sync:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @arg_id: Argument to pass with the method invocation.
 * @out_user: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.FindUserById">FindUserById()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_call_find_user_by_id() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_find_user_by_id_sync (
    OrgFreedesktopAccounts *proxy,
    gint64 arg_id,
    gchar **out_user,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "FindUserById",
    g_variant_new ("(x)",
                   arg_id),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(o)",
                 out_user);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_call_find_user_by_name:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.FindUserByName">FindUserByName()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_call_find_user_by_name_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_call_find_user_by_name_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_call_find_user_by_name (
    OrgFreedesktopAccounts *proxy,
    const gchar *arg_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "FindUserByName",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_call_find_user_by_name_finish:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @out_user: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_call_find_user_by_name().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_call_find_user_by_name().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_find_user_by_name_finish (
    OrgFreedesktopAccounts *proxy,
    gchar **out_user,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(o)",
                 out_user);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_call_find_user_by_name_sync:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @out_user: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.FindUserByName">FindUserByName()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_call_find_user_by_name() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_find_user_by_name_sync (
    OrgFreedesktopAccounts *proxy,
    const gchar *arg_name,
    gchar **out_user,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "FindUserByName",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(o)",
                 out_user);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_call_create_user:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_fullname: Argument to pass with the method invocation.
 * @arg_accountType: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.CreateUser">CreateUser()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_call_create_user_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_call_create_user_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_call_create_user (
    OrgFreedesktopAccounts *proxy,
    const gchar *arg_name,
    const gchar *arg_fullname,
    gint arg_accountType,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "CreateUser",
    g_variant_new ("(ssi)",
                   arg_name,
                   arg_fullname,
                   arg_accountType),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_call_create_user_finish:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @out_user: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_call_create_user().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_call_create_user().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_create_user_finish (
    OrgFreedesktopAccounts *proxy,
    gchar **out_user,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(o)",
                 out_user);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_call_create_user_sync:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @arg_fullname: Argument to pass with the method invocation.
 * @arg_accountType: Argument to pass with the method invocation.
 * @out_user: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.CreateUser">CreateUser()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_call_create_user() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_create_user_sync (
    OrgFreedesktopAccounts *proxy,
    const gchar *arg_name,
    const gchar *arg_fullname,
    gint arg_accountType,
    gchar **out_user,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "CreateUser",
    g_variant_new ("(ssi)",
                   arg_name,
                   arg_fullname,
                   arg_accountType),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(o)",
                 out_user);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_call_cache_user:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.CacheUser">CacheUser()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_call_cache_user_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_call_cache_user_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_call_cache_user (
    OrgFreedesktopAccounts *proxy,
    const gchar *arg_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "CacheUser",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_call_cache_user_finish:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @out_user: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_call_cache_user().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_call_cache_user().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_cache_user_finish (
    OrgFreedesktopAccounts *proxy,
    gchar **out_user,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(o)",
                 out_user);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_call_cache_user_sync:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @out_user: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.CacheUser">CacheUser()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_call_cache_user() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_cache_user_sync (
    OrgFreedesktopAccounts *proxy,
    const gchar *arg_name,
    gchar **out_user,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "CacheUser",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(o)",
                 out_user);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_call_uncache_user:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.UncacheUser">UncacheUser()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_call_uncache_user_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_call_uncache_user_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_call_uncache_user (
    OrgFreedesktopAccounts *proxy,
    const gchar *arg_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "UncacheUser",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_call_uncache_user_finish:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_call_uncache_user().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_call_uncache_user().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_uncache_user_finish (
    OrgFreedesktopAccounts *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_call_uncache_user_sync:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @arg_name: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.UncacheUser">UncacheUser()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_call_uncache_user() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_uncache_user_sync (
    OrgFreedesktopAccounts *proxy,
    const gchar *arg_name,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "UncacheUser",
    g_variant_new ("(s)",
                   arg_name),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_call_delete_user:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @arg_id: Argument to pass with the method invocation.
 * @arg_removeFiles: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.DeleteUser">DeleteUser()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_call_delete_user_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_call_delete_user_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_accounts_call_delete_user (
    OrgFreedesktopAccounts *proxy,
    gint64 arg_id,
    gboolean arg_removeFiles,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "DeleteUser",
    g_variant_new ("(xb)",
                   arg_id,
                   arg_removeFiles),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_accounts_call_delete_user_finish:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_call_delete_user().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_accounts_call_delete_user().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_delete_user_finish (
    OrgFreedesktopAccounts *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_call_delete_user_sync:
 * @proxy: A #OrgFreedesktopAccountsProxy.
 * @arg_id: Argument to pass with the method invocation.
 * @arg_removeFiles: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-Accounts.DeleteUser">DeleteUser()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_call_delete_user() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_accounts_call_delete_user_sync (
    OrgFreedesktopAccounts *proxy,
    gint64 arg_id,
    gboolean arg_removeFiles,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "DeleteUser",
    g_variant_new ("(xb)",
                   arg_id,
                   arg_removeFiles),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_accounts_complete_list_cached_users:
 * @object: A #OrgFreedesktopAccounts.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @users: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts.ListCachedUsers">ListCachedUsers()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_complete_list_cached_users (
    OrgFreedesktopAccounts *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *const *users)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(^ao)",
                   users));
}

/**
 * org_freedesktop_accounts_complete_find_user_by_id:
 * @object: A #OrgFreedesktopAccounts.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @user: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts.FindUserById">FindUserById()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_complete_find_user_by_id (
    OrgFreedesktopAccounts *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *user)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(o)",
                   user));
}

/**
 * org_freedesktop_accounts_complete_find_user_by_name:
 * @object: A #OrgFreedesktopAccounts.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @user: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts.FindUserByName">FindUserByName()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_complete_find_user_by_name (
    OrgFreedesktopAccounts *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *user)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(o)",
                   user));
}

/**
 * org_freedesktop_accounts_complete_create_user:
 * @object: A #OrgFreedesktopAccounts.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @user: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts.CreateUser">CreateUser()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_complete_create_user (
    OrgFreedesktopAccounts *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *user)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(o)",
                   user));
}

/**
 * org_freedesktop_accounts_complete_cache_user:
 * @object: A #OrgFreedesktopAccounts.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @user: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts.CacheUser">CacheUser()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_complete_cache_user (
    OrgFreedesktopAccounts *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *user)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(o)",
                   user));
}

/**
 * org_freedesktop_accounts_complete_uncache_user:
 * @object: A #OrgFreedesktopAccounts.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts.UncacheUser">UncacheUser()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_complete_uncache_user (
    OrgFreedesktopAccounts *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_accounts_complete_delete_user:
 * @object: A #OrgFreedesktopAccounts.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-Accounts.DeleteUser">DeleteUser()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_accounts_complete_delete_user (
    OrgFreedesktopAccounts *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/* ------------------------------------------------------------------------ */

/**
 * OrgFreedesktopAccountsProxy:
 *
 * The #OrgFreedesktopAccountsProxy structure contains only private data and should only be accessed using the provided API.
 */

/**
 * OrgFreedesktopAccountsProxyClass:
 * @parent_class: The parent class.
 *
 * Class structure for #OrgFreedesktopAccountsProxy.
 */

struct _OrgFreedesktopAccountsProxyPrivate
{
  GData *qdata;
};

static void org_freedesktop_accounts_proxy_iface_init (OrgFreedesktopAccountsIface *iface);

#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (OrgFreedesktopAccountsProxy, org_freedesktop_accounts_proxy, G_TYPE_DBUS_PROXY,
                         G_ADD_PRIVATE (OrgFreedesktopAccountsProxy)
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_FREEDESKTOP_ACCOUNTS, org_freedesktop_accounts_proxy_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (OrgFreedesktopAccountsProxy, org_freedesktop_accounts_proxy, G_TYPE_DBUS_PROXY,
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_FREEDESKTOP_ACCOUNTS, org_freedesktop_accounts_proxy_iface_init))

#endif
static void
org_freedesktop_accounts_proxy_finalize (GObject *object)
{
  OrgFreedesktopAccountsProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_PROXY (object);
  g_datalist_clear (&proxy->priv->qdata);
  G_OBJECT_CLASS (org_freedesktop_accounts_proxy_parent_class)->finalize (object);
}

static void
org_freedesktop_accounts_proxy_get_property (GObject      *object,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  const _ExtendedGDBusPropertyInfo *info;
  GVariant *variant;
  g_assert (prop_id != 0 && prop_id - 1 < 1);
  info = (const _ExtendedGDBusPropertyInfo *) _org_freedesktop_accounts_property_info_pointers[prop_id - 1];
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (object), info->parent_struct.name);
  if (info->use_gvariant)
    {
      g_value_set_variant (value, variant);
    }
  else
    {
      if (variant != NULL)
        g_dbus_gvariant_to_gvalue (variant, value);
    }
  if (variant != NULL)
    g_variant_unref (variant);
}

static void
org_freedesktop_accounts_proxy_set_property_cb (GDBusProxy *proxy,
  GAsyncResult *res,
  gpointer      user_data)
{
  const _ExtendedGDBusPropertyInfo *info = user_data;
  GError *error;
  GVariant *_ret;
  error = NULL;
  _ret = g_dbus_proxy_call_finish (proxy, res, &error);
  if (!_ret)
    {
      g_warning ("Error setting property '%s' on interface org.freedesktop.Accounts: %s (%s, %d)",
                 info->parent_struct.name, 
                 error->message, g_quark_to_string (error->domain), error->code);
      g_error_free (error);
    }
  else
    {
      g_variant_unref (_ret);
    }
}

static void
org_freedesktop_accounts_proxy_set_property (GObject      *object,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  const _ExtendedGDBusPropertyInfo *info;
  GVariant *variant;
  g_assert (prop_id != 0 && prop_id - 1 < 1);
  info = (const _ExtendedGDBusPropertyInfo *) _org_freedesktop_accounts_property_info_pointers[prop_id - 1];
  variant = g_dbus_gvalue_to_gvariant (value, G_VARIANT_TYPE (info->parent_struct.signature));
  g_dbus_proxy_call (G_DBUS_PROXY (object),
    "org.freedesktop.DBus.Properties.Set",
    g_variant_new ("(ssv)", "org.freedesktop.Accounts", info->parent_struct.name, variant),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    NULL, (GAsyncReadyCallback) org_freedesktop_accounts_proxy_set_property_cb, (GDBusPropertyInfo *) &info->parent_struct);
  g_variant_unref (variant);
}

static void
org_freedesktop_accounts_proxy_g_signal (GDBusProxy *proxy,
  const gchar *sender_name G_GNUC_UNUSED,
  const gchar *signal_name,
  GVariant *parameters)
{
  _ExtendedGDBusSignalInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  gsize n;
  guint signal_id;
  info = (_ExtendedGDBusSignalInfo *) g_dbus_interface_info_lookup_signal ((GDBusInterfaceInfo *) &_org_freedesktop_accounts_interface_info.parent_struct, signal_name);
  if (info == NULL)
    return;
  num_params = g_variant_n_children (parameters);
  paramv = g_new0 (GValue, num_params + 1);
  g_value_init (&paramv[0], TYPE_ORG_FREEDESKTOP_ACCOUNTS);
  g_value_set_object (&paramv[0], proxy);
  g_variant_iter_init (&iter, parameters);
  n = 1;
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.args[n - 1];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_ORG_FREEDESKTOP_ACCOUNTS);
  g_signal_emitv (paramv, signal_id, 0, NULL);
  for (n = 0; n < num_params + 1; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static void
org_freedesktop_accounts_proxy_g_properties_changed (GDBusProxy *_proxy,
  GVariant *changed_properties,
  const gchar *const *invalidated_properties)
{
  OrgFreedesktopAccountsProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_PROXY (_proxy);
  guint n;
  const gchar *key;
  GVariantIter *iter;
  _ExtendedGDBusPropertyInfo *info;
  g_variant_get (changed_properties, "a{sv}", &iter);
  while (g_variant_iter_next (iter, "{&sv}", &key, NULL))
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_freedesktop_accounts_interface_info.parent_struct, key);
      g_datalist_remove_data (&proxy->priv->qdata, key);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
  g_variant_iter_free (iter);
  for (n = 0; invalidated_properties[n] != NULL; n++)
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_freedesktop_accounts_interface_info.parent_struct, invalidated_properties[n]);
      g_datalist_remove_data (&proxy->priv->qdata, invalidated_properties[n]);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
}

static const gchar *
org_freedesktop_accounts_proxy_get_daemon_version (OrgFreedesktopAccounts *object)
{
  OrgFreedesktopAccountsProxy *proxy = ORG_FREEDESKTOP_ACCOUNTS_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "DaemonVersion");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static void
org_freedesktop_accounts_proxy_init (OrgFreedesktopAccountsProxy *proxy)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  proxy->priv = org_freedesktop_accounts_proxy_get_instance_private (proxy);
#else
  proxy->priv = G_TYPE_INSTANCE_GET_PRIVATE (proxy, TYPE_ORG_FREEDESKTOP_ACCOUNTS_PROXY, OrgFreedesktopAccountsProxyPrivate);
#endif

  g_dbus_proxy_set_interface_info (G_DBUS_PROXY (proxy), org_freedesktop_accounts_interface_info ());
}

static void
org_freedesktop_accounts_proxy_class_init (OrgFreedesktopAccountsProxyClass *klass)
{
  GObjectClass *gobject_class;
  GDBusProxyClass *proxy_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize     = org_freedesktop_accounts_proxy_finalize;
  gobject_class->get_property = org_freedesktop_accounts_proxy_get_property;
  gobject_class->set_property = org_freedesktop_accounts_proxy_set_property;

  proxy_class = G_DBUS_PROXY_CLASS (klass);
  proxy_class->g_signal = org_freedesktop_accounts_proxy_g_signal;
  proxy_class->g_properties_changed = org_freedesktop_accounts_proxy_g_properties_changed;

  org_freedesktop_accounts_override_properties (gobject_class, 1);

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (OrgFreedesktopAccountsProxyPrivate));
#endif
}

static void
org_freedesktop_accounts_proxy_iface_init (OrgFreedesktopAccountsIface *iface)
{
  iface->get_daemon_version = org_freedesktop_accounts_proxy_get_daemon_version;
}

/**
 * org_freedesktop_accounts_proxy_new:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-Accounts.top_of_page">org.freedesktop.Accounts</link>. See g_dbus_proxy_new() for more details.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_proxy_new_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_proxy_new_sync() for the synchronous, blocking version of this constructor.
 */
void
org_freedesktop_accounts_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_ORG_FREEDESKTOP_ACCOUNTS_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.freedesktop.Accounts", NULL);
}

/**
 * org_freedesktop_accounts_proxy_new_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_proxy_new().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with org_freedesktop_accounts_proxy_new().
 *
 * Returns: (transfer full) (type OrgFreedesktopAccountsProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgFreedesktopAccounts *
org_freedesktop_accounts_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return ORG_FREEDESKTOP_ACCOUNTS (ret);
  else
    return NULL;
}

/**
 * org_freedesktop_accounts_proxy_new_sync:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Synchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-Accounts.top_of_page">org.freedesktop.Accounts</link>. See g_dbus_proxy_new_sync() for more details.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_proxy_new() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type OrgFreedesktopAccountsProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgFreedesktopAccounts *
org_freedesktop_accounts_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_ORG_FREEDESKTOP_ACCOUNTS_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.freedesktop.Accounts", NULL);
  if (ret != NULL)
    return ORG_FREEDESKTOP_ACCOUNTS (ret);
  else
    return NULL;
}


/**
 * org_freedesktop_accounts_proxy_new_for_bus:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Like org_freedesktop_accounts_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_accounts_proxy_new_for_bus_finish() to get the result of the operation.
 *
 * See org_freedesktop_accounts_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
 */
void
org_freedesktop_accounts_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_ORG_FREEDESKTOP_ACCOUNTS_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.freedesktop.Accounts", NULL);
}

/**
 * org_freedesktop_accounts_proxy_new_for_bus_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_accounts_proxy_new_for_bus().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with org_freedesktop_accounts_proxy_new_for_bus().
 *
 * Returns: (transfer full) (type OrgFreedesktopAccountsProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgFreedesktopAccounts *
org_freedesktop_accounts_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return ORG_FREEDESKTOP_ACCOUNTS (ret);
  else
    return NULL;
}

/**
 * org_freedesktop_accounts_proxy_new_for_bus_sync:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Like org_freedesktop_accounts_proxy_new_sync() but takes a #GBusType instead of a #GDBusConnection.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_accounts_proxy_new_for_bus() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type OrgFreedesktopAccountsProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgFreedesktopAccounts *
org_freedesktop_accounts_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_ORG_FREEDESKTOP_ACCOUNTS_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.freedesktop.Accounts", NULL);
  if (ret != NULL)
    return ORG_FREEDESKTOP_ACCOUNTS (ret);
  else
    return NULL;
}


/* ------------------------------------------------------------------------ */

/**
 * OrgFreedesktopAccountsSkeleton:
 *
 * The #OrgFreedesktopAccountsSkeleton structure contains only private data and should only be accessed using the provided API.
 */

/**
 * OrgFreedesktopAccountsSkeletonClass:
 * @parent_class: The parent class.
 *
 * Class structure for #OrgFreedesktopAccountsSkeleton.
 */

struct _OrgFreedesktopAccountsSkeletonPrivate
{
  GValue *properties;
  GList *changed_properties;
  GSource *changed_properties_idle_source;
  GMainContext *context;
  GMutex lock;
};

static void
_org_freedesktop_accounts_skeleton_handle_method_call (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name,
  const gchar *method_name,
  GVariant *parameters,
  GDBusMethodInvocation *invocation,
  gpointer user_data)
{
  OrgFreedesktopAccountsSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_SKELETON (user_data);
  _ExtendedGDBusMethodInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  guint num_extra;
  gsize n;
  guint signal_id;
  GValue return_value = G_VALUE_INIT;
  info = (_ExtendedGDBusMethodInfo *) g_dbus_method_invocation_get_method_info (invocation);
  g_assert (info != NULL);
  num_params = g_variant_n_children (parameters);
  num_extra = info->pass_fdlist ? 3 : 2;  paramv = g_new0 (GValue, num_params + num_extra);
  n = 0;
  g_value_init (&paramv[n], TYPE_ORG_FREEDESKTOP_ACCOUNTS);
  g_value_set_object (&paramv[n++], skeleton);
  g_value_init (&paramv[n], G_TYPE_DBUS_METHOD_INVOCATION);
  g_value_set_object (&paramv[n++], invocation);
  if (info->pass_fdlist)
    {
#ifdef G_OS_UNIX
      g_value_init (&paramv[n], G_TYPE_UNIX_FD_LIST);
      g_value_set_object (&paramv[n++], g_dbus_message_get_unix_fd_list (g_dbus_method_invocation_get_message (invocation)));
#else
      g_assert_not_reached ();
#endif
    }
  g_variant_iter_init (&iter, parameters);
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.in_args[n - num_extra];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_ORG_FREEDESKTOP_ACCOUNTS);
  g_value_init (&return_value, G_TYPE_BOOLEAN);
  g_signal_emitv (paramv, signal_id, 0, &return_value);
  if (!g_value_get_boolean (&return_value))
    g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD, "Method %s is not implemented on interface %s", method_name, interface_name);
  g_value_unset (&return_value);
  for (n = 0; n < num_params + num_extra; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static GVariant *
_org_freedesktop_accounts_skeleton_handle_get_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GError **error,
  gpointer user_data)
{
  OrgFreedesktopAccountsSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  GVariant *ret;
  ret = NULL;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_freedesktop_accounts_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      g_value_init (&value, pspec->value_type);
      g_object_get_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      ret = g_dbus_gvalue_to_gvariant (&value, G_VARIANT_TYPE (info->parent_struct.signature));
      g_value_unset (&value);
    }
  return ret;
}

static gboolean
_org_freedesktop_accounts_skeleton_handle_set_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GVariant *variant,
  GError **error,
  gpointer user_data)
{
  OrgFreedesktopAccountsSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  gboolean ret;
  ret = FALSE;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_freedesktop_accounts_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      if (info->use_gvariant)
        g_value_set_variant (&value, variant);
      else
        g_dbus_gvariant_to_gvalue (variant, &value);
      g_object_set_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      g_value_unset (&value);
      ret = TRUE;
    }
  return ret;
}

static const GDBusInterfaceVTable _org_freedesktop_accounts_skeleton_vtable =
{
  _org_freedesktop_accounts_skeleton_handle_method_call,
  _org_freedesktop_accounts_skeleton_handle_get_property,
  _org_freedesktop_accounts_skeleton_handle_set_property,
  {NULL}
};

static GDBusInterfaceInfo *
org_freedesktop_accounts_skeleton_dbus_interface_get_info (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return org_freedesktop_accounts_interface_info ();
}

static GDBusInterfaceVTable *
org_freedesktop_accounts_skeleton_dbus_interface_get_vtable (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return (GDBusInterfaceVTable *) &_org_freedesktop_accounts_skeleton_vtable;
}

static GVariant *
org_freedesktop_accounts_skeleton_dbus_interface_get_properties (GDBusInterfaceSkeleton *_skeleton)
{
  OrgFreedesktopAccountsSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_SKELETON (_skeleton);

  GVariantBuilder builder;
  guint n;
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
#else
  g_variant_builder_init(&builder, G_VARIANT_TYPE ("a{sv}"));
#endif
  if (_org_freedesktop_accounts_interface_info.parent_struct.properties == NULL)
    goto out;
  for (n = 0; _org_freedesktop_accounts_interface_info.parent_struct.properties[n] != NULL; n++)
    {
      GDBusPropertyInfo *info = _org_freedesktop_accounts_interface_info.parent_struct.properties[n];
      if (info->flags & G_DBUS_PROPERTY_INFO_FLAGS_READABLE)
        {
          GVariant *value;
          value = _org_freedesktop_accounts_skeleton_handle_get_property (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)), NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.freedesktop.Accounts", info->name, NULL, skeleton);
          if (value != NULL)
            {
              g_variant_take_ref (value);
              g_variant_builder_add (&builder, "{sv}", info->name, value);
              g_variant_unref (value);
            }
        }
    }
out:
  return g_variant_builder_end (&builder);
}

static gboolean _org_freedesktop_accounts_emit_changed (gpointer user_data);

static void
org_freedesktop_accounts_skeleton_dbus_interface_flush (GDBusInterfaceSkeleton *_skeleton)
{
  OrgFreedesktopAccountsSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_SKELETON (_skeleton);
  gboolean emit_changed = FALSE;

  g_mutex_lock (&skeleton->priv->lock);
  if (skeleton->priv->changed_properties_idle_source != NULL)
    {
      g_source_destroy (skeleton->priv->changed_properties_idle_source);
      skeleton->priv->changed_properties_idle_source = NULL;
      emit_changed = TRUE;
    }
  g_mutex_unlock (&skeleton->priv->lock);

  if (emit_changed)
    _org_freedesktop_accounts_emit_changed (skeleton);
}

static void
_org_freedesktop_accounts_on_signal_user_added (
    OrgFreedesktopAccounts *object,
    const gchar *arg_user)
{
  OrgFreedesktopAccountsSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(o)",
                   arg_user));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.freedesktop.Accounts", "UserAdded",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void
_org_freedesktop_accounts_on_signal_user_deleted (
    OrgFreedesktopAccounts *object,
    const gchar *arg_user)
{
  OrgFreedesktopAccountsSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(o)",
                   arg_user));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.freedesktop.Accounts", "UserDeleted",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void org_freedesktop_accounts_skeleton_iface_init (OrgFreedesktopAccountsIface *iface);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (OrgFreedesktopAccountsSkeleton, org_freedesktop_accounts_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_ADD_PRIVATE (OrgFreedesktopAccountsSkeleton)
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_FREEDESKTOP_ACCOUNTS, org_freedesktop_accounts_skeleton_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (OrgFreedesktopAccountsSkeleton, org_freedesktop_accounts_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_FREEDESKTOP_ACCOUNTS, org_freedesktop_accounts_skeleton_iface_init))

#endif
static void
org_freedesktop_accounts_skeleton_finalize (GObject *object)
{
  OrgFreedesktopAccountsSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_SKELETON (object);
  guint n;
  for (n = 0; n < 1; n++)
    g_value_unset (&skeleton->priv->properties[n]);
  g_free (skeleton->priv->properties);
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
  if (skeleton->priv->changed_properties_idle_source != NULL)
    g_source_destroy (skeleton->priv->changed_properties_idle_source);
  g_main_context_unref (skeleton->priv->context);
  g_mutex_clear (&skeleton->priv->lock);
  G_OBJECT_CLASS (org_freedesktop_accounts_skeleton_parent_class)->finalize (object);
}

static void
org_freedesktop_accounts_skeleton_get_property (GObject      *object,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  OrgFreedesktopAccountsSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_SKELETON (object);
  g_assert (prop_id != 0 && prop_id - 1 < 1);
  g_mutex_lock (&skeleton->priv->lock);
  g_value_copy (&skeleton->priv->properties[prop_id - 1], value);
  g_mutex_unlock (&skeleton->priv->lock);
}

static gboolean
_org_freedesktop_accounts_emit_changed (gpointer user_data)
{
  OrgFreedesktopAccountsSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_SKELETON (user_data);
  GList *l;
  GVariantBuilder builder;
  GVariantBuilder invalidated_builder;
  guint num_changes;

  g_mutex_lock (&skeleton->priv->lock);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
  g_variant_builder_init_static (&invalidated_builder, G_VARIANT_TYPE ("as"));
#else
  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
  g_variant_builder_init (&invalidated_builder, G_VARIANT_TYPE ("as"));
#endif
  for (l = skeleton->priv->changed_properties, num_changes = 0; l != NULL; l = l->next)
    {
      ChangedProperty *cp = l->data;
      GVariant *variant;
      const GValue *cur_value;

      cur_value = &skeleton->priv->properties[cp->prop_id - 1];
      if (!_g_value_equal (cur_value, &cp->orig_value))
        {
          variant = g_dbus_gvalue_to_gvariant (cur_value, G_VARIANT_TYPE (cp->info->parent_struct.signature));
          g_variant_builder_add (&builder, "{sv}", cp->info->parent_struct.name, variant);
          g_variant_unref (variant);
          num_changes++;
        }
    }
  if (num_changes > 0)
    {
      GList *connections, *ll;
      GVariant *signal_variant;
      signal_variant = g_variant_ref_sink (g_variant_new ("(sa{sv}as)", "org.freedesktop.Accounts",
                                           &builder, &invalidated_builder));
      connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));
      for (ll = connections; ll != NULL; ll = ll->next)
        {
          GDBusConnection *connection = ll->data;

          g_dbus_connection_emit_signal (connection,
                                         NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)),
                                         "org.freedesktop.DBus.Properties",
                                         "PropertiesChanged",
                                         signal_variant,
                                         NULL);
        }
      g_variant_unref (signal_variant);
      g_list_free_full (connections, g_object_unref);
    }
  else
    {
      g_variant_builder_clear (&builder);
      g_variant_builder_clear (&invalidated_builder);
    }
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
  skeleton->priv->changed_properties = NULL;
  skeleton->priv->changed_properties_idle_source = NULL;
  g_mutex_unlock (&skeleton->priv->lock);
  return FALSE;
}

static void
_org_freedesktop_accounts_schedule_emit_changed (OrgFreedesktopAccountsSkeleton *skeleton, const _ExtendedGDBusPropertyInfo *info, guint prop_id, const GValue *orig_value)
{
  ChangedProperty *cp;
  GList *l;
  cp = NULL;
  for (l = skeleton->priv->changed_properties; l != NULL; l = l->next)
    {
      ChangedProperty *i_cp = l->data;
      if (i_cp->info == info)
        {
          cp = i_cp;
          break;
        }
    }
  if (cp == NULL)
    {
      cp = g_new0 (ChangedProperty, 1);
      cp->prop_id = prop_id;
      cp->info = info;
      skeleton->priv->changed_properties = g_list_prepend (skeleton->priv->changed_properties, cp);
      g_value_init (&cp->orig_value, G_VALUE_TYPE (orig_value));
      g_value_copy (orig_value, &cp->orig_value);
    }
}

static void
org_freedesktop_accounts_skeleton_notify (GObject      *object,
  GParamSpec *pspec G_GNUC_UNUSED)
{
  OrgFreedesktopAccountsSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_SKELETON (object);
  g_mutex_lock (&skeleton->priv->lock);
  if (skeleton->priv->changed_properties != NULL &&
      skeleton->priv->changed_properties_idle_source == NULL)
    {
      skeleton->priv->changed_properties_idle_source = g_idle_source_new ();
      g_source_set_priority (skeleton->priv->changed_properties_idle_source, G_PRIORITY_DEFAULT);
      g_source_set_callback (skeleton->priv->changed_properties_idle_source, _org_freedesktop_accounts_emit_changed, g_object_ref (skeleton), (GDestroyNotify) g_object_unref);
      g_source_set_name (skeleton->priv->changed_properties_idle_source, "[generated] _org_freedesktop_accounts_emit_changed");
      g_source_attach (skeleton->priv->changed_properties_idle_source, skeleton->priv->context);
      g_source_unref (skeleton->priv->changed_properties_idle_source);
    }
  g_mutex_unlock (&skeleton->priv->lock);
}

static void
org_freedesktop_accounts_skeleton_set_property (GObject      *object,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec)
{
  const _ExtendedGDBusPropertyInfo *info;
  OrgFreedesktopAccountsSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_SKELETON (object);
  g_assert (prop_id != 0 && prop_id - 1 < 1);
  info = (const _ExtendedGDBusPropertyInfo *) _org_freedesktop_accounts_property_info_pointers[prop_id - 1];
  g_mutex_lock (&skeleton->priv->lock);
  g_object_freeze_notify (object);
  if (!_g_value_equal (value, &skeleton->priv->properties[prop_id - 1]))
    {
      if (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)) != NULL &&
          info->emits_changed_signal)
        _org_freedesktop_accounts_schedule_emit_changed (skeleton, info, prop_id, &skeleton->priv->properties[prop_id - 1]);
      g_value_copy (value, &skeleton->priv->properties[prop_id - 1]);
      g_object_notify_by_pspec (object, pspec);
    }
  g_mutex_unlock (&skeleton->priv->lock);
  g_object_thaw_notify (object);
}

static void
org_freedesktop_accounts_skeleton_init (OrgFreedesktopAccountsSkeleton *skeleton)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  skeleton->priv = org_freedesktop_accounts_skeleton_get_instance_private (skeleton);
#else
  skeleton->priv = G_TYPE_INSTANCE_GET_PRIVATE (skeleton, TYPE_ORG_FREEDESKTOP_ACCOUNTS_SKELETON, OrgFreedesktopAccountsSkeletonPrivate);
#endif

  g_mutex_init (&skeleton->priv->lock);
  skeleton->priv->context = g_main_context_ref_thread_default ();
  skeleton->priv->properties = g_new0 (GValue, 1);
  g_value_init (&skeleton->priv->properties[0], G_TYPE_STRING);
}

static const gchar *
org_freedesktop_accounts_skeleton_get_daemon_version (OrgFreedesktopAccounts *object)
{
  OrgFreedesktopAccountsSkeleton *skeleton = ORG_FREEDESKTOP_ACCOUNTS_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[0]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static void
org_freedesktop_accounts_skeleton_class_init (OrgFreedesktopAccountsSkeletonClass *klass)
{
  GObjectClass *gobject_class;
  GDBusInterfaceSkeletonClass *skeleton_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize = org_freedesktop_accounts_skeleton_finalize;
  gobject_class->get_property = org_freedesktop_accounts_skeleton_get_property;
  gobject_class->set_property = org_freedesktop_accounts_skeleton_set_property;
  gobject_class->notify       = org_freedesktop_accounts_skeleton_notify;


  org_freedesktop_accounts_override_properties (gobject_class, 1);

  skeleton_class = G_DBUS_INTERFACE_SKELETON_CLASS (klass);
  skeleton_class->get_info = org_freedesktop_accounts_skeleton_dbus_interface_get_info;
  skeleton_class->get_properties = org_freedesktop_accounts_skeleton_dbus_interface_get_properties;
  skeleton_class->flush = org_freedesktop_accounts_skeleton_dbus_interface_flush;
  skeleton_class->get_vtable = org_freedesktop_accounts_skeleton_dbus_interface_get_vtable;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (OrgFreedesktopAccountsSkeletonPrivate));
#endif
}

static void
org_freedesktop_accounts_skeleton_iface_init (OrgFreedesktopAccountsIface *iface)
{
  iface->user_added = _org_freedesktop_accounts_on_signal_user_added;
  iface->user_deleted = _org_freedesktop_accounts_on_signal_user_deleted;
  iface->get_daemon_version = org_freedesktop_accounts_skeleton_get_daemon_version;
}

/**
 * org_freedesktop_accounts_skeleton_new:
 *
 * Creates a skeleton object for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-Accounts.top_of_page">org.freedesktop.Accounts</link>.
 *
 * Returns: (transfer full) (type OrgFreedesktopAccountsSkeleton): The skeleton object.
 */
OrgFreedesktopAccounts *
org_freedesktop_accounts_skeleton_new (void)
{
  return ORG_FREEDESKTOP_ACCOUNTS (g_object_new (TYPE_ORG_FREEDESKTOP_ACCOUNTS_SKELETON, NULL));
}

/* ------------------------------------------------------------------------
 * Code for interface org.freedesktop.ScreenSaver
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:OrgFreedesktopScreenSaver
 * @title: OrgFreedesktopScreenSaver
 * @short_description: Generated C code for the org.freedesktop.ScreenSaver D-Bus interface
 *
 * This section contains code for working with the <link linkend="gdbus-interface-org-freedesktop-ScreenSaver.top_of_page">org.freedesktop.ScreenSaver</link> D-Bus interface in C.
 */

enum
{
  _ORG_FREEDESKTOP_SCREEN_SAVER_ACTIVE_CHANGED,
};

static unsigned _ORG_FREEDESKTOP_SCREEN_SAVER_SIGNALS[1] = { 0 };

/* ---- Introspection data for org.freedesktop.ScreenSaver ---- */

static const _ExtendedGDBusMethodInfo _org_freedesktop_screen_saver_method_info_lock =
{
  {
    -1,
    (gchar *) "Lock",
    NULL,
    NULL,
    NULL
  },
  "handle-lock",
  FALSE
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_screen_saver_method_info_simulate_user_activity =
{
  {
    -1,
    (gchar *) "SimulateUserActivity",
    NULL,
    NULL,
    NULL
  },
  "handle-simulate-user-activity",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_method_info_get_active_OUT_ARG_unnamed_arg0 =
{
  {
    -1,
    (gchar *) "unnamed_arg0",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_screen_saver_method_info_get_active_OUT_ARG_pointers[] =
{
  &_org_freedesktop_screen_saver_method_info_get_active_OUT_ARG_unnamed_arg0.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_screen_saver_method_info_get_active =
{
  {
    -1,
    (gchar *) "GetActive",
    NULL,
    (GDBusArgInfo **) &_org_freedesktop_screen_saver_method_info_get_active_OUT_ARG_pointers,
    NULL
  },
  "handle-get-active",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_method_info_get_active_time_OUT_ARG_seconds =
{
  {
    -1,
    (gchar *) "seconds",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_screen_saver_method_info_get_active_time_OUT_ARG_pointers[] =
{
  &_org_freedesktop_screen_saver_method_info_get_active_time_OUT_ARG_seconds.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_screen_saver_method_info_get_active_time =
{
  {
    -1,
    (gchar *) "GetActiveTime",
    NULL,
    (GDBusArgInfo **) &_org_freedesktop_screen_saver_method_info_get_active_time_OUT_ARG_pointers,
    NULL
  },
  "handle-get-active-time",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_method_info_get_session_idle_time_OUT_ARG_seconds =
{
  {
    -1,
    (gchar *) "seconds",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_screen_saver_method_info_get_session_idle_time_OUT_ARG_pointers[] =
{
  &_org_freedesktop_screen_saver_method_info_get_session_idle_time_OUT_ARG_seconds.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_screen_saver_method_info_get_session_idle_time =
{
  {
    -1,
    (gchar *) "GetSessionIdleTime",
    NULL,
    (GDBusArgInfo **) &_org_freedesktop_screen_saver_method_info_get_session_idle_time_OUT_ARG_pointers,
    NULL
  },
  "handle-get-session-idle-time",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_method_info_set_active_IN_ARG_e =
{
  {
    -1,
    (gchar *) "e",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_screen_saver_method_info_set_active_IN_ARG_pointers[] =
{
  &_org_freedesktop_screen_saver_method_info_set_active_IN_ARG_e.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_method_info_set_active_OUT_ARG_unnamed_arg1 =
{
  {
    -1,
    (gchar *) "unnamed_arg1",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_screen_saver_method_info_set_active_OUT_ARG_pointers[] =
{
  &_org_freedesktop_screen_saver_method_info_set_active_OUT_ARG_unnamed_arg1.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_screen_saver_method_info_set_active =
{
  {
    -1,
    (gchar *) "SetActive",
    (GDBusArgInfo **) &_org_freedesktop_screen_saver_method_info_set_active_IN_ARG_pointers,
    (GDBusArgInfo **) &_org_freedesktop_screen_saver_method_info_set_active_OUT_ARG_pointers,
    NULL
  },
  "handle-set-active",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_method_info_inhibit_IN_ARG_application_name =
{
  {
    -1,
    (gchar *) "application_name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_method_info_inhibit_IN_ARG_reason_for_inhibit =
{
  {
    -1,
    (gchar *) "reason_for_inhibit",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_screen_saver_method_info_inhibit_IN_ARG_pointers[] =
{
  &_org_freedesktop_screen_saver_method_info_inhibit_IN_ARG_application_name.parent_struct,
  &_org_freedesktop_screen_saver_method_info_inhibit_IN_ARG_reason_for_inhibit.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_method_info_inhibit_OUT_ARG_cookie =
{
  {
    -1,
    (gchar *) "cookie",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_screen_saver_method_info_inhibit_OUT_ARG_pointers[] =
{
  &_org_freedesktop_screen_saver_method_info_inhibit_OUT_ARG_cookie.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_screen_saver_method_info_inhibit =
{
  {
    -1,
    (gchar *) "Inhibit",
    (GDBusArgInfo **) &_org_freedesktop_screen_saver_method_info_inhibit_IN_ARG_pointers,
    (GDBusArgInfo **) &_org_freedesktop_screen_saver_method_info_inhibit_OUT_ARG_pointers,
    NULL
  },
  "handle-inhibit",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_method_info_un_inhibit_IN_ARG_cookie =
{
  {
    -1,
    (gchar *) "cookie",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_screen_saver_method_info_un_inhibit_IN_ARG_pointers[] =
{
  &_org_freedesktop_screen_saver_method_info_un_inhibit_IN_ARG_cookie.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_screen_saver_method_info_un_inhibit =
{
  {
    -1,
    (gchar *) "UnInhibit",
    (GDBusArgInfo **) &_org_freedesktop_screen_saver_method_info_un_inhibit_IN_ARG_pointers,
    NULL,
    NULL
  },
  "handle-un-inhibit",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_method_info_throttle_IN_ARG_application_name =
{
  {
    -1,
    (gchar *) "application_name",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_method_info_throttle_IN_ARG_reason_for_inhibit =
{
  {
    -1,
    (gchar *) "reason_for_inhibit",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_screen_saver_method_info_throttle_IN_ARG_pointers[] =
{
  &_org_freedesktop_screen_saver_method_info_throttle_IN_ARG_application_name.parent_struct,
  &_org_freedesktop_screen_saver_method_info_throttle_IN_ARG_reason_for_inhibit.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_method_info_throttle_OUT_ARG_cookie =
{
  {
    -1,
    (gchar *) "cookie",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_screen_saver_method_info_throttle_OUT_ARG_pointers[] =
{
  &_org_freedesktop_screen_saver_method_info_throttle_OUT_ARG_cookie.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_screen_saver_method_info_throttle =
{
  {
    -1,
    (gchar *) "Throttle",
    (GDBusArgInfo **) &_org_freedesktop_screen_saver_method_info_throttle_IN_ARG_pointers,
    (GDBusArgInfo **) &_org_freedesktop_screen_saver_method_info_throttle_OUT_ARG_pointers,
    NULL
  },
  "handle-throttle",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_method_info_un_throttle_IN_ARG_cookie =
{
  {
    -1,
    (gchar *) "cookie",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_screen_saver_method_info_un_throttle_IN_ARG_pointers[] =
{
  &_org_freedesktop_screen_saver_method_info_un_throttle_IN_ARG_cookie.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_freedesktop_screen_saver_method_info_un_throttle =
{
  {
    -1,
    (gchar *) "UnThrottle",
    (GDBusArgInfo **) &_org_freedesktop_screen_saver_method_info_un_throttle_IN_ARG_pointers,
    NULL,
    NULL
  },
  "handle-un-throttle",
  FALSE
};

static const GDBusMethodInfo * const _org_freedesktop_screen_saver_method_info_pointers[] =
{
  &_org_freedesktop_screen_saver_method_info_lock.parent_struct,
  &_org_freedesktop_screen_saver_method_info_simulate_user_activity.parent_struct,
  &_org_freedesktop_screen_saver_method_info_get_active.parent_struct,
  &_org_freedesktop_screen_saver_method_info_get_active_time.parent_struct,
  &_org_freedesktop_screen_saver_method_info_get_session_idle_time.parent_struct,
  &_org_freedesktop_screen_saver_method_info_set_active.parent_struct,
  &_org_freedesktop_screen_saver_method_info_inhibit.parent_struct,
  &_org_freedesktop_screen_saver_method_info_un_inhibit.parent_struct,
  &_org_freedesktop_screen_saver_method_info_throttle.parent_struct,
  &_org_freedesktop_screen_saver_method_info_un_throttle.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_freedesktop_screen_saver_signal_info_active_changed_ARG_unnamed_arg0 =
{
  {
    -1,
    (gchar *) "unnamed_arg0",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_freedesktop_screen_saver_signal_info_active_changed_ARG_pointers[] =
{
  &_org_freedesktop_screen_saver_signal_info_active_changed_ARG_unnamed_arg0.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _org_freedesktop_screen_saver_signal_info_active_changed =
{
  {
    -1,
    (gchar *) "ActiveChanged",
    (GDBusArgInfo **) &_org_freedesktop_screen_saver_signal_info_active_changed_ARG_pointers,
    NULL
  },
  "active-changed"
};

static const GDBusSignalInfo * const _org_freedesktop_screen_saver_signal_info_pointers[] =
{
  &_org_freedesktop_screen_saver_signal_info_active_changed.parent_struct,
  NULL
};

static const _ExtendedGDBusInterfaceInfo _org_freedesktop_screen_saver_interface_info =
{
  {
    -1,
    (gchar *) "org.freedesktop.ScreenSaver",
    (GDBusMethodInfo **) &_org_freedesktop_screen_saver_method_info_pointers,
    (GDBusSignalInfo **) &_org_freedesktop_screen_saver_signal_info_pointers,
    NULL,
    NULL
  },
  "org-freedesktop-screen-saver",
};


/**
 * org_freedesktop_screen_saver_interface_info:
 *
 * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-ScreenSaver.top_of_page">org.freedesktop.ScreenSaver</link> D-Bus interface.
 *
 * Returns: (transfer none): A #GDBusInterfaceInfo. Do not free.
 */
GDBusInterfaceInfo *
org_freedesktop_screen_saver_interface_info (void)
{
  return (GDBusInterfaceInfo *) &_org_freedesktop_screen_saver_interface_info.parent_struct;
}

/**
 * org_freedesktop_screen_saver_override_properties:
 * @klass: The class structure for a #GObject derived class.
 * @property_id_begin: The property id to assign to the first overridden property.
 *
 * Overrides all #GObject properties in the #OrgFreedesktopScreenSaver interface for a concrete class.
 * The properties are overridden in the order they are defined.
 *
 * Returns: The last property id.
 */
guint
org_freedesktop_screen_saver_override_properties (GObjectClass *klass G_GNUC_UNUSED, guint property_id_begin)
{
  return property_id_begin - 1;
}


inline static void
org_freedesktop_screen_saver_signal_marshal_active_changed (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__BOOLEAN (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_screen_saver_method_marshal_lock (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_screen_saver_method_marshal_simulate_user_activity (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_screen_saver_method_marshal_get_active (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_screen_saver_method_marshal_get_active_time (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_screen_saver_method_marshal_get_session_idle_time (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_screen_saver_method_marshal_set_active (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_BOOLEAN (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_screen_saver_method_marshal_inhibit (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_screen_saver_method_marshal_un_inhibit (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_UINT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_screen_saver_method_marshal_throttle (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_freedesktop_screen_saver_method_marshal_un_throttle (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_UINT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}


/**
 * OrgFreedesktopScreenSaver:
 *
 * Abstract interface type for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-ScreenSaver.top_of_page">org.freedesktop.ScreenSaver</link>.
 */

/**
 * OrgFreedesktopScreenSaverIface:
 * @parent_iface: The parent interface.
 * @handle_get_active: Handler for the #OrgFreedesktopScreenSaver::handle-get-active signal.
 * @handle_get_active_time: Handler for the #OrgFreedesktopScreenSaver::handle-get-active-time signal.
 * @handle_get_session_idle_time: Handler for the #OrgFreedesktopScreenSaver::handle-get-session-idle-time signal.
 * @handle_inhibit: Handler for the #OrgFreedesktopScreenSaver::handle-inhibit signal.
 * @handle_lock: Handler for the #OrgFreedesktopScreenSaver::handle-lock signal.
 * @handle_set_active: Handler for the #OrgFreedesktopScreenSaver::handle-set-active signal.
 * @handle_simulate_user_activity: Handler for the #OrgFreedesktopScreenSaver::handle-simulate-user-activity signal.
 * @handle_throttle: Handler for the #OrgFreedesktopScreenSaver::handle-throttle signal.
 * @handle_un_inhibit: Handler for the #OrgFreedesktopScreenSaver::handle-un-inhibit signal.
 * @handle_un_throttle: Handler for the #OrgFreedesktopScreenSaver::handle-un-throttle signal.
 * @active_changed: Handler for the #OrgFreedesktopScreenSaver::active-changed signal.
 *
 * Virtual table for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-ScreenSaver.top_of_page">org.freedesktop.ScreenSaver</link>.
 */

typedef OrgFreedesktopScreenSaverIface OrgFreedesktopScreenSaverInterface;
G_DEFINE_INTERFACE (OrgFreedesktopScreenSaver, org_freedesktop_screen_saver, G_TYPE_OBJECT)

static void
org_freedesktop_screen_saver_default_init (OrgFreedesktopScreenSaverIface *iface)
{
  /* GObject signals for incoming D-Bus method calls: */
  /**
   * OrgFreedesktopScreenSaver::handle-lock:
   * @object: A #OrgFreedesktopScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.Lock">Lock()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_screen_saver_complete_lock() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-lock",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopScreenSaverIface, handle_lock),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_screen_saver_method_marshal_lock,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * OrgFreedesktopScreenSaver::handle-simulate-user-activity:
   * @object: A #OrgFreedesktopScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.SimulateUserActivity">SimulateUserActivity()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_screen_saver_complete_simulate_user_activity() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-simulate-user-activity",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopScreenSaverIface, handle_simulate_user_activity),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_screen_saver_method_marshal_simulate_user_activity,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * OrgFreedesktopScreenSaver::handle-get-active:
   * @object: A #OrgFreedesktopScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.GetActive">GetActive()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_screen_saver_complete_get_active() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-active",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopScreenSaverIface, handle_get_active),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_screen_saver_method_marshal_get_active,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * OrgFreedesktopScreenSaver::handle-get-active-time:
   * @object: A #OrgFreedesktopScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.GetActiveTime">GetActiveTime()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_screen_saver_complete_get_active_time() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-active-time",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopScreenSaverIface, handle_get_active_time),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_screen_saver_method_marshal_get_active_time,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * OrgFreedesktopScreenSaver::handle-get-session-idle-time:
   * @object: A #OrgFreedesktopScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.GetSessionIdleTime">GetSessionIdleTime()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_screen_saver_complete_get_session_idle_time() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-session-idle-time",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopScreenSaverIface, handle_get_session_idle_time),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_screen_saver_method_marshal_get_session_idle_time,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * OrgFreedesktopScreenSaver::handle-set-active:
   * @object: A #OrgFreedesktopScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_e: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.SetActive">SetActive()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_screen_saver_complete_set_active() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-active",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopScreenSaverIface, handle_set_active),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_screen_saver_method_marshal_set_active,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_BOOLEAN);

  /**
   * OrgFreedesktopScreenSaver::handle-inhibit:
   * @object: A #OrgFreedesktopScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_application_name: Argument passed by remote caller.
   * @arg_reason_for_inhibit: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.Inhibit">Inhibit()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_screen_saver_complete_inhibit() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-inhibit",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopScreenSaverIface, handle_inhibit),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_screen_saver_method_marshal_inhibit,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * OrgFreedesktopScreenSaver::handle-un-inhibit:
   * @object: A #OrgFreedesktopScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_cookie: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.UnInhibit">UnInhibit()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_screen_saver_complete_un_inhibit() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-un-inhibit",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopScreenSaverIface, handle_un_inhibit),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_screen_saver_method_marshal_un_inhibit,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_UINT);

  /**
   * OrgFreedesktopScreenSaver::handle-throttle:
   * @object: A #OrgFreedesktopScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_application_name: Argument passed by remote caller.
   * @arg_reason_for_inhibit: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.Throttle">Throttle()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_screen_saver_complete_throttle() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-throttle",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopScreenSaverIface, handle_throttle),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_screen_saver_method_marshal_throttle,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * OrgFreedesktopScreenSaver::handle-un-throttle:
   * @object: A #OrgFreedesktopScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_cookie: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.UnThrottle">UnThrottle()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_freedesktop_screen_saver_complete_un_throttle() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-un-throttle",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgFreedesktopScreenSaverIface, handle_un_throttle),
    g_signal_accumulator_true_handled,
    NULL,
      org_freedesktop_screen_saver_method_marshal_un_throttle,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_UINT);

  /* GObject signals for received D-Bus signals: */
  /**
   * OrgFreedesktopScreenSaver::active-changed:
   * @object: A #OrgFreedesktopScreenSaver.
   * @arg_unnamed_arg0: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-org-freedesktop-ScreenSaver.ActiveChanged">"ActiveChanged"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _ORG_FREEDESKTOP_SCREEN_SAVER_SIGNALS[_ORG_FREEDESKTOP_SCREEN_SAVER_ACTIVE_CHANGED] =
    g_signal_new ("active-changed",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (OrgFreedesktopScreenSaverIface, active_changed),
      NULL,
      NULL,
      org_freedesktop_screen_saver_signal_marshal_active_changed,
      G_TYPE_NONE,
      1, G_TYPE_BOOLEAN);

}

/**
 * org_freedesktop_screen_saver_emit_active_changed:
 * @object: A #OrgFreedesktopScreenSaver.
 * @arg_unnamed_arg0: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-org-freedesktop-ScreenSaver.ActiveChanged">"ActiveChanged"</link> D-Bus signal.
 */
void
org_freedesktop_screen_saver_emit_active_changed (
    OrgFreedesktopScreenSaver *object,
    gboolean arg_unnamed_arg0)
{
  g_signal_emit (object, _ORG_FREEDESKTOP_SCREEN_SAVER_SIGNALS[_ORG_FREEDESKTOP_SCREEN_SAVER_ACTIVE_CHANGED], 0, arg_unnamed_arg0);
}

/**
 * org_freedesktop_screen_saver_call_lock:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.Lock">Lock()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_screen_saver_call_lock_finish() to get the result of the operation.
 *
 * See org_freedesktop_screen_saver_call_lock_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_screen_saver_call_lock (
    OrgFreedesktopScreenSaver *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "Lock",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_screen_saver_call_lock_finish:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_screen_saver_call_lock().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_screen_saver_call_lock().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_lock_finish (
    OrgFreedesktopScreenSaver *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_lock_sync:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.Lock">Lock()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_screen_saver_call_lock() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_lock_sync (
    OrgFreedesktopScreenSaver *proxy,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "Lock",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_simulate_user_activity:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.SimulateUserActivity">SimulateUserActivity()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_screen_saver_call_simulate_user_activity_finish() to get the result of the operation.
 *
 * See org_freedesktop_screen_saver_call_simulate_user_activity_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_screen_saver_call_simulate_user_activity (
    OrgFreedesktopScreenSaver *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SimulateUserActivity",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_screen_saver_call_simulate_user_activity_finish:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_screen_saver_call_simulate_user_activity().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_screen_saver_call_simulate_user_activity().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_simulate_user_activity_finish (
    OrgFreedesktopScreenSaver *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_simulate_user_activity_sync:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.SimulateUserActivity">SimulateUserActivity()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_screen_saver_call_simulate_user_activity() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_simulate_user_activity_sync (
    OrgFreedesktopScreenSaver *proxy,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SimulateUserActivity",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_get_active:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.GetActive">GetActive()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_screen_saver_call_get_active_finish() to get the result of the operation.
 *
 * See org_freedesktop_screen_saver_call_get_active_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_screen_saver_call_get_active (
    OrgFreedesktopScreenSaver *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetActive",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_screen_saver_call_get_active_finish:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @out_unnamed_arg0: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_screen_saver_call_get_active().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_screen_saver_call_get_active().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_get_active_finish (
    OrgFreedesktopScreenSaver *proxy,
    gboolean *out_unnamed_arg0,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_unnamed_arg0);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_get_active_sync:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @out_unnamed_arg0: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.GetActive">GetActive()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_screen_saver_call_get_active() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_get_active_sync (
    OrgFreedesktopScreenSaver *proxy,
    gboolean *out_unnamed_arg0,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetActive",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_unnamed_arg0);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_get_active_time:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.GetActiveTime">GetActiveTime()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_screen_saver_call_get_active_time_finish() to get the result of the operation.
 *
 * See org_freedesktop_screen_saver_call_get_active_time_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_screen_saver_call_get_active_time (
    OrgFreedesktopScreenSaver *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetActiveTime",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_screen_saver_call_get_active_time_finish:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @out_seconds: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_screen_saver_call_get_active_time().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_screen_saver_call_get_active_time().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_get_active_time_finish (
    OrgFreedesktopScreenSaver *proxy,
    guint *out_seconds,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_seconds);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_get_active_time_sync:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @out_seconds: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.GetActiveTime">GetActiveTime()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_screen_saver_call_get_active_time() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_get_active_time_sync (
    OrgFreedesktopScreenSaver *proxy,
    guint *out_seconds,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetActiveTime",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_seconds);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_get_session_idle_time:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.GetSessionIdleTime">GetSessionIdleTime()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_screen_saver_call_get_session_idle_time_finish() to get the result of the operation.
 *
 * See org_freedesktop_screen_saver_call_get_session_idle_time_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_screen_saver_call_get_session_idle_time (
    OrgFreedesktopScreenSaver *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetSessionIdleTime",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_screen_saver_call_get_session_idle_time_finish:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @out_seconds: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_screen_saver_call_get_session_idle_time().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_screen_saver_call_get_session_idle_time().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_get_session_idle_time_finish (
    OrgFreedesktopScreenSaver *proxy,
    guint *out_seconds,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_seconds);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_get_session_idle_time_sync:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @out_seconds: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.GetSessionIdleTime">GetSessionIdleTime()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_screen_saver_call_get_session_idle_time() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_get_session_idle_time_sync (
    OrgFreedesktopScreenSaver *proxy,
    guint *out_seconds,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetSessionIdleTime",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_seconds);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_set_active:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @arg_e: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.SetActive">SetActive()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_screen_saver_call_set_active_finish() to get the result of the operation.
 *
 * See org_freedesktop_screen_saver_call_set_active_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_screen_saver_call_set_active (
    OrgFreedesktopScreenSaver *proxy,
    gboolean arg_e,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetActive",
    g_variant_new ("(b)",
                   arg_e),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_screen_saver_call_set_active_finish:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @out_unnamed_arg1: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_screen_saver_call_set_active().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_screen_saver_call_set_active().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_set_active_finish (
    OrgFreedesktopScreenSaver *proxy,
    gboolean *out_unnamed_arg1,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_unnamed_arg1);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_set_active_sync:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @arg_e: Argument to pass with the method invocation.
 * @out_unnamed_arg1: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.SetActive">SetActive()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_screen_saver_call_set_active() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_set_active_sync (
    OrgFreedesktopScreenSaver *proxy,
    gboolean arg_e,
    gboolean *out_unnamed_arg1,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetActive",
    g_variant_new ("(b)",
                   arg_e),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_unnamed_arg1);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_inhibit:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @arg_application_name: Argument to pass with the method invocation.
 * @arg_reason_for_inhibit: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.Inhibit">Inhibit()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_screen_saver_call_inhibit_finish() to get the result of the operation.
 *
 * See org_freedesktop_screen_saver_call_inhibit_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_screen_saver_call_inhibit (
    OrgFreedesktopScreenSaver *proxy,
    const gchar *arg_application_name,
    const gchar *arg_reason_for_inhibit,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "Inhibit",
    g_variant_new ("(ss)",
                   arg_application_name,
                   arg_reason_for_inhibit),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_screen_saver_call_inhibit_finish:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @out_cookie: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_screen_saver_call_inhibit().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_screen_saver_call_inhibit().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_inhibit_finish (
    OrgFreedesktopScreenSaver *proxy,
    guint *out_cookie,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_cookie);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_inhibit_sync:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @arg_application_name: Argument to pass with the method invocation.
 * @arg_reason_for_inhibit: Argument to pass with the method invocation.
 * @out_cookie: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.Inhibit">Inhibit()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_screen_saver_call_inhibit() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_inhibit_sync (
    OrgFreedesktopScreenSaver *proxy,
    const gchar *arg_application_name,
    const gchar *arg_reason_for_inhibit,
    guint *out_cookie,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "Inhibit",
    g_variant_new ("(ss)",
                   arg_application_name,
                   arg_reason_for_inhibit),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_cookie);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_un_inhibit:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @arg_cookie: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.UnInhibit">UnInhibit()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_screen_saver_call_un_inhibit_finish() to get the result of the operation.
 *
 * See org_freedesktop_screen_saver_call_un_inhibit_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_screen_saver_call_un_inhibit (
    OrgFreedesktopScreenSaver *proxy,
    guint arg_cookie,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "UnInhibit",
    g_variant_new ("(u)",
                   arg_cookie),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_screen_saver_call_un_inhibit_finish:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_screen_saver_call_un_inhibit().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_screen_saver_call_un_inhibit().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_un_inhibit_finish (
    OrgFreedesktopScreenSaver *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_un_inhibit_sync:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @arg_cookie: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.UnInhibit">UnInhibit()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_screen_saver_call_un_inhibit() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_un_inhibit_sync (
    OrgFreedesktopScreenSaver *proxy,
    guint arg_cookie,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "UnInhibit",
    g_variant_new ("(u)",
                   arg_cookie),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_throttle:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @arg_application_name: Argument to pass with the method invocation.
 * @arg_reason_for_inhibit: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.Throttle">Throttle()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_screen_saver_call_throttle_finish() to get the result of the operation.
 *
 * See org_freedesktop_screen_saver_call_throttle_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_screen_saver_call_throttle (
    OrgFreedesktopScreenSaver *proxy,
    const gchar *arg_application_name,
    const gchar *arg_reason_for_inhibit,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "Throttle",
    g_variant_new ("(ss)",
                   arg_application_name,
                   arg_reason_for_inhibit),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_screen_saver_call_throttle_finish:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @out_cookie: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_screen_saver_call_throttle().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_screen_saver_call_throttle().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_throttle_finish (
    OrgFreedesktopScreenSaver *proxy,
    guint *out_cookie,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_cookie);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_throttle_sync:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @arg_application_name: Argument to pass with the method invocation.
 * @arg_reason_for_inhibit: Argument to pass with the method invocation.
 * @out_cookie: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.Throttle">Throttle()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_screen_saver_call_throttle() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_throttle_sync (
    OrgFreedesktopScreenSaver *proxy,
    const gchar *arg_application_name,
    const gchar *arg_reason_for_inhibit,
    guint *out_cookie,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "Throttle",
    g_variant_new ("(ss)",
                   arg_application_name,
                   arg_reason_for_inhibit),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_cookie);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_un_throttle:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @arg_cookie: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.UnThrottle">UnThrottle()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_screen_saver_call_un_throttle_finish() to get the result of the operation.
 *
 * See org_freedesktop_screen_saver_call_un_throttle_sync() for the synchronous, blocking version of this method.
 */
void
org_freedesktop_screen_saver_call_un_throttle (
    OrgFreedesktopScreenSaver *proxy,
    guint arg_cookie,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "UnThrottle",
    g_variant_new ("(u)",
                   arg_cookie),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_freedesktop_screen_saver_call_un_throttle_finish:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_screen_saver_call_un_throttle().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_freedesktop_screen_saver_call_un_throttle().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_un_throttle_finish (
    OrgFreedesktopScreenSaver *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_call_un_throttle_sync:
 * @proxy: A #OrgFreedesktopScreenSaverProxy.
 * @arg_cookie: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.UnThrottle">UnThrottle()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_screen_saver_call_un_throttle() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_freedesktop_screen_saver_call_un_throttle_sync (
    OrgFreedesktopScreenSaver *proxy,
    guint arg_cookie,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "UnThrottle",
    g_variant_new ("(u)",
                   arg_cookie),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_freedesktop_screen_saver_complete_lock:
 * @object: A #OrgFreedesktopScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.Lock">Lock()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_screen_saver_complete_lock (
    OrgFreedesktopScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_screen_saver_complete_simulate_user_activity:
 * @object: A #OrgFreedesktopScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.SimulateUserActivity">SimulateUserActivity()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_screen_saver_complete_simulate_user_activity (
    OrgFreedesktopScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_screen_saver_complete_get_active:
 * @object: A #OrgFreedesktopScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @unnamed_arg0: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.GetActive">GetActive()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_screen_saver_complete_get_active (
    OrgFreedesktopScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    gboolean unnamed_arg0)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(b)",
                   unnamed_arg0));
}

/**
 * org_freedesktop_screen_saver_complete_get_active_time:
 * @object: A #OrgFreedesktopScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @seconds: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.GetActiveTime">GetActiveTime()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_screen_saver_complete_get_active_time (
    OrgFreedesktopScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    guint seconds)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(u)",
                   seconds));
}

/**
 * org_freedesktop_screen_saver_complete_get_session_idle_time:
 * @object: A #OrgFreedesktopScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @seconds: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.GetSessionIdleTime">GetSessionIdleTime()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_screen_saver_complete_get_session_idle_time (
    OrgFreedesktopScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    guint seconds)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(u)",
                   seconds));
}

/**
 * org_freedesktop_screen_saver_complete_set_active:
 * @object: A #OrgFreedesktopScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @unnamed_arg1: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.SetActive">SetActive()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_screen_saver_complete_set_active (
    OrgFreedesktopScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    gboolean unnamed_arg1)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(b)",
                   unnamed_arg1));
}

/**
 * org_freedesktop_screen_saver_complete_inhibit:
 * @object: A #OrgFreedesktopScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @cookie: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.Inhibit">Inhibit()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_screen_saver_complete_inhibit (
    OrgFreedesktopScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    guint cookie)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(u)",
                   cookie));
}

/**
 * org_freedesktop_screen_saver_complete_un_inhibit:
 * @object: A #OrgFreedesktopScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.UnInhibit">UnInhibit()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_screen_saver_complete_un_inhibit (
    OrgFreedesktopScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_freedesktop_screen_saver_complete_throttle:
 * @object: A #OrgFreedesktopScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @cookie: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.Throttle">Throttle()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_screen_saver_complete_throttle (
    OrgFreedesktopScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    guint cookie)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(u)",
                   cookie));
}

/**
 * org_freedesktop_screen_saver_complete_un_throttle:
 * @object: A #OrgFreedesktopScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-ScreenSaver.UnThrottle">UnThrottle()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_freedesktop_screen_saver_complete_un_throttle (
    OrgFreedesktopScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/* ------------------------------------------------------------------------ */

/**
 * OrgFreedesktopScreenSaverProxy:
 *
 * The #OrgFreedesktopScreenSaverProxy structure contains only private data and should only be accessed using the provided API.
 */

/**
 * OrgFreedesktopScreenSaverProxyClass:
 * @parent_class: The parent class.
 *
 * Class structure for #OrgFreedesktopScreenSaverProxy.
 */

struct _OrgFreedesktopScreenSaverProxyPrivate
{
  GData *qdata;
};

static void org_freedesktop_screen_saver_proxy_iface_init (OrgFreedesktopScreenSaverIface *iface);

#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (OrgFreedesktopScreenSaverProxy, org_freedesktop_screen_saver_proxy, G_TYPE_DBUS_PROXY,
                         G_ADD_PRIVATE (OrgFreedesktopScreenSaverProxy)
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_FREEDESKTOP_SCREEN_SAVER, org_freedesktop_screen_saver_proxy_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (OrgFreedesktopScreenSaverProxy, org_freedesktop_screen_saver_proxy, G_TYPE_DBUS_PROXY,
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_FREEDESKTOP_SCREEN_SAVER, org_freedesktop_screen_saver_proxy_iface_init))

#endif
static void
org_freedesktop_screen_saver_proxy_finalize (GObject *object)
{
  OrgFreedesktopScreenSaverProxy *proxy = ORG_FREEDESKTOP_SCREEN_SAVER_PROXY (object);
  g_datalist_clear (&proxy->priv->qdata);
  G_OBJECT_CLASS (org_freedesktop_screen_saver_proxy_parent_class)->finalize (object);
}

static void
org_freedesktop_screen_saver_proxy_get_property (GObject      *object G_GNUC_UNUSED,
  guint         prop_id G_GNUC_UNUSED,
  GValue       *value G_GNUC_UNUSED,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
}

static void
org_freedesktop_screen_saver_proxy_set_property (GObject      *object G_GNUC_UNUSED,
  guint         prop_id G_GNUC_UNUSED,
  const GValue *value G_GNUC_UNUSED,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
}

static void
org_freedesktop_screen_saver_proxy_g_signal (GDBusProxy *proxy,
  const gchar *sender_name G_GNUC_UNUSED,
  const gchar *signal_name,
  GVariant *parameters)
{
  _ExtendedGDBusSignalInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  gsize n;
  guint signal_id;
  info = (_ExtendedGDBusSignalInfo *) g_dbus_interface_info_lookup_signal ((GDBusInterfaceInfo *) &_org_freedesktop_screen_saver_interface_info.parent_struct, signal_name);
  if (info == NULL)
    return;
  num_params = g_variant_n_children (parameters);
  paramv = g_new0 (GValue, num_params + 1);
  g_value_init (&paramv[0], TYPE_ORG_FREEDESKTOP_SCREEN_SAVER);
  g_value_set_object (&paramv[0], proxy);
  g_variant_iter_init (&iter, parameters);
  n = 1;
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.args[n - 1];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_ORG_FREEDESKTOP_SCREEN_SAVER);
  g_signal_emitv (paramv, signal_id, 0, NULL);
  for (n = 0; n < num_params + 1; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static void
org_freedesktop_screen_saver_proxy_g_properties_changed (GDBusProxy *_proxy,
  GVariant *changed_properties,
  const gchar *const *invalidated_properties)
{
  OrgFreedesktopScreenSaverProxy *proxy = ORG_FREEDESKTOP_SCREEN_SAVER_PROXY (_proxy);
  guint n;
  const gchar *key;
  GVariantIter *iter;
  _ExtendedGDBusPropertyInfo *info;
  g_variant_get (changed_properties, "a{sv}", &iter);
  while (g_variant_iter_next (iter, "{&sv}", &key, NULL))
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_freedesktop_screen_saver_interface_info.parent_struct, key);
      g_datalist_remove_data (&proxy->priv->qdata, key);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
  g_variant_iter_free (iter);
  for (n = 0; invalidated_properties[n] != NULL; n++)
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_freedesktop_screen_saver_interface_info.parent_struct, invalidated_properties[n]);
      g_datalist_remove_data (&proxy->priv->qdata, invalidated_properties[n]);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
}

static void
org_freedesktop_screen_saver_proxy_init (OrgFreedesktopScreenSaverProxy *proxy)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  proxy->priv = org_freedesktop_screen_saver_proxy_get_instance_private (proxy);
#else
  proxy->priv = G_TYPE_INSTANCE_GET_PRIVATE (proxy, TYPE_ORG_FREEDESKTOP_SCREEN_SAVER_PROXY, OrgFreedesktopScreenSaverProxyPrivate);
#endif

  g_dbus_proxy_set_interface_info (G_DBUS_PROXY (proxy), org_freedesktop_screen_saver_interface_info ());
}

static void
org_freedesktop_screen_saver_proxy_class_init (OrgFreedesktopScreenSaverProxyClass *klass)
{
  GObjectClass *gobject_class;
  GDBusProxyClass *proxy_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize     = org_freedesktop_screen_saver_proxy_finalize;
  gobject_class->get_property = org_freedesktop_screen_saver_proxy_get_property;
  gobject_class->set_property = org_freedesktop_screen_saver_proxy_set_property;

  proxy_class = G_DBUS_PROXY_CLASS (klass);
  proxy_class->g_signal = org_freedesktop_screen_saver_proxy_g_signal;
  proxy_class->g_properties_changed = org_freedesktop_screen_saver_proxy_g_properties_changed;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (OrgFreedesktopScreenSaverProxyPrivate));
#endif
}

static void
org_freedesktop_screen_saver_proxy_iface_init (OrgFreedesktopScreenSaverIface *iface G_GNUC_UNUSED)
{
}

/**
 * org_freedesktop_screen_saver_proxy_new:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-ScreenSaver.top_of_page">org.freedesktop.ScreenSaver</link>. See g_dbus_proxy_new() for more details.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_screen_saver_proxy_new_finish() to get the result of the operation.
 *
 * See org_freedesktop_screen_saver_proxy_new_sync() for the synchronous, blocking version of this constructor.
 */
void
org_freedesktop_screen_saver_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_ORG_FREEDESKTOP_SCREEN_SAVER_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.freedesktop.ScreenSaver", NULL);
}

/**
 * org_freedesktop_screen_saver_proxy_new_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_screen_saver_proxy_new().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with org_freedesktop_screen_saver_proxy_new().
 *
 * Returns: (transfer full) (type OrgFreedesktopScreenSaverProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgFreedesktopScreenSaver *
org_freedesktop_screen_saver_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return ORG_FREEDESKTOP_SCREEN_SAVER (ret);
  else
    return NULL;
}

/**
 * org_freedesktop_screen_saver_proxy_new_sync:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Synchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-ScreenSaver.top_of_page">org.freedesktop.ScreenSaver</link>. See g_dbus_proxy_new_sync() for more details.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_screen_saver_proxy_new() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type OrgFreedesktopScreenSaverProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgFreedesktopScreenSaver *
org_freedesktop_screen_saver_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_ORG_FREEDESKTOP_SCREEN_SAVER_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.freedesktop.ScreenSaver", NULL);
  if (ret != NULL)
    return ORG_FREEDESKTOP_SCREEN_SAVER (ret);
  else
    return NULL;
}


/**
 * org_freedesktop_screen_saver_proxy_new_for_bus:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Like org_freedesktop_screen_saver_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_freedesktop_screen_saver_proxy_new_for_bus_finish() to get the result of the operation.
 *
 * See org_freedesktop_screen_saver_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
 */
void
org_freedesktop_screen_saver_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_ORG_FREEDESKTOP_SCREEN_SAVER_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.freedesktop.ScreenSaver", NULL);
}

/**
 * org_freedesktop_screen_saver_proxy_new_for_bus_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_freedesktop_screen_saver_proxy_new_for_bus().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with org_freedesktop_screen_saver_proxy_new_for_bus().
 *
 * Returns: (transfer full) (type OrgFreedesktopScreenSaverProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgFreedesktopScreenSaver *
org_freedesktop_screen_saver_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return ORG_FREEDESKTOP_SCREEN_SAVER (ret);
  else
    return NULL;
}

/**
 * org_freedesktop_screen_saver_proxy_new_for_bus_sync:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Like org_freedesktop_screen_saver_proxy_new_sync() but takes a #GBusType instead of a #GDBusConnection.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See org_freedesktop_screen_saver_proxy_new_for_bus() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type OrgFreedesktopScreenSaverProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgFreedesktopScreenSaver *
org_freedesktop_screen_saver_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_ORG_FREEDESKTOP_SCREEN_SAVER_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.freedesktop.ScreenSaver", NULL);
  if (ret != NULL)
    return ORG_FREEDESKTOP_SCREEN_SAVER (ret);
  else
    return NULL;
}


/* ------------------------------------------------------------------------ */

/**
 * OrgFreedesktopScreenSaverSkeleton:
 *
 * The #OrgFreedesktopScreenSaverSkeleton structure contains only private data and should only be accessed using the provided API.
 */

/**
 * OrgFreedesktopScreenSaverSkeletonClass:
 * @parent_class: The parent class.
 *
 * Class structure for #OrgFreedesktopScreenSaverSkeleton.
 */

struct _OrgFreedesktopScreenSaverSkeletonPrivate
{
  GValue *properties;
  GList *changed_properties;
  GSource *changed_properties_idle_source;
  GMainContext *context;
  GMutex lock;
};

static void
_org_freedesktop_screen_saver_skeleton_handle_method_call (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name,
  const gchar *method_name,
  GVariant *parameters,
  GDBusMethodInvocation *invocation,
  gpointer user_data)
{
  OrgFreedesktopScreenSaverSkeleton *skeleton = ORG_FREEDESKTOP_SCREEN_SAVER_SKELETON (user_data);
  _ExtendedGDBusMethodInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  guint num_extra;
  gsize n;
  guint signal_id;
  GValue return_value = G_VALUE_INIT;
  info = (_ExtendedGDBusMethodInfo *) g_dbus_method_invocation_get_method_info (invocation);
  g_assert (info != NULL);
  num_params = g_variant_n_children (parameters);
  num_extra = info->pass_fdlist ? 3 : 2;  paramv = g_new0 (GValue, num_params + num_extra);
  n = 0;
  g_value_init (&paramv[n], TYPE_ORG_FREEDESKTOP_SCREEN_SAVER);
  g_value_set_object (&paramv[n++], skeleton);
  g_value_init (&paramv[n], G_TYPE_DBUS_METHOD_INVOCATION);
  g_value_set_object (&paramv[n++], invocation);
  if (info->pass_fdlist)
    {
#ifdef G_OS_UNIX
      g_value_init (&paramv[n], G_TYPE_UNIX_FD_LIST);
      g_value_set_object (&paramv[n++], g_dbus_message_get_unix_fd_list (g_dbus_method_invocation_get_message (invocation)));
#else
      g_assert_not_reached ();
#endif
    }
  g_variant_iter_init (&iter, parameters);
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.in_args[n - num_extra];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_ORG_FREEDESKTOP_SCREEN_SAVER);
  g_value_init (&return_value, G_TYPE_BOOLEAN);
  g_signal_emitv (paramv, signal_id, 0, &return_value);
  if (!g_value_get_boolean (&return_value))
    g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD, "Method %s is not implemented on interface %s", method_name, interface_name);
  g_value_unset (&return_value);
  for (n = 0; n < num_params + num_extra; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static GVariant *
_org_freedesktop_screen_saver_skeleton_handle_get_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GError **error,
  gpointer user_data)
{
  OrgFreedesktopScreenSaverSkeleton *skeleton = ORG_FREEDESKTOP_SCREEN_SAVER_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  GVariant *ret;
  ret = NULL;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_freedesktop_screen_saver_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      g_value_init (&value, pspec->value_type);
      g_object_get_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      ret = g_dbus_gvalue_to_gvariant (&value, G_VARIANT_TYPE (info->parent_struct.signature));
      g_value_unset (&value);
    }
  return ret;
}

static gboolean
_org_freedesktop_screen_saver_skeleton_handle_set_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GVariant *variant,
  GError **error,
  gpointer user_data)
{
  OrgFreedesktopScreenSaverSkeleton *skeleton = ORG_FREEDESKTOP_SCREEN_SAVER_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  gboolean ret;
  ret = FALSE;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_freedesktop_screen_saver_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      if (info->use_gvariant)
        g_value_set_variant (&value, variant);
      else
        g_dbus_gvariant_to_gvalue (variant, &value);
      g_object_set_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      g_value_unset (&value);
      ret = TRUE;
    }
  return ret;
}

static const GDBusInterfaceVTable _org_freedesktop_screen_saver_skeleton_vtable =
{
  _org_freedesktop_screen_saver_skeleton_handle_method_call,
  _org_freedesktop_screen_saver_skeleton_handle_get_property,
  _org_freedesktop_screen_saver_skeleton_handle_set_property,
  {NULL}
};

static GDBusInterfaceInfo *
org_freedesktop_screen_saver_skeleton_dbus_interface_get_info (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return org_freedesktop_screen_saver_interface_info ();
}

static GDBusInterfaceVTable *
org_freedesktop_screen_saver_skeleton_dbus_interface_get_vtable (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return (GDBusInterfaceVTable *) &_org_freedesktop_screen_saver_skeleton_vtable;
}

static GVariant *
org_freedesktop_screen_saver_skeleton_dbus_interface_get_properties (GDBusInterfaceSkeleton *_skeleton)
{
  OrgFreedesktopScreenSaverSkeleton *skeleton = ORG_FREEDESKTOP_SCREEN_SAVER_SKELETON (_skeleton);

  GVariantBuilder builder;
  guint n;
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
#else
  g_variant_builder_init(&builder, G_VARIANT_TYPE ("a{sv}"));
#endif
  if (_org_freedesktop_screen_saver_interface_info.parent_struct.properties == NULL)
    goto out;
  for (n = 0; _org_freedesktop_screen_saver_interface_info.parent_struct.properties[n] != NULL; n++)
    {
      GDBusPropertyInfo *info = _org_freedesktop_screen_saver_interface_info.parent_struct.properties[n];
      if (info->flags & G_DBUS_PROPERTY_INFO_FLAGS_READABLE)
        {
          GVariant *value;
          value = _org_freedesktop_screen_saver_skeleton_handle_get_property (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)), NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.freedesktop.ScreenSaver", info->name, NULL, skeleton);
          if (value != NULL)
            {
              g_variant_take_ref (value);
              g_variant_builder_add (&builder, "{sv}", info->name, value);
              g_variant_unref (value);
            }
        }
    }
out:
  return g_variant_builder_end (&builder);
}

static void
org_freedesktop_screen_saver_skeleton_dbus_interface_flush (GDBusInterfaceSkeleton *_skeleton G_GNUC_UNUSED)
{
}

static void
_org_freedesktop_screen_saver_on_signal_active_changed (
    OrgFreedesktopScreenSaver *object,
    gboolean arg_unnamed_arg0)
{
  OrgFreedesktopScreenSaverSkeleton *skeleton = ORG_FREEDESKTOP_SCREEN_SAVER_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(b)",
                   arg_unnamed_arg0));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.freedesktop.ScreenSaver", "ActiveChanged",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void org_freedesktop_screen_saver_skeleton_iface_init (OrgFreedesktopScreenSaverIface *iface);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (OrgFreedesktopScreenSaverSkeleton, org_freedesktop_screen_saver_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_ADD_PRIVATE (OrgFreedesktopScreenSaverSkeleton)
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_FREEDESKTOP_SCREEN_SAVER, org_freedesktop_screen_saver_skeleton_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (OrgFreedesktopScreenSaverSkeleton, org_freedesktop_screen_saver_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_FREEDESKTOP_SCREEN_SAVER, org_freedesktop_screen_saver_skeleton_iface_init))

#endif
static void
org_freedesktop_screen_saver_skeleton_finalize (GObject *object)
{
  OrgFreedesktopScreenSaverSkeleton *skeleton = ORG_FREEDESKTOP_SCREEN_SAVER_SKELETON (object);
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
  if (skeleton->priv->changed_properties_idle_source != NULL)
    g_source_destroy (skeleton->priv->changed_properties_idle_source);
  g_main_context_unref (skeleton->priv->context);
  g_mutex_clear (&skeleton->priv->lock);
  G_OBJECT_CLASS (org_freedesktop_screen_saver_skeleton_parent_class)->finalize (object);
}

static void
org_freedesktop_screen_saver_skeleton_init (OrgFreedesktopScreenSaverSkeleton *skeleton)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  skeleton->priv = org_freedesktop_screen_saver_skeleton_get_instance_private (skeleton);
#else
  skeleton->priv = G_TYPE_INSTANCE_GET_PRIVATE (skeleton, TYPE_ORG_FREEDESKTOP_SCREEN_SAVER_SKELETON, OrgFreedesktopScreenSaverSkeletonPrivate);
#endif

  g_mutex_init (&skeleton->priv->lock);
  skeleton->priv->context = g_main_context_ref_thread_default ();
}

static void
org_freedesktop_screen_saver_skeleton_class_init (OrgFreedesktopScreenSaverSkeletonClass *klass)
{
  GObjectClass *gobject_class;
  GDBusInterfaceSkeletonClass *skeleton_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize = org_freedesktop_screen_saver_skeleton_finalize;

  skeleton_class = G_DBUS_INTERFACE_SKELETON_CLASS (klass);
  skeleton_class->get_info = org_freedesktop_screen_saver_skeleton_dbus_interface_get_info;
  skeleton_class->get_properties = org_freedesktop_screen_saver_skeleton_dbus_interface_get_properties;
  skeleton_class->flush = org_freedesktop_screen_saver_skeleton_dbus_interface_flush;
  skeleton_class->get_vtable = org_freedesktop_screen_saver_skeleton_dbus_interface_get_vtable;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (OrgFreedesktopScreenSaverSkeletonPrivate));
#endif
}

static void
org_freedesktop_screen_saver_skeleton_iface_init (OrgFreedesktopScreenSaverIface *iface)
{
  iface->active_changed = _org_freedesktop_screen_saver_on_signal_active_changed;
}

/**
 * org_freedesktop_screen_saver_skeleton_new:
 *
 * Creates a skeleton object for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-ScreenSaver.top_of_page">org.freedesktop.ScreenSaver</link>.
 *
 * Returns: (transfer full) (type OrgFreedesktopScreenSaverSkeleton): The skeleton object.
 */
OrgFreedesktopScreenSaver *
org_freedesktop_screen_saver_skeleton_new (void)
{
  return ORG_FREEDESKTOP_SCREEN_SAVER (g_object_new (TYPE_ORG_FREEDESKTOP_SCREEN_SAVER_SKELETON, NULL));
}

/* ------------------------------------------------------------------------
 * Code for interface org.gnome.ScreenSaver
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:OrgGnomeScreenSaver
 * @title: OrgGnomeScreenSaver
 * @short_description: Generated C code for the org.gnome.ScreenSaver D-Bus interface
 *
 * This section contains code for working with the <link linkend="gdbus-interface-org-gnome-ScreenSaver.top_of_page">org.gnome.ScreenSaver</link> D-Bus interface in C.
 */

enum
{
  _ORG_GNOME_SCREEN_SAVER_ACTIVE_CHANGED,
};

static unsigned _ORG_GNOME_SCREEN_SAVER_SIGNALS[1] = { 0 };

/* ---- Introspection data for org.gnome.ScreenSaver ---- */

static const _ExtendedGDBusMethodInfo _org_gnome_screen_saver_method_info_lock =
{
  {
    -1,
    (gchar *) "Lock",
    NULL,
    NULL,
    NULL
  },
  "handle-lock",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_screen_saver_method_info_get_active_OUT_ARG_active =
{
  {
    -1,
    (gchar *) "active",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_screen_saver_method_info_get_active_OUT_ARG_pointers[] =
{
  &_org_gnome_screen_saver_method_info_get_active_OUT_ARG_active.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_screen_saver_method_info_get_active =
{
  {
    -1,
    (gchar *) "GetActive",
    NULL,
    (GDBusArgInfo **) &_org_gnome_screen_saver_method_info_get_active_OUT_ARG_pointers,
    NULL
  },
  "handle-get-active",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_screen_saver_method_info_set_active_IN_ARG_value =
{
  {
    -1,
    (gchar *) "value",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_screen_saver_method_info_set_active_IN_ARG_pointers[] =
{
  &_org_gnome_screen_saver_method_info_set_active_IN_ARG_value.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_screen_saver_method_info_set_active =
{
  {
    -1,
    (gchar *) "SetActive",
    (GDBusArgInfo **) &_org_gnome_screen_saver_method_info_set_active_IN_ARG_pointers,
    NULL,
    NULL
  },
  "handle-set-active",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_screen_saver_method_info_get_active_time_OUT_ARG_value =
{
  {
    -1,
    (gchar *) "value",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_screen_saver_method_info_get_active_time_OUT_ARG_pointers[] =
{
  &_org_gnome_screen_saver_method_info_get_active_time_OUT_ARG_value.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_screen_saver_method_info_get_active_time =
{
  {
    -1,
    (gchar *) "GetActiveTime",
    NULL,
    (GDBusArgInfo **) &_org_gnome_screen_saver_method_info_get_active_time_OUT_ARG_pointers,
    NULL
  },
  "handle-get-active-time",
  FALSE
};

static const GDBusMethodInfo * const _org_gnome_screen_saver_method_info_pointers[] =
{
  &_org_gnome_screen_saver_method_info_lock.parent_struct,
  &_org_gnome_screen_saver_method_info_get_active.parent_struct,
  &_org_gnome_screen_saver_method_info_set_active.parent_struct,
  &_org_gnome_screen_saver_method_info_get_active_time.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_gnome_screen_saver_signal_info_active_changed_ARG_new_value =
{
  {
    -1,
    (gchar *) "new_value",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_screen_saver_signal_info_active_changed_ARG_pointers[] =
{
  &_org_gnome_screen_saver_signal_info_active_changed_ARG_new_value.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _org_gnome_screen_saver_signal_info_active_changed =
{
  {
    -1,
    (gchar *) "ActiveChanged",
    (GDBusArgInfo **) &_org_gnome_screen_saver_signal_info_active_changed_ARG_pointers,
    NULL
  },
  "active-changed"
};

static const GDBusSignalInfo * const _org_gnome_screen_saver_signal_info_pointers[] =
{
  &_org_gnome_screen_saver_signal_info_active_changed.parent_struct,
  NULL
};

static const _ExtendedGDBusInterfaceInfo _org_gnome_screen_saver_interface_info =
{
  {
    -1,
    (gchar *) "org.gnome.ScreenSaver",
    (GDBusMethodInfo **) &_org_gnome_screen_saver_method_info_pointers,
    (GDBusSignalInfo **) &_org_gnome_screen_saver_signal_info_pointers,
    NULL,
    NULL
  },
  "org-gnome-screen-saver",
};


/**
 * org_gnome_screen_saver_interface_info:
 *
 * Gets a machine-readable description of the <link linkend="gdbus-interface-org-gnome-ScreenSaver.top_of_page">org.gnome.ScreenSaver</link> D-Bus interface.
 *
 * Returns: (transfer none): A #GDBusInterfaceInfo. Do not free.
 */
GDBusInterfaceInfo *
org_gnome_screen_saver_interface_info (void)
{
  return (GDBusInterfaceInfo *) &_org_gnome_screen_saver_interface_info.parent_struct;
}

/**
 * org_gnome_screen_saver_override_properties:
 * @klass: The class structure for a #GObject derived class.
 * @property_id_begin: The property id to assign to the first overridden property.
 *
 * Overrides all #GObject properties in the #OrgGnomeScreenSaver interface for a concrete class.
 * The properties are overridden in the order they are defined.
 *
 * Returns: The last property id.
 */
guint
org_gnome_screen_saver_override_properties (GObjectClass *klass G_GNUC_UNUSED, guint property_id_begin)
{
  return property_id_begin - 1;
}


inline static void
org_gnome_screen_saver_signal_marshal_active_changed (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__BOOLEAN (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_screen_saver_method_marshal_lock (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_screen_saver_method_marshal_get_active (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_screen_saver_method_marshal_set_active (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_BOOLEAN (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_screen_saver_method_marshal_get_active_time (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}


/**
 * OrgGnomeScreenSaver:
 *
 * Abstract interface type for the D-Bus interface <link linkend="gdbus-interface-org-gnome-ScreenSaver.top_of_page">org.gnome.ScreenSaver</link>.
 */

/**
 * OrgGnomeScreenSaverIface:
 * @parent_iface: The parent interface.
 * @handle_get_active: Handler for the #OrgGnomeScreenSaver::handle-get-active signal.
 * @handle_get_active_time: Handler for the #OrgGnomeScreenSaver::handle-get-active-time signal.
 * @handle_lock: Handler for the #OrgGnomeScreenSaver::handle-lock signal.
 * @handle_set_active: Handler for the #OrgGnomeScreenSaver::handle-set-active signal.
 * @active_changed: Handler for the #OrgGnomeScreenSaver::active-changed signal.
 *
 * Virtual table for the D-Bus interface <link linkend="gdbus-interface-org-gnome-ScreenSaver.top_of_page">org.gnome.ScreenSaver</link>.
 */

typedef OrgGnomeScreenSaverIface OrgGnomeScreenSaverInterface;
G_DEFINE_INTERFACE (OrgGnomeScreenSaver, org_gnome_screen_saver, G_TYPE_OBJECT)

static void
org_gnome_screen_saver_default_init (OrgGnomeScreenSaverIface *iface)
{
  /* GObject signals for incoming D-Bus method calls: */
  /**
   * OrgGnomeScreenSaver::handle-lock:
   * @object: A #OrgGnomeScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-ScreenSaver.Lock">Lock()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_screen_saver_complete_lock() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-lock",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeScreenSaverIface, handle_lock),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_screen_saver_method_marshal_lock,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * OrgGnomeScreenSaver::handle-get-active:
   * @object: A #OrgGnomeScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-ScreenSaver.GetActive">GetActive()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_screen_saver_complete_get_active() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-active",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeScreenSaverIface, handle_get_active),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_screen_saver_method_marshal_get_active,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * OrgGnomeScreenSaver::handle-set-active:
   * @object: A #OrgGnomeScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_value: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-ScreenSaver.SetActive">SetActive()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_screen_saver_complete_set_active() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-set-active",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeScreenSaverIface, handle_set_active),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_screen_saver_method_marshal_set_active,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_BOOLEAN);

  /**
   * OrgGnomeScreenSaver::handle-get-active-time:
   * @object: A #OrgGnomeScreenSaver.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-ScreenSaver.GetActiveTime">GetActiveTime()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_screen_saver_complete_get_active_time() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-active-time",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeScreenSaverIface, handle_get_active_time),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_screen_saver_method_marshal_get_active_time,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /* GObject signals for received D-Bus signals: */
  /**
   * OrgGnomeScreenSaver::active-changed:
   * @object: A #OrgGnomeScreenSaver.
   * @arg_new_value: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-org-gnome-ScreenSaver.ActiveChanged">"ActiveChanged"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _ORG_GNOME_SCREEN_SAVER_SIGNALS[_ORG_GNOME_SCREEN_SAVER_ACTIVE_CHANGED] =
    g_signal_new ("active-changed",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (OrgGnomeScreenSaverIface, active_changed),
      NULL,
      NULL,
      org_gnome_screen_saver_signal_marshal_active_changed,
      G_TYPE_NONE,
      1, G_TYPE_BOOLEAN);

}

/**
 * org_gnome_screen_saver_emit_active_changed:
 * @object: A #OrgGnomeScreenSaver.
 * @arg_new_value: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-org-gnome-ScreenSaver.ActiveChanged">"ActiveChanged"</link> D-Bus signal.
 */
void
org_gnome_screen_saver_emit_active_changed (
    OrgGnomeScreenSaver *object,
    gboolean arg_new_value)
{
  g_signal_emit (object, _ORG_GNOME_SCREEN_SAVER_SIGNALS[_ORG_GNOME_SCREEN_SAVER_ACTIVE_CHANGED], 0, arg_new_value);
}

/**
 * org_gnome_screen_saver_call_lock:
 * @proxy: A #OrgGnomeScreenSaverProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-ScreenSaver.Lock">Lock()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_screen_saver_call_lock_finish() to get the result of the operation.
 *
 * See org_gnome_screen_saver_call_lock_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_screen_saver_call_lock (
    OrgGnomeScreenSaver *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "Lock",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_screen_saver_call_lock_finish:
 * @proxy: A #OrgGnomeScreenSaverProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_screen_saver_call_lock().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_screen_saver_call_lock().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_screen_saver_call_lock_finish (
    OrgGnomeScreenSaver *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_screen_saver_call_lock_sync:
 * @proxy: A #OrgGnomeScreenSaverProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-ScreenSaver.Lock">Lock()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_screen_saver_call_lock() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_screen_saver_call_lock_sync (
    OrgGnomeScreenSaver *proxy,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "Lock",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_screen_saver_call_get_active:
 * @proxy: A #OrgGnomeScreenSaverProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-ScreenSaver.GetActive">GetActive()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_screen_saver_call_get_active_finish() to get the result of the operation.
 *
 * See org_gnome_screen_saver_call_get_active_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_screen_saver_call_get_active (
    OrgGnomeScreenSaver *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetActive",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_screen_saver_call_get_active_finish:
 * @proxy: A #OrgGnomeScreenSaverProxy.
 * @out_active: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_screen_saver_call_get_active().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_screen_saver_call_get_active().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_screen_saver_call_get_active_finish (
    OrgGnomeScreenSaver *proxy,
    gboolean *out_active,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_active);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_screen_saver_call_get_active_sync:
 * @proxy: A #OrgGnomeScreenSaverProxy.
 * @out_active: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-ScreenSaver.GetActive">GetActive()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_screen_saver_call_get_active() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_screen_saver_call_get_active_sync (
    OrgGnomeScreenSaver *proxy,
    gboolean *out_active,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetActive",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_active);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_screen_saver_call_set_active:
 * @proxy: A #OrgGnomeScreenSaverProxy.
 * @arg_value: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-ScreenSaver.SetActive">SetActive()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_screen_saver_call_set_active_finish() to get the result of the operation.
 *
 * See org_gnome_screen_saver_call_set_active_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_screen_saver_call_set_active (
    OrgGnomeScreenSaver *proxy,
    gboolean arg_value,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "SetActive",
    g_variant_new ("(b)",
                   arg_value),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_screen_saver_call_set_active_finish:
 * @proxy: A #OrgGnomeScreenSaverProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_screen_saver_call_set_active().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_screen_saver_call_set_active().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_screen_saver_call_set_active_finish (
    OrgGnomeScreenSaver *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_screen_saver_call_set_active_sync:
 * @proxy: A #OrgGnomeScreenSaverProxy.
 * @arg_value: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-ScreenSaver.SetActive">SetActive()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_screen_saver_call_set_active() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_screen_saver_call_set_active_sync (
    OrgGnomeScreenSaver *proxy,
    gboolean arg_value,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "SetActive",
    g_variant_new ("(b)",
                   arg_value),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_screen_saver_call_get_active_time:
 * @proxy: A #OrgGnomeScreenSaverProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-ScreenSaver.GetActiveTime">GetActiveTime()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_screen_saver_call_get_active_time_finish() to get the result of the operation.
 *
 * See org_gnome_screen_saver_call_get_active_time_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_screen_saver_call_get_active_time (
    OrgGnomeScreenSaver *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetActiveTime",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_screen_saver_call_get_active_time_finish:
 * @proxy: A #OrgGnomeScreenSaverProxy.
 * @out_value: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_screen_saver_call_get_active_time().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_screen_saver_call_get_active_time().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_screen_saver_call_get_active_time_finish (
    OrgGnomeScreenSaver *proxy,
    guint *out_value,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_value);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_screen_saver_call_get_active_time_sync:
 * @proxy: A #OrgGnomeScreenSaverProxy.
 * @out_value: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-ScreenSaver.GetActiveTime">GetActiveTime()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_screen_saver_call_get_active_time() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_screen_saver_call_get_active_time_sync (
    OrgGnomeScreenSaver *proxy,
    guint *out_value,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetActiveTime",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_value);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_screen_saver_complete_lock:
 * @object: A #OrgGnomeScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-ScreenSaver.Lock">Lock()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_screen_saver_complete_lock (
    OrgGnomeScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_gnome_screen_saver_complete_get_active:
 * @object: A #OrgGnomeScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @active: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-ScreenSaver.GetActive">GetActive()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_screen_saver_complete_get_active (
    OrgGnomeScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    gboolean active)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(b)",
                   active));
}

/**
 * org_gnome_screen_saver_complete_set_active:
 * @object: A #OrgGnomeScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-ScreenSaver.SetActive">SetActive()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_screen_saver_complete_set_active (
    OrgGnomeScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_gnome_screen_saver_complete_get_active_time:
 * @object: A #OrgGnomeScreenSaver.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @value: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-ScreenSaver.GetActiveTime">GetActiveTime()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_screen_saver_complete_get_active_time (
    OrgGnomeScreenSaver *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    guint value)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(u)",
                   value));
}

/* ------------------------------------------------------------------------ */

/**
 * OrgGnomeScreenSaverProxy:
 *
 * The #OrgGnomeScreenSaverProxy structure contains only private data and should only be accessed using the provided API.
 */

/**
 * OrgGnomeScreenSaverProxyClass:
 * @parent_class: The parent class.
 *
 * Class structure for #OrgGnomeScreenSaverProxy.
 */

struct _OrgGnomeScreenSaverProxyPrivate
{
  GData *qdata;
};

static void org_gnome_screen_saver_proxy_iface_init (OrgGnomeScreenSaverIface *iface);

#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (OrgGnomeScreenSaverProxy, org_gnome_screen_saver_proxy, G_TYPE_DBUS_PROXY,
                         G_ADD_PRIVATE (OrgGnomeScreenSaverProxy)
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_GNOME_SCREEN_SAVER, org_gnome_screen_saver_proxy_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (OrgGnomeScreenSaverProxy, org_gnome_screen_saver_proxy, G_TYPE_DBUS_PROXY,
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_GNOME_SCREEN_SAVER, org_gnome_screen_saver_proxy_iface_init))

#endif
static void
org_gnome_screen_saver_proxy_finalize (GObject *object)
{
  OrgGnomeScreenSaverProxy *proxy = ORG_GNOME_SCREEN_SAVER_PROXY (object);
  g_datalist_clear (&proxy->priv->qdata);
  G_OBJECT_CLASS (org_gnome_screen_saver_proxy_parent_class)->finalize (object);
}

static void
org_gnome_screen_saver_proxy_get_property (GObject      *object G_GNUC_UNUSED,
  guint         prop_id G_GNUC_UNUSED,
  GValue       *value G_GNUC_UNUSED,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
}

static void
org_gnome_screen_saver_proxy_set_property (GObject      *object G_GNUC_UNUSED,
  guint         prop_id G_GNUC_UNUSED,
  const GValue *value G_GNUC_UNUSED,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
}

static void
org_gnome_screen_saver_proxy_g_signal (GDBusProxy *proxy,
  const gchar *sender_name G_GNUC_UNUSED,
  const gchar *signal_name,
  GVariant *parameters)
{
  _ExtendedGDBusSignalInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  gsize n;
  guint signal_id;
  info = (_ExtendedGDBusSignalInfo *) g_dbus_interface_info_lookup_signal ((GDBusInterfaceInfo *) &_org_gnome_screen_saver_interface_info.parent_struct, signal_name);
  if (info == NULL)
    return;
  num_params = g_variant_n_children (parameters);
  paramv = g_new0 (GValue, num_params + 1);
  g_value_init (&paramv[0], TYPE_ORG_GNOME_SCREEN_SAVER);
  g_value_set_object (&paramv[0], proxy);
  g_variant_iter_init (&iter, parameters);
  n = 1;
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.args[n - 1];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_ORG_GNOME_SCREEN_SAVER);
  g_signal_emitv (paramv, signal_id, 0, NULL);
  for (n = 0; n < num_params + 1; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static void
org_gnome_screen_saver_proxy_g_properties_changed (GDBusProxy *_proxy,
  GVariant *changed_properties,
  const gchar *const *invalidated_properties)
{
  OrgGnomeScreenSaverProxy *proxy = ORG_GNOME_SCREEN_SAVER_PROXY (_proxy);
  guint n;
  const gchar *key;
  GVariantIter *iter;
  _ExtendedGDBusPropertyInfo *info;
  g_variant_get (changed_properties, "a{sv}", &iter);
  while (g_variant_iter_next (iter, "{&sv}", &key, NULL))
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_gnome_screen_saver_interface_info.parent_struct, key);
      g_datalist_remove_data (&proxy->priv->qdata, key);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
  g_variant_iter_free (iter);
  for (n = 0; invalidated_properties[n] != NULL; n++)
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_gnome_screen_saver_interface_info.parent_struct, invalidated_properties[n]);
      g_datalist_remove_data (&proxy->priv->qdata, invalidated_properties[n]);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
}

static void
org_gnome_screen_saver_proxy_init (OrgGnomeScreenSaverProxy *proxy)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  proxy->priv = org_gnome_screen_saver_proxy_get_instance_private (proxy);
#else
  proxy->priv = G_TYPE_INSTANCE_GET_PRIVATE (proxy, TYPE_ORG_GNOME_SCREEN_SAVER_PROXY, OrgGnomeScreenSaverProxyPrivate);
#endif

  g_dbus_proxy_set_interface_info (G_DBUS_PROXY (proxy), org_gnome_screen_saver_interface_info ());
}

static void
org_gnome_screen_saver_proxy_class_init (OrgGnomeScreenSaverProxyClass *klass)
{
  GObjectClass *gobject_class;
  GDBusProxyClass *proxy_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize     = org_gnome_screen_saver_proxy_finalize;
  gobject_class->get_property = org_gnome_screen_saver_proxy_get_property;
  gobject_class->set_property = org_gnome_screen_saver_proxy_set_property;

  proxy_class = G_DBUS_PROXY_CLASS (klass);
  proxy_class->g_signal = org_gnome_screen_saver_proxy_g_signal;
  proxy_class->g_properties_changed = org_gnome_screen_saver_proxy_g_properties_changed;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (OrgGnomeScreenSaverProxyPrivate));
#endif
}

static void
org_gnome_screen_saver_proxy_iface_init (OrgGnomeScreenSaverIface *iface G_GNUC_UNUSED)
{
}

/**
 * org_gnome_screen_saver_proxy_new:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-gnome-ScreenSaver.top_of_page">org.gnome.ScreenSaver</link>. See g_dbus_proxy_new() for more details.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_screen_saver_proxy_new_finish() to get the result of the operation.
 *
 * See org_gnome_screen_saver_proxy_new_sync() for the synchronous, blocking version of this constructor.
 */
void
org_gnome_screen_saver_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_ORG_GNOME_SCREEN_SAVER_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.gnome.ScreenSaver", NULL);
}

/**
 * org_gnome_screen_saver_proxy_new_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_screen_saver_proxy_new().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with org_gnome_screen_saver_proxy_new().
 *
 * Returns: (transfer full) (type OrgGnomeScreenSaverProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgGnomeScreenSaver *
org_gnome_screen_saver_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return ORG_GNOME_SCREEN_SAVER (ret);
  else
    return NULL;
}

/**
 * org_gnome_screen_saver_proxy_new_sync:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Synchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-gnome-ScreenSaver.top_of_page">org.gnome.ScreenSaver</link>. See g_dbus_proxy_new_sync() for more details.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See org_gnome_screen_saver_proxy_new() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type OrgGnomeScreenSaverProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgGnomeScreenSaver *
org_gnome_screen_saver_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_ORG_GNOME_SCREEN_SAVER_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.gnome.ScreenSaver", NULL);
  if (ret != NULL)
    return ORG_GNOME_SCREEN_SAVER (ret);
  else
    return NULL;
}


/**
 * org_gnome_screen_saver_proxy_new_for_bus:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Like org_gnome_screen_saver_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_screen_saver_proxy_new_for_bus_finish() to get the result of the operation.
 *
 * See org_gnome_screen_saver_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
 */
void
org_gnome_screen_saver_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_ORG_GNOME_SCREEN_SAVER_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.gnome.ScreenSaver", NULL);
}

/**
 * org_gnome_screen_saver_proxy_new_for_bus_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_screen_saver_proxy_new_for_bus().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with org_gnome_screen_saver_proxy_new_for_bus().
 *
 * Returns: (transfer full) (type OrgGnomeScreenSaverProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgGnomeScreenSaver *
org_gnome_screen_saver_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return ORG_GNOME_SCREEN_SAVER (ret);
  else
    return NULL;
}

/**
 * org_gnome_screen_saver_proxy_new_for_bus_sync:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Like org_gnome_screen_saver_proxy_new_sync() but takes a #GBusType instead of a #GDBusConnection.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See org_gnome_screen_saver_proxy_new_for_bus() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type OrgGnomeScreenSaverProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgGnomeScreenSaver *
org_gnome_screen_saver_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_ORG_GNOME_SCREEN_SAVER_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.gnome.ScreenSaver", NULL);
  if (ret != NULL)
    return ORG_GNOME_SCREEN_SAVER (ret);
  else
    return NULL;
}


/* ------------------------------------------------------------------------ */

/**
 * OrgGnomeScreenSaverSkeleton:
 *
 * The #OrgGnomeScreenSaverSkeleton structure contains only private data and should only be accessed using the provided API.
 */

/**
 * OrgGnomeScreenSaverSkeletonClass:
 * @parent_class: The parent class.
 *
 * Class structure for #OrgGnomeScreenSaverSkeleton.
 */

struct _OrgGnomeScreenSaverSkeletonPrivate
{
  GValue *properties;
  GList *changed_properties;
  GSource *changed_properties_idle_source;
  GMainContext *context;
  GMutex lock;
};

static void
_org_gnome_screen_saver_skeleton_handle_method_call (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name,
  const gchar *method_name,
  GVariant *parameters,
  GDBusMethodInvocation *invocation,
  gpointer user_data)
{
  OrgGnomeScreenSaverSkeleton *skeleton = ORG_GNOME_SCREEN_SAVER_SKELETON (user_data);
  _ExtendedGDBusMethodInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  guint num_extra;
  gsize n;
  guint signal_id;
  GValue return_value = G_VALUE_INIT;
  info = (_ExtendedGDBusMethodInfo *) g_dbus_method_invocation_get_method_info (invocation);
  g_assert (info != NULL);
  num_params = g_variant_n_children (parameters);
  num_extra = info->pass_fdlist ? 3 : 2;  paramv = g_new0 (GValue, num_params + num_extra);
  n = 0;
  g_value_init (&paramv[n], TYPE_ORG_GNOME_SCREEN_SAVER);
  g_value_set_object (&paramv[n++], skeleton);
  g_value_init (&paramv[n], G_TYPE_DBUS_METHOD_INVOCATION);
  g_value_set_object (&paramv[n++], invocation);
  if (info->pass_fdlist)
    {
#ifdef G_OS_UNIX
      g_value_init (&paramv[n], G_TYPE_UNIX_FD_LIST);
      g_value_set_object (&paramv[n++], g_dbus_message_get_unix_fd_list (g_dbus_method_invocation_get_message (invocation)));
#else
      g_assert_not_reached ();
#endif
    }
  g_variant_iter_init (&iter, parameters);
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.in_args[n - num_extra];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_ORG_GNOME_SCREEN_SAVER);
  g_value_init (&return_value, G_TYPE_BOOLEAN);
  g_signal_emitv (paramv, signal_id, 0, &return_value);
  if (!g_value_get_boolean (&return_value))
    g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD, "Method %s is not implemented on interface %s", method_name, interface_name);
  g_value_unset (&return_value);
  for (n = 0; n < num_params + num_extra; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static GVariant *
_org_gnome_screen_saver_skeleton_handle_get_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GError **error,
  gpointer user_data)
{
  OrgGnomeScreenSaverSkeleton *skeleton = ORG_GNOME_SCREEN_SAVER_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  GVariant *ret;
  ret = NULL;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_gnome_screen_saver_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      g_value_init (&value, pspec->value_type);
      g_object_get_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      ret = g_dbus_gvalue_to_gvariant (&value, G_VARIANT_TYPE (info->parent_struct.signature));
      g_value_unset (&value);
    }
  return ret;
}

static gboolean
_org_gnome_screen_saver_skeleton_handle_set_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GVariant *variant,
  GError **error,
  gpointer user_data)
{
  OrgGnomeScreenSaverSkeleton *skeleton = ORG_GNOME_SCREEN_SAVER_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  gboolean ret;
  ret = FALSE;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_gnome_screen_saver_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      if (info->use_gvariant)
        g_value_set_variant (&value, variant);
      else
        g_dbus_gvariant_to_gvalue (variant, &value);
      g_object_set_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      g_value_unset (&value);
      ret = TRUE;
    }
  return ret;
}

static const GDBusInterfaceVTable _org_gnome_screen_saver_skeleton_vtable =
{
  _org_gnome_screen_saver_skeleton_handle_method_call,
  _org_gnome_screen_saver_skeleton_handle_get_property,
  _org_gnome_screen_saver_skeleton_handle_set_property,
  {NULL}
};

static GDBusInterfaceInfo *
org_gnome_screen_saver_skeleton_dbus_interface_get_info (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return org_gnome_screen_saver_interface_info ();
}

static GDBusInterfaceVTable *
org_gnome_screen_saver_skeleton_dbus_interface_get_vtable (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return (GDBusInterfaceVTable *) &_org_gnome_screen_saver_skeleton_vtable;
}

static GVariant *
org_gnome_screen_saver_skeleton_dbus_interface_get_properties (GDBusInterfaceSkeleton *_skeleton)
{
  OrgGnomeScreenSaverSkeleton *skeleton = ORG_GNOME_SCREEN_SAVER_SKELETON (_skeleton);

  GVariantBuilder builder;
  guint n;
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
#else
  g_variant_builder_init(&builder, G_VARIANT_TYPE ("a{sv}"));
#endif
  if (_org_gnome_screen_saver_interface_info.parent_struct.properties == NULL)
    goto out;
  for (n = 0; _org_gnome_screen_saver_interface_info.parent_struct.properties[n] != NULL; n++)
    {
      GDBusPropertyInfo *info = _org_gnome_screen_saver_interface_info.parent_struct.properties[n];
      if (info->flags & G_DBUS_PROPERTY_INFO_FLAGS_READABLE)
        {
          GVariant *value;
          value = _org_gnome_screen_saver_skeleton_handle_get_property (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)), NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.gnome.ScreenSaver", info->name, NULL, skeleton);
          if (value != NULL)
            {
              g_variant_take_ref (value);
              g_variant_builder_add (&builder, "{sv}", info->name, value);
              g_variant_unref (value);
            }
        }
    }
out:
  return g_variant_builder_end (&builder);
}

static void
org_gnome_screen_saver_skeleton_dbus_interface_flush (GDBusInterfaceSkeleton *_skeleton G_GNUC_UNUSED)
{
}

static void
_org_gnome_screen_saver_on_signal_active_changed (
    OrgGnomeScreenSaver *object,
    gboolean arg_new_value)
{
  OrgGnomeScreenSaverSkeleton *skeleton = ORG_GNOME_SCREEN_SAVER_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(b)",
                   arg_new_value));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.gnome.ScreenSaver", "ActiveChanged",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void org_gnome_screen_saver_skeleton_iface_init (OrgGnomeScreenSaverIface *iface);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (OrgGnomeScreenSaverSkeleton, org_gnome_screen_saver_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_ADD_PRIVATE (OrgGnomeScreenSaverSkeleton)
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_GNOME_SCREEN_SAVER, org_gnome_screen_saver_skeleton_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (OrgGnomeScreenSaverSkeleton, org_gnome_screen_saver_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_GNOME_SCREEN_SAVER, org_gnome_screen_saver_skeleton_iface_init))

#endif
static void
org_gnome_screen_saver_skeleton_finalize (GObject *object)
{
  OrgGnomeScreenSaverSkeleton *skeleton = ORG_GNOME_SCREEN_SAVER_SKELETON (object);
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
  if (skeleton->priv->changed_properties_idle_source != NULL)
    g_source_destroy (skeleton->priv->changed_properties_idle_source);
  g_main_context_unref (skeleton->priv->context);
  g_mutex_clear (&skeleton->priv->lock);
  G_OBJECT_CLASS (org_gnome_screen_saver_skeleton_parent_class)->finalize (object);
}

static void
org_gnome_screen_saver_skeleton_init (OrgGnomeScreenSaverSkeleton *skeleton)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  skeleton->priv = org_gnome_screen_saver_skeleton_get_instance_private (skeleton);
#else
  skeleton->priv = G_TYPE_INSTANCE_GET_PRIVATE (skeleton, TYPE_ORG_GNOME_SCREEN_SAVER_SKELETON, OrgGnomeScreenSaverSkeletonPrivate);
#endif

  g_mutex_init (&skeleton->priv->lock);
  skeleton->priv->context = g_main_context_ref_thread_default ();
}

static void
org_gnome_screen_saver_skeleton_class_init (OrgGnomeScreenSaverSkeletonClass *klass)
{
  GObjectClass *gobject_class;
  GDBusInterfaceSkeletonClass *skeleton_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize = org_gnome_screen_saver_skeleton_finalize;

  skeleton_class = G_DBUS_INTERFACE_SKELETON_CLASS (klass);
  skeleton_class->get_info = org_gnome_screen_saver_skeleton_dbus_interface_get_info;
  skeleton_class->get_properties = org_gnome_screen_saver_skeleton_dbus_interface_get_properties;
  skeleton_class->flush = org_gnome_screen_saver_skeleton_dbus_interface_flush;
  skeleton_class->get_vtable = org_gnome_screen_saver_skeleton_dbus_interface_get_vtable;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (OrgGnomeScreenSaverSkeletonPrivate));
#endif
}

static void
org_gnome_screen_saver_skeleton_iface_init (OrgGnomeScreenSaverIface *iface)
{
  iface->active_changed = _org_gnome_screen_saver_on_signal_active_changed;
}

/**
 * org_gnome_screen_saver_skeleton_new:
 *
 * Creates a skeleton object for the D-Bus interface <link linkend="gdbus-interface-org-gnome-ScreenSaver.top_of_page">org.gnome.ScreenSaver</link>.
 *
 * Returns: (transfer full) (type OrgGnomeScreenSaverSkeleton): The skeleton object.
 */
OrgGnomeScreenSaver *
org_gnome_screen_saver_skeleton_new (void)
{
  return ORG_GNOME_SCREEN_SAVER (g_object_new (TYPE_ORG_GNOME_SCREEN_SAVER_SKELETON, NULL));
}

/* ------------------------------------------------------------------------
 * Code for interface org.gnome.SessionManager
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:OrgGnomeSessionManager
 * @title: OrgGnomeSessionManager
 * @short_description: Generated C code for the org.gnome.SessionManager D-Bus interface
 *
 * This section contains code for working with the <link linkend="gdbus-interface-org-gnome-SessionManager.top_of_page">org.gnome.SessionManager</link> D-Bus interface in C.
 */

enum
{
  _ORG_GNOME_SESSION_MANAGER_CLIENT_ADDED,
  _ORG_GNOME_SESSION_MANAGER_CLIENT_REMOVED,
  _ORG_GNOME_SESSION_MANAGER_INHIBITOR_ADDED,
  _ORG_GNOME_SESSION_MANAGER_INHIBITOR_REMOVED,
  _ORG_GNOME_SESSION_MANAGER_SESSION_RUNNING,
  _ORG_GNOME_SESSION_MANAGER_SESSION_OVER,
};

static unsigned _ORG_GNOME_SESSION_MANAGER_SIGNALS[6] = { 0 };

/* ---- Introspection data for org.gnome.SessionManager ---- */

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_setenv_IN_ARG_variable =
{
  {
    -1,
    (gchar *) "variable",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_setenv_IN_ARG_value =
{
  {
    -1,
    (gchar *) "value",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_setenv_IN_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_setenv_IN_ARG_variable.parent_struct,
  &_org_gnome_session_manager_method_info_setenv_IN_ARG_value.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_setenv =
{
  {
    -1,
    (gchar *) "Setenv",
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_setenv_IN_ARG_pointers,
    NULL,
    NULL
  },
  "handle-setenv",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_get_locale_IN_ARG_category =
{
  {
    -1,
    (gchar *) "category",
    (gchar *) "i",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_get_locale_IN_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_get_locale_IN_ARG_category.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_get_locale_OUT_ARG_value =
{
  {
    -1,
    (gchar *) "value",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_get_locale_OUT_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_get_locale_OUT_ARG_value.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_get_locale =
{
  {
    -1,
    (gchar *) "GetLocale",
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_get_locale_IN_ARG_pointers,
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_get_locale_OUT_ARG_pointers,
    NULL
  },
  "handle-get-locale",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_initialization_error_IN_ARG_message =
{
  {
    -1,
    (gchar *) "message",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_initialization_error_IN_ARG_fatal =
{
  {
    -1,
    (gchar *) "fatal",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_initialization_error_IN_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_initialization_error_IN_ARG_message.parent_struct,
  &_org_gnome_session_manager_method_info_initialization_error_IN_ARG_fatal.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_initialization_error =
{
  {
    -1,
    (gchar *) "InitializationError",
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_initialization_error_IN_ARG_pointers,
    NULL,
    NULL
  },
  "handle-initialization-error",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_register_client_IN_ARG_app_id =
{
  {
    -1,
    (gchar *) "app_id",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_register_client_IN_ARG_client_startup_id =
{
  {
    -1,
    (gchar *) "client_startup_id",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_register_client_IN_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_register_client_IN_ARG_app_id.parent_struct,
  &_org_gnome_session_manager_method_info_register_client_IN_ARG_client_startup_id.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_register_client_OUT_ARG_client_id =
{
  {
    -1,
    (gchar *) "client_id",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_register_client_OUT_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_register_client_OUT_ARG_client_id.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_gnome_session_manager_method_register_client_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_gnome_session_manager_method_register_client_annotation_info_pointers[] =
{
  &_org_gnome_session_manager_method_register_client_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_register_client =
{
  {
    -1,
    (gchar *) "RegisterClient",
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_register_client_IN_ARG_pointers,
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_register_client_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_org_gnome_session_manager_method_register_client_annotation_info_pointers
  },
  "handle-register-client",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_unregister_client_IN_ARG_client_id =
{
  {
    -1,
    (gchar *) "client_id",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_unregister_client_IN_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_unregister_client_IN_ARG_client_id.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_gnome_session_manager_method_unregister_client_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_gnome_session_manager_method_unregister_client_annotation_info_pointers[] =
{
  &_org_gnome_session_manager_method_unregister_client_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_unregister_client =
{
  {
    -1,
    (gchar *) "UnregisterClient",
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_unregister_client_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_gnome_session_manager_method_unregister_client_annotation_info_pointers
  },
  "handle-unregister-client",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_inhibit_IN_ARG_app_id =
{
  {
    -1,
    (gchar *) "app_id",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_inhibit_IN_ARG_toplevel_xid =
{
  {
    -1,
    (gchar *) "toplevel_xid",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_inhibit_IN_ARG_reason =
{
  {
    -1,
    (gchar *) "reason",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_inhibit_IN_ARG_flags =
{
  {
    -1,
    (gchar *) "flags",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_inhibit_IN_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_inhibit_IN_ARG_app_id.parent_struct,
  &_org_gnome_session_manager_method_info_inhibit_IN_ARG_toplevel_xid.parent_struct,
  &_org_gnome_session_manager_method_info_inhibit_IN_ARG_reason.parent_struct,
  &_org_gnome_session_manager_method_info_inhibit_IN_ARG_flags.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_inhibit_OUT_ARG_inhibit_cookie =
{
  {
    -1,
    (gchar *) "inhibit_cookie",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_inhibit_OUT_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_inhibit_OUT_ARG_inhibit_cookie.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_gnome_session_manager_method_inhibit_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_gnome_session_manager_method_inhibit_annotation_info_pointers[] =
{
  &_org_gnome_session_manager_method_inhibit_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_inhibit =
{
  {
    -1,
    (gchar *) "Inhibit",
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_inhibit_IN_ARG_pointers,
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_inhibit_OUT_ARG_pointers,
    (GDBusAnnotationInfo **) &_org_gnome_session_manager_method_inhibit_annotation_info_pointers
  },
  "handle-inhibit",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_uninhibit_IN_ARG_inhibit_cookie =
{
  {
    -1,
    (gchar *) "inhibit_cookie",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_uninhibit_IN_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_uninhibit_IN_ARG_inhibit_cookie.parent_struct,
  NULL
};

static const GDBusAnnotationInfo _org_gnome_session_manager_method_uninhibit_annotation_info_0 =
{
  -1,
  (gchar *) "org.freedesktop.DBus.GLib.Async",
  (gchar *) "",
  NULL
};

static const GDBusAnnotationInfo * const _org_gnome_session_manager_method_uninhibit_annotation_info_pointers[] =
{
  &_org_gnome_session_manager_method_uninhibit_annotation_info_0,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_uninhibit =
{
  {
    -1,
    (gchar *) "Uninhibit",
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_uninhibit_IN_ARG_pointers,
    NULL,
    (GDBusAnnotationInfo **) &_org_gnome_session_manager_method_uninhibit_annotation_info_pointers
  },
  "handle-uninhibit",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_is_inhibited_IN_ARG_flags =
{
  {
    -1,
    (gchar *) "flags",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_is_inhibited_IN_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_is_inhibited_IN_ARG_flags.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_is_inhibited_OUT_ARG_is_inhibited =
{
  {
    -1,
    (gchar *) "is_inhibited",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_is_inhibited_OUT_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_is_inhibited_OUT_ARG_is_inhibited.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_is_inhibited =
{
  {
    -1,
    (gchar *) "IsInhibited",
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_is_inhibited_IN_ARG_pointers,
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_is_inhibited_OUT_ARG_pointers,
    NULL
  },
  "handle-is-inhibited",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_get_clients_OUT_ARG_clients =
{
  {
    -1,
    (gchar *) "clients",
    (gchar *) "ao",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_get_clients_OUT_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_get_clients_OUT_ARG_clients.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_get_clients =
{
  {
    -1,
    (gchar *) "GetClients",
    NULL,
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_get_clients_OUT_ARG_pointers,
    NULL
  },
  "handle-get-clients",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_get_inhibitors_OUT_ARG_inhibitors =
{
  {
    -1,
    (gchar *) "inhibitors",
    (gchar *) "ao",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_get_inhibitors_OUT_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_get_inhibitors_OUT_ARG_inhibitors.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_get_inhibitors =
{
  {
    -1,
    (gchar *) "GetInhibitors",
    NULL,
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_get_inhibitors_OUT_ARG_pointers,
    NULL
  },
  "handle-get-inhibitors",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_is_autostart_condition_handled_IN_ARG_condition =
{
  {
    -1,
    (gchar *) "condition",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_is_autostart_condition_handled_IN_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_is_autostart_condition_handled_IN_ARG_condition.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_is_autostart_condition_handled_OUT_ARG_handled =
{
  {
    -1,
    (gchar *) "handled",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_is_autostart_condition_handled_OUT_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_is_autostart_condition_handled_OUT_ARG_handled.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_is_autostart_condition_handled =
{
  {
    -1,
    (gchar *) "IsAutostartConditionHandled",
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_is_autostart_condition_handled_IN_ARG_pointers,
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_is_autostart_condition_handled_OUT_ARG_pointers,
    NULL
  },
  "handle-is-autostart-condition-handled",
  FALSE
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_shutdown =
{
  {
    -1,
    (gchar *) "Shutdown",
    NULL,
    NULL,
    NULL
  },
  "handle-shutdown",
  FALSE
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_reboot =
{
  {
    -1,
    (gchar *) "Reboot",
    NULL,
    NULL,
    NULL
  },
  "handle-reboot",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_can_shutdown_OUT_ARG_is_available =
{
  {
    -1,
    (gchar *) "is_available",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_can_shutdown_OUT_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_can_shutdown_OUT_ARG_is_available.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_can_shutdown =
{
  {
    -1,
    (gchar *) "CanShutdown",
    NULL,
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_can_shutdown_OUT_ARG_pointers,
    NULL
  },
  "handle-can-shutdown",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_logout_IN_ARG_mode =
{
  {
    -1,
    (gchar *) "mode",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_logout_IN_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_logout_IN_ARG_mode.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_logout =
{
  {
    -1,
    (gchar *) "Logout",
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_logout_IN_ARG_pointers,
    NULL,
    NULL
  },
  "handle-logout",
  FALSE
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_method_info_is_session_running_OUT_ARG_running =
{
  {
    -1,
    (gchar *) "running",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_method_info_is_session_running_OUT_ARG_pointers[] =
{
  &_org_gnome_session_manager_method_info_is_session_running_OUT_ARG_running.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _org_gnome_session_manager_method_info_is_session_running =
{
  {
    -1,
    (gchar *) "IsSessionRunning",
    NULL,
    (GDBusArgInfo **) &_org_gnome_session_manager_method_info_is_session_running_OUT_ARG_pointers,
    NULL
  },
  "handle-is-session-running",
  FALSE
};

static const GDBusMethodInfo * const _org_gnome_session_manager_method_info_pointers[] =
{
  &_org_gnome_session_manager_method_info_setenv.parent_struct,
  &_org_gnome_session_manager_method_info_get_locale.parent_struct,
  &_org_gnome_session_manager_method_info_initialization_error.parent_struct,
  &_org_gnome_session_manager_method_info_register_client.parent_struct,
  &_org_gnome_session_manager_method_info_unregister_client.parent_struct,
  &_org_gnome_session_manager_method_info_inhibit.parent_struct,
  &_org_gnome_session_manager_method_info_uninhibit.parent_struct,
  &_org_gnome_session_manager_method_info_is_inhibited.parent_struct,
  &_org_gnome_session_manager_method_info_get_clients.parent_struct,
  &_org_gnome_session_manager_method_info_get_inhibitors.parent_struct,
  &_org_gnome_session_manager_method_info_is_autostart_condition_handled.parent_struct,
  &_org_gnome_session_manager_method_info_shutdown.parent_struct,
  &_org_gnome_session_manager_method_info_reboot.parent_struct,
  &_org_gnome_session_manager_method_info_can_shutdown.parent_struct,
  &_org_gnome_session_manager_method_info_logout.parent_struct,
  &_org_gnome_session_manager_method_info_is_session_running.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_signal_info_client_added_ARG_id =
{
  {
    -1,
    (gchar *) "id",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_signal_info_client_added_ARG_pointers[] =
{
  &_org_gnome_session_manager_signal_info_client_added_ARG_id.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _org_gnome_session_manager_signal_info_client_added =
{
  {
    -1,
    (gchar *) "ClientAdded",
    (GDBusArgInfo **) &_org_gnome_session_manager_signal_info_client_added_ARG_pointers,
    NULL
  },
  "client-added"
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_signal_info_client_removed_ARG_id =
{
  {
    -1,
    (gchar *) "id",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_signal_info_client_removed_ARG_pointers[] =
{
  &_org_gnome_session_manager_signal_info_client_removed_ARG_id.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _org_gnome_session_manager_signal_info_client_removed =
{
  {
    -1,
    (gchar *) "ClientRemoved",
    (GDBusArgInfo **) &_org_gnome_session_manager_signal_info_client_removed_ARG_pointers,
    NULL
  },
  "client-removed"
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_signal_info_inhibitor_added_ARG_id =
{
  {
    -1,
    (gchar *) "id",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_signal_info_inhibitor_added_ARG_pointers[] =
{
  &_org_gnome_session_manager_signal_info_inhibitor_added_ARG_id.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _org_gnome_session_manager_signal_info_inhibitor_added =
{
  {
    -1,
    (gchar *) "InhibitorAdded",
    (GDBusArgInfo **) &_org_gnome_session_manager_signal_info_inhibitor_added_ARG_pointers,
    NULL
  },
  "inhibitor-added"
};

static const _ExtendedGDBusArgInfo _org_gnome_session_manager_signal_info_inhibitor_removed_ARG_id =
{
  {
    -1,
    (gchar *) "id",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _org_gnome_session_manager_signal_info_inhibitor_removed_ARG_pointers[] =
{
  &_org_gnome_session_manager_signal_info_inhibitor_removed_ARG_id.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _org_gnome_session_manager_signal_info_inhibitor_removed =
{
  {
    -1,
    (gchar *) "InhibitorRemoved",
    (GDBusArgInfo **) &_org_gnome_session_manager_signal_info_inhibitor_removed_ARG_pointers,
    NULL
  },
  "inhibitor-removed"
};

static const _ExtendedGDBusSignalInfo _org_gnome_session_manager_signal_info_session_running =
{
  {
    -1,
    (gchar *) "SessionRunning",
    NULL,
    NULL
  },
  "session-running"
};

static const _ExtendedGDBusSignalInfo _org_gnome_session_manager_signal_info_session_over =
{
  {
    -1,
    (gchar *) "SessionOver",
    NULL,
    NULL
  },
  "session-over"
};

static const GDBusSignalInfo * const _org_gnome_session_manager_signal_info_pointers[] =
{
  &_org_gnome_session_manager_signal_info_client_added.parent_struct,
  &_org_gnome_session_manager_signal_info_client_removed.parent_struct,
  &_org_gnome_session_manager_signal_info_inhibitor_added.parent_struct,
  &_org_gnome_session_manager_signal_info_inhibitor_removed.parent_struct,
  &_org_gnome_session_manager_signal_info_session_running.parent_struct,
  &_org_gnome_session_manager_signal_info_session_over.parent_struct,
  NULL
};

static const _ExtendedGDBusPropertyInfo _org_gnome_session_manager_property_info_session_name =
{
  {
    -1,
    (gchar *) "SessionName",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "session-name",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_gnome_session_manager_property_info_session_is_active =
{
  {
    -1,
    (gchar *) "SessionIsActive",
    (gchar *) "b",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "session-is-active",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _org_gnome_session_manager_property_info_inhibited_actions =
{
  {
    -1,
    (gchar *) "InhibitedActions",
    (gchar *) "u",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "inhibited-actions",
  FALSE,
  TRUE
};

static const GDBusPropertyInfo * const _org_gnome_session_manager_property_info_pointers[] =
{
  &_org_gnome_session_manager_property_info_session_name.parent_struct,
  &_org_gnome_session_manager_property_info_session_is_active.parent_struct,
  &_org_gnome_session_manager_property_info_inhibited_actions.parent_struct,
  NULL
};

static const _ExtendedGDBusInterfaceInfo _org_gnome_session_manager_interface_info =
{
  {
    -1,
    (gchar *) "org.gnome.SessionManager",
    (GDBusMethodInfo **) &_org_gnome_session_manager_method_info_pointers,
    (GDBusSignalInfo **) &_org_gnome_session_manager_signal_info_pointers,
    (GDBusPropertyInfo **) &_org_gnome_session_manager_property_info_pointers,
    NULL
  },
  "org-gnome-session-manager",
};


/**
 * org_gnome_session_manager_interface_info:
 *
 * Gets a machine-readable description of the <link linkend="gdbus-interface-org-gnome-SessionManager.top_of_page">org.gnome.SessionManager</link> D-Bus interface.
 *
 * Returns: (transfer none): A #GDBusInterfaceInfo. Do not free.
 */
GDBusInterfaceInfo *
org_gnome_session_manager_interface_info (void)
{
  return (GDBusInterfaceInfo *) &_org_gnome_session_manager_interface_info.parent_struct;
}

/**
 * org_gnome_session_manager_override_properties:
 * @klass: The class structure for a #GObject derived class.
 * @property_id_begin: The property id to assign to the first overridden property.
 *
 * Overrides all #GObject properties in the #OrgGnomeSessionManager interface for a concrete class.
 * The properties are overridden in the order they are defined.
 *
 * Returns: The last property id.
 */
guint
org_gnome_session_manager_override_properties (GObjectClass *klass, guint property_id_begin)
{
  g_object_class_override_property (klass, property_id_begin++, "session-name");
  g_object_class_override_property (klass, property_id_begin++, "session-is-active");
  g_object_class_override_property (klass, property_id_begin++, "inhibited-actions");
  return property_id_begin - 1;
}


inline static void
org_gnome_session_manager_signal_marshal_client_added (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_signal_marshal_client_removed (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_signal_marshal_inhibitor_added (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_signal_marshal_inhibitor_removed (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_signal_marshal_session_running (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__VOID (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_signal_marshal_session_over (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__VOID (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_setenv (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_get_locale (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_INT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_initialization_error (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_BOOLEAN (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_register_client (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_unregister_client (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_inhibit (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING_UINT_STRING_UINT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_uninhibit (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_UINT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_is_inhibited (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_UINT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_get_clients (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_get_inhibitors (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_is_autostart_condition_handled (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_shutdown (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_reboot (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_can_shutdown (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_logout (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_UINT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
org_gnome_session_manager_method_marshal_is_session_running (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}


/**
 * OrgGnomeSessionManager:
 *
 * Abstract interface type for the D-Bus interface <link linkend="gdbus-interface-org-gnome-SessionManager.top_of_page">org.gnome.SessionManager</link>.
 */

/**
 * OrgGnomeSessionManagerIface:
 * @parent_iface: The parent interface.
 * @handle_can_shutdown: Handler for the #OrgGnomeSessionManager::handle-can-shutdown signal.
 * @handle_get_clients: Handler for the #OrgGnomeSessionManager::handle-get-clients signal.
 * @handle_get_inhibitors: Handler for the #OrgGnomeSessionManager::handle-get-inhibitors signal.
 * @handle_get_locale: Handler for the #OrgGnomeSessionManager::handle-get-locale signal.
 * @handle_inhibit: Handler for the #OrgGnomeSessionManager::handle-inhibit signal.
 * @handle_initialization_error: Handler for the #OrgGnomeSessionManager::handle-initialization-error signal.
 * @handle_is_autostart_condition_handled: Handler for the #OrgGnomeSessionManager::handle-is-autostart-condition-handled signal.
 * @handle_is_inhibited: Handler for the #OrgGnomeSessionManager::handle-is-inhibited signal.
 * @handle_is_session_running: Handler for the #OrgGnomeSessionManager::handle-is-session-running signal.
 * @handle_logout: Handler for the #OrgGnomeSessionManager::handle-logout signal.
 * @handle_reboot: Handler for the #OrgGnomeSessionManager::handle-reboot signal.
 * @handle_register_client: Handler for the #OrgGnomeSessionManager::handle-register-client signal.
 * @handle_setenv: Handler for the #OrgGnomeSessionManager::handle-setenv signal.
 * @handle_shutdown: Handler for the #OrgGnomeSessionManager::handle-shutdown signal.
 * @handle_uninhibit: Handler for the #OrgGnomeSessionManager::handle-uninhibit signal.
 * @handle_unregister_client: Handler for the #OrgGnomeSessionManager::handle-unregister-client signal.
 * @get_inhibited_actions: Getter for the #OrgGnomeSessionManager:inhibited-actions property.
 * @get_session_is_active: Getter for the #OrgGnomeSessionManager:session-is-active property.
 * @get_session_name: Getter for the #OrgGnomeSessionManager:session-name property.
 * @client_added: Handler for the #OrgGnomeSessionManager::client-added signal.
 * @client_removed: Handler for the #OrgGnomeSessionManager::client-removed signal.
 * @inhibitor_added: Handler for the #OrgGnomeSessionManager::inhibitor-added signal.
 * @inhibitor_removed: Handler for the #OrgGnomeSessionManager::inhibitor-removed signal.
 * @session_over: Handler for the #OrgGnomeSessionManager::session-over signal.
 * @session_running: Handler for the #OrgGnomeSessionManager::session-running signal.
 *
 * Virtual table for the D-Bus interface <link linkend="gdbus-interface-org-gnome-SessionManager.top_of_page">org.gnome.SessionManager</link>.
 */

typedef OrgGnomeSessionManagerIface OrgGnomeSessionManagerInterface;
G_DEFINE_INTERFACE (OrgGnomeSessionManager, org_gnome_session_manager, G_TYPE_OBJECT)

static void
org_gnome_session_manager_default_init (OrgGnomeSessionManagerIface *iface)
{
  /* GObject signals for incoming D-Bus method calls: */
  /**
   * OrgGnomeSessionManager::handle-setenv:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_variable: Argument passed by remote caller.
   * @arg_value: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.Setenv">Setenv()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_setenv() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-setenv",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_setenv),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_setenv,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * OrgGnomeSessionManager::handle-get-locale:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_category: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.GetLocale">GetLocale()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_get_locale() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-locale",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_get_locale),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_get_locale,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_INT);

  /**
   * OrgGnomeSessionManager::handle-initialization-error:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_message: Argument passed by remote caller.
   * @arg_fatal: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.InitializationError">InitializationError()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_initialization_error() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-initialization-error",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_initialization_error),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_initialization_error,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_BOOLEAN);

  /**
   * OrgGnomeSessionManager::handle-register-client:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_app_id: Argument passed by remote caller.
   * @arg_client_startup_id: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.RegisterClient">RegisterClient()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_register_client() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-register-client",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_register_client),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_register_client,
    G_TYPE_BOOLEAN,
    3,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_STRING);

  /**
   * OrgGnomeSessionManager::handle-unregister-client:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_client_id: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.UnregisterClient">UnregisterClient()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_unregister_client() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-unregister-client",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_unregister_client),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_unregister_client,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgGnomeSessionManager::handle-inhibit:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_app_id: Argument passed by remote caller.
   * @arg_toplevel_xid: Argument passed by remote caller.
   * @arg_reason: Argument passed by remote caller.
   * @arg_flags: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.Inhibit">Inhibit()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_inhibit() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-inhibit",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_inhibit),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_inhibit,
    G_TYPE_BOOLEAN,
    5,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_UINT);

  /**
   * OrgGnomeSessionManager::handle-uninhibit:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_inhibit_cookie: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.Uninhibit">Uninhibit()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_uninhibit() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-uninhibit",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_uninhibit),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_uninhibit,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_UINT);

  /**
   * OrgGnomeSessionManager::handle-is-inhibited:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_flags: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.IsInhibited">IsInhibited()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_is_inhibited() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-is-inhibited",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_is_inhibited),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_is_inhibited,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_UINT);

  /**
   * OrgGnomeSessionManager::handle-get-clients:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.GetClients">GetClients()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_get_clients() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-clients",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_get_clients),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_get_clients,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * OrgGnomeSessionManager::handle-get-inhibitors:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.GetInhibitors">GetInhibitors()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_get_inhibitors() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-inhibitors",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_get_inhibitors),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_get_inhibitors,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * OrgGnomeSessionManager::handle-is-autostart-condition-handled:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_condition: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.IsAutostartConditionHandled">IsAutostartConditionHandled()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_is_autostart_condition_handled() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-is-autostart-condition-handled",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_is_autostart_condition_handled),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_is_autostart_condition_handled,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_STRING);

  /**
   * OrgGnomeSessionManager::handle-shutdown:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.Shutdown">Shutdown()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_shutdown() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-shutdown",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_shutdown),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_shutdown,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * OrgGnomeSessionManager::handle-reboot:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.Reboot">Reboot()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_reboot() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-reboot",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_reboot),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_reboot,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * OrgGnomeSessionManager::handle-can-shutdown:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.CanShutdown">CanShutdown()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_can_shutdown() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-can-shutdown",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_can_shutdown),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_can_shutdown,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * OrgGnomeSessionManager::handle-logout:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_mode: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.Logout">Logout()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_logout() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-logout",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_logout),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_logout,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_UINT);

  /**
   * OrgGnomeSessionManager::handle-is-session-running:
   * @object: A #OrgGnomeSessionManager.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager.IsSessionRunning">IsSessionRunning()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call org_gnome_session_manager_complete_is_session_running() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-is-session-running",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, handle_is_session_running),
    g_signal_accumulator_true_handled,
    NULL,
      org_gnome_session_manager_method_marshal_is_session_running,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /* GObject signals for received D-Bus signals: */
  /**
   * OrgGnomeSessionManager::client-added:
   * @object: A #OrgGnomeSessionManager.
   * @arg_id: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-org-gnome-SessionManager.ClientAdded">"ClientAdded"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _ORG_GNOME_SESSION_MANAGER_SIGNALS[_ORG_GNOME_SESSION_MANAGER_CLIENT_ADDED] =
    g_signal_new ("client-added",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, client_added),
      NULL,
      NULL,
      org_gnome_session_manager_signal_marshal_client_added,
      G_TYPE_NONE,
      1, G_TYPE_STRING);

  /**
   * OrgGnomeSessionManager::client-removed:
   * @object: A #OrgGnomeSessionManager.
   * @arg_id: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-org-gnome-SessionManager.ClientRemoved">"ClientRemoved"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _ORG_GNOME_SESSION_MANAGER_SIGNALS[_ORG_GNOME_SESSION_MANAGER_CLIENT_REMOVED] =
    g_signal_new ("client-removed",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, client_removed),
      NULL,
      NULL,
      org_gnome_session_manager_signal_marshal_client_removed,
      G_TYPE_NONE,
      1, G_TYPE_STRING);

  /**
   * OrgGnomeSessionManager::inhibitor-added:
   * @object: A #OrgGnomeSessionManager.
   * @arg_id: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-org-gnome-SessionManager.InhibitorAdded">"InhibitorAdded"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _ORG_GNOME_SESSION_MANAGER_SIGNALS[_ORG_GNOME_SESSION_MANAGER_INHIBITOR_ADDED] =
    g_signal_new ("inhibitor-added",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, inhibitor_added),
      NULL,
      NULL,
      org_gnome_session_manager_signal_marshal_inhibitor_added,
      G_TYPE_NONE,
      1, G_TYPE_STRING);

  /**
   * OrgGnomeSessionManager::inhibitor-removed:
   * @object: A #OrgGnomeSessionManager.
   * @arg_id: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-org-gnome-SessionManager.InhibitorRemoved">"InhibitorRemoved"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _ORG_GNOME_SESSION_MANAGER_SIGNALS[_ORG_GNOME_SESSION_MANAGER_INHIBITOR_REMOVED] =
    g_signal_new ("inhibitor-removed",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, inhibitor_removed),
      NULL,
      NULL,
      org_gnome_session_manager_signal_marshal_inhibitor_removed,
      G_TYPE_NONE,
      1, G_TYPE_STRING);

  /**
   * OrgGnomeSessionManager::session-running:
   * @object: A #OrgGnomeSessionManager.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-org-gnome-SessionManager.SessionRunning">"SessionRunning"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _ORG_GNOME_SESSION_MANAGER_SIGNALS[_ORG_GNOME_SESSION_MANAGER_SESSION_RUNNING] =
    g_signal_new ("session-running",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, session_running),
      NULL,
      NULL,
      org_gnome_session_manager_signal_marshal_session_running,
      G_TYPE_NONE,
      0);

  /**
   * OrgGnomeSessionManager::session-over:
   * @object: A #OrgGnomeSessionManager.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-org-gnome-SessionManager.SessionOver">"SessionOver"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _ORG_GNOME_SESSION_MANAGER_SIGNALS[_ORG_GNOME_SESSION_MANAGER_SESSION_OVER] =
    g_signal_new ("session-over",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (OrgGnomeSessionManagerIface, session_over),
      NULL,
      NULL,
      org_gnome_session_manager_signal_marshal_session_over,
      G_TYPE_NONE,
      0);

  /* GObject properties for D-Bus properties: */
  /**
   * OrgGnomeSessionManager:session-name:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-gnome-SessionManager.SessionName">"SessionName"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("session-name", "SessionName", "SessionName", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgGnomeSessionManager:session-is-active:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-gnome-SessionManager.SessionIsActive">"SessionIsActive"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_boolean ("session-is-active", "SessionIsActive", "SessionIsActive", FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * OrgGnomeSessionManager:inhibited-actions:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-org-gnome-SessionManager.InhibitedActions">"InhibitedActions"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_uint ("inhibited-actions", "InhibitedActions", "InhibitedActions", 0, G_MAXUINT32, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
}

/**
 * org_gnome_session_manager_get_session_name: (skip)
 * @object: A #OrgGnomeSessionManager.
 *
 * Gets the value of the <link linkend="gdbus-property-org-gnome-SessionManager.SessionName">"SessionName"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use org_gnome_session_manager_dup_session_name() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
org_gnome_session_manager_get_session_name (OrgGnomeSessionManager *object)
{
  g_return_val_if_fail (IS_ORG_GNOME_SESSION_MANAGER (object), NULL);

  return ORG_GNOME_SESSION_MANAGER_GET_IFACE (object)->get_session_name (object);
}

/**
 * org_gnome_session_manager_dup_session_name: (skip)
 * @object: A #OrgGnomeSessionManager.
 *
 * Gets a copy of the <link linkend="gdbus-property-org-gnome-SessionManager.SessionName">"SessionName"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
org_gnome_session_manager_dup_session_name (OrgGnomeSessionManager *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "session-name", &value, NULL);
  return value;
}

/**
 * org_gnome_session_manager_set_session_name: (skip)
 * @object: A #OrgGnomeSessionManager.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-gnome-SessionManager.SessionName">"SessionName"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_gnome_session_manager_set_session_name (OrgGnomeSessionManager *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "session-name", value, NULL);
}

/**
 * org_gnome_session_manager_get_session_is_active: (skip)
 * @object: A #OrgGnomeSessionManager.
 *
 * Gets the value of the <link linkend="gdbus-property-org-gnome-SessionManager.SessionIsActive">"SessionIsActive"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
gboolean 
org_gnome_session_manager_get_session_is_active (OrgGnomeSessionManager *object)
{
  g_return_val_if_fail (IS_ORG_GNOME_SESSION_MANAGER (object), FALSE);

  return ORG_GNOME_SESSION_MANAGER_GET_IFACE (object)->get_session_is_active (object);
}

/**
 * org_gnome_session_manager_set_session_is_active: (skip)
 * @object: A #OrgGnomeSessionManager.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-gnome-SessionManager.SessionIsActive">"SessionIsActive"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_gnome_session_manager_set_session_is_active (OrgGnomeSessionManager *object, gboolean value)
{
  g_object_set (G_OBJECT (object), "session-is-active", value, NULL);
}

/**
 * org_gnome_session_manager_get_inhibited_actions: (skip)
 * @object: A #OrgGnomeSessionManager.
 *
 * Gets the value of the <link linkend="gdbus-property-org-gnome-SessionManager.InhibitedActions">"InhibitedActions"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
guint 
org_gnome_session_manager_get_inhibited_actions (OrgGnomeSessionManager *object)
{
  g_return_val_if_fail (IS_ORG_GNOME_SESSION_MANAGER (object), 0);

  return ORG_GNOME_SESSION_MANAGER_GET_IFACE (object)->get_inhibited_actions (object);
}

/**
 * org_gnome_session_manager_set_inhibited_actions: (skip)
 * @object: A #OrgGnomeSessionManager.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-org-gnome-SessionManager.InhibitedActions">"InhibitedActions"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
org_gnome_session_manager_set_inhibited_actions (OrgGnomeSessionManager *object, guint value)
{
  g_object_set (G_OBJECT (object), "inhibited-actions", value, NULL);
}

/**
 * org_gnome_session_manager_emit_client_added:
 * @object: A #OrgGnomeSessionManager.
 * @arg_id: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-org-gnome-SessionManager.ClientAdded">"ClientAdded"</link> D-Bus signal.
 */
void
org_gnome_session_manager_emit_client_added (
    OrgGnomeSessionManager *object,
    const gchar *arg_id)
{
  g_signal_emit (object, _ORG_GNOME_SESSION_MANAGER_SIGNALS[_ORG_GNOME_SESSION_MANAGER_CLIENT_ADDED], 0, arg_id);
}

/**
 * org_gnome_session_manager_emit_client_removed:
 * @object: A #OrgGnomeSessionManager.
 * @arg_id: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-org-gnome-SessionManager.ClientRemoved">"ClientRemoved"</link> D-Bus signal.
 */
void
org_gnome_session_manager_emit_client_removed (
    OrgGnomeSessionManager *object,
    const gchar *arg_id)
{
  g_signal_emit (object, _ORG_GNOME_SESSION_MANAGER_SIGNALS[_ORG_GNOME_SESSION_MANAGER_CLIENT_REMOVED], 0, arg_id);
}

/**
 * org_gnome_session_manager_emit_inhibitor_added:
 * @object: A #OrgGnomeSessionManager.
 * @arg_id: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-org-gnome-SessionManager.InhibitorAdded">"InhibitorAdded"</link> D-Bus signal.
 */
void
org_gnome_session_manager_emit_inhibitor_added (
    OrgGnomeSessionManager *object,
    const gchar *arg_id)
{
  g_signal_emit (object, _ORG_GNOME_SESSION_MANAGER_SIGNALS[_ORG_GNOME_SESSION_MANAGER_INHIBITOR_ADDED], 0, arg_id);
}

/**
 * org_gnome_session_manager_emit_inhibitor_removed:
 * @object: A #OrgGnomeSessionManager.
 * @arg_id: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-org-gnome-SessionManager.InhibitorRemoved">"InhibitorRemoved"</link> D-Bus signal.
 */
void
org_gnome_session_manager_emit_inhibitor_removed (
    OrgGnomeSessionManager *object,
    const gchar *arg_id)
{
  g_signal_emit (object, _ORG_GNOME_SESSION_MANAGER_SIGNALS[_ORG_GNOME_SESSION_MANAGER_INHIBITOR_REMOVED], 0, arg_id);
}

/**
 * org_gnome_session_manager_emit_session_running:
 * @object: A #OrgGnomeSessionManager.
 *
 * Emits the <link linkend="gdbus-signal-org-gnome-SessionManager.SessionRunning">"SessionRunning"</link> D-Bus signal.
 */
void
org_gnome_session_manager_emit_session_running (
    OrgGnomeSessionManager *object)
{
  g_signal_emit (object, _ORG_GNOME_SESSION_MANAGER_SIGNALS[_ORG_GNOME_SESSION_MANAGER_SESSION_RUNNING], 0);
}

/**
 * org_gnome_session_manager_emit_session_over:
 * @object: A #OrgGnomeSessionManager.
 *
 * Emits the <link linkend="gdbus-signal-org-gnome-SessionManager.SessionOver">"SessionOver"</link> D-Bus signal.
 */
void
org_gnome_session_manager_emit_session_over (
    OrgGnomeSessionManager *object)
{
  g_signal_emit (object, _ORG_GNOME_SESSION_MANAGER_SIGNALS[_ORG_GNOME_SESSION_MANAGER_SESSION_OVER], 0);
}

/**
 * org_gnome_session_manager_call_setenv:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_variable: Argument to pass with the method invocation.
 * @arg_value: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.Setenv">Setenv()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_setenv_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_setenv_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_setenv (
    OrgGnomeSessionManager *proxy,
    const gchar *arg_variable,
    const gchar *arg_value,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "Setenv",
    g_variant_new ("(ss)",
                   arg_variable,
                   arg_value),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_setenv_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_setenv().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_setenv().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_setenv_finish (
    OrgGnomeSessionManager *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_setenv_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_variable: Argument to pass with the method invocation.
 * @arg_value: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.Setenv">Setenv()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_setenv() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_setenv_sync (
    OrgGnomeSessionManager *proxy,
    const gchar *arg_variable,
    const gchar *arg_value,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "Setenv",
    g_variant_new ("(ss)",
                   arg_variable,
                   arg_value),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_get_locale:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_category: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.GetLocale">GetLocale()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_get_locale_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_get_locale_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_get_locale (
    OrgGnomeSessionManager *proxy,
    gint arg_category,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetLocale",
    g_variant_new ("(i)",
                   arg_category),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_get_locale_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @out_value: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_get_locale().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_get_locale().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_get_locale_finish (
    OrgGnomeSessionManager *proxy,
    gchar **out_value,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_value);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_get_locale_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_category: Argument to pass with the method invocation.
 * @out_value: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.GetLocale">GetLocale()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_get_locale() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_get_locale_sync (
    OrgGnomeSessionManager *proxy,
    gint arg_category,
    gchar **out_value,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetLocale",
    g_variant_new ("(i)",
                   arg_category),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_value);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_initialization_error:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_message: Argument to pass with the method invocation.
 * @arg_fatal: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.InitializationError">InitializationError()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_initialization_error_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_initialization_error_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_initialization_error (
    OrgGnomeSessionManager *proxy,
    const gchar *arg_message,
    gboolean arg_fatal,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "InitializationError",
    g_variant_new ("(sb)",
                   arg_message,
                   arg_fatal),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_initialization_error_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_initialization_error().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_initialization_error().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_initialization_error_finish (
    OrgGnomeSessionManager *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_initialization_error_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_message: Argument to pass with the method invocation.
 * @arg_fatal: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.InitializationError">InitializationError()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_initialization_error() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_initialization_error_sync (
    OrgGnomeSessionManager *proxy,
    const gchar *arg_message,
    gboolean arg_fatal,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "InitializationError",
    g_variant_new ("(sb)",
                   arg_message,
                   arg_fatal),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_register_client:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_app_id: Argument to pass with the method invocation.
 * @arg_client_startup_id: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.RegisterClient">RegisterClient()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_register_client_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_register_client_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_register_client (
    OrgGnomeSessionManager *proxy,
    const gchar *arg_app_id,
    const gchar *arg_client_startup_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "RegisterClient",
    g_variant_new ("(ss)",
                   arg_app_id,
                   arg_client_startup_id),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_register_client_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @out_client_id: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_register_client().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_register_client().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_register_client_finish (
    OrgGnomeSessionManager *proxy,
    gchar **out_client_id,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(o)",
                 out_client_id);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_register_client_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_app_id: Argument to pass with the method invocation.
 * @arg_client_startup_id: Argument to pass with the method invocation.
 * @out_client_id: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.RegisterClient">RegisterClient()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_register_client() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_register_client_sync (
    OrgGnomeSessionManager *proxy,
    const gchar *arg_app_id,
    const gchar *arg_client_startup_id,
    gchar **out_client_id,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "RegisterClient",
    g_variant_new ("(ss)",
                   arg_app_id,
                   arg_client_startup_id),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(o)",
                 out_client_id);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_unregister_client:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_client_id: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.UnregisterClient">UnregisterClient()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_unregister_client_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_unregister_client_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_unregister_client (
    OrgGnomeSessionManager *proxy,
    const gchar *arg_client_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "UnregisterClient",
    g_variant_new ("(o)",
                   arg_client_id),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_unregister_client_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_unregister_client().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_unregister_client().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_unregister_client_finish (
    OrgGnomeSessionManager *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_unregister_client_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_client_id: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.UnregisterClient">UnregisterClient()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_unregister_client() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_unregister_client_sync (
    OrgGnomeSessionManager *proxy,
    const gchar *arg_client_id,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "UnregisterClient",
    g_variant_new ("(o)",
                   arg_client_id),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_inhibit:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_app_id: Argument to pass with the method invocation.
 * @arg_toplevel_xid: Argument to pass with the method invocation.
 * @arg_reason: Argument to pass with the method invocation.
 * @arg_flags: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.Inhibit">Inhibit()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_inhibit_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_inhibit_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_inhibit (
    OrgGnomeSessionManager *proxy,
    const gchar *arg_app_id,
    guint arg_toplevel_xid,
    const gchar *arg_reason,
    guint arg_flags,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "Inhibit",
    g_variant_new ("(susu)",
                   arg_app_id,
                   arg_toplevel_xid,
                   arg_reason,
                   arg_flags),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_inhibit_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @out_inhibit_cookie: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_inhibit().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_inhibit().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_inhibit_finish (
    OrgGnomeSessionManager *proxy,
    guint *out_inhibit_cookie,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_inhibit_cookie);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_inhibit_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_app_id: Argument to pass with the method invocation.
 * @arg_toplevel_xid: Argument to pass with the method invocation.
 * @arg_reason: Argument to pass with the method invocation.
 * @arg_flags: Argument to pass with the method invocation.
 * @out_inhibit_cookie: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.Inhibit">Inhibit()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_inhibit() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_inhibit_sync (
    OrgGnomeSessionManager *proxy,
    const gchar *arg_app_id,
    guint arg_toplevel_xid,
    const gchar *arg_reason,
    guint arg_flags,
    guint *out_inhibit_cookie,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "Inhibit",
    g_variant_new ("(susu)",
                   arg_app_id,
                   arg_toplevel_xid,
                   arg_reason,
                   arg_flags),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_inhibit_cookie);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_uninhibit:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_inhibit_cookie: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.Uninhibit">Uninhibit()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_uninhibit_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_uninhibit_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_uninhibit (
    OrgGnomeSessionManager *proxy,
    guint arg_inhibit_cookie,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "Uninhibit",
    g_variant_new ("(u)",
                   arg_inhibit_cookie),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_uninhibit_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_uninhibit().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_uninhibit().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_uninhibit_finish (
    OrgGnomeSessionManager *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_uninhibit_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_inhibit_cookie: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.Uninhibit">Uninhibit()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_uninhibit() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_uninhibit_sync (
    OrgGnomeSessionManager *proxy,
    guint arg_inhibit_cookie,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "Uninhibit",
    g_variant_new ("(u)",
                   arg_inhibit_cookie),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_is_inhibited:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_flags: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.IsInhibited">IsInhibited()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_is_inhibited_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_is_inhibited_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_is_inhibited (
    OrgGnomeSessionManager *proxy,
    guint arg_flags,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "IsInhibited",
    g_variant_new ("(u)",
                   arg_flags),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_is_inhibited_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @out_is_inhibited: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_is_inhibited().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_is_inhibited().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_is_inhibited_finish (
    OrgGnomeSessionManager *proxy,
    gboolean *out_is_inhibited,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_is_inhibited);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_is_inhibited_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_flags: Argument to pass with the method invocation.
 * @out_is_inhibited: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.IsInhibited">IsInhibited()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_is_inhibited() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_is_inhibited_sync (
    OrgGnomeSessionManager *proxy,
    guint arg_flags,
    gboolean *out_is_inhibited,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "IsInhibited",
    g_variant_new ("(u)",
                   arg_flags),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_is_inhibited);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_get_clients:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.GetClients">GetClients()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_get_clients_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_get_clients_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_get_clients (
    OrgGnomeSessionManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetClients",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_get_clients_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @out_clients: (out) (optional) (array zero-terminated=1): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_get_clients().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_get_clients().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_get_clients_finish (
    OrgGnomeSessionManager *proxy,
    gchar ***out_clients,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(^ao)",
                 out_clients);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_get_clients_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @out_clients: (out) (optional) (array zero-terminated=1): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.GetClients">GetClients()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_get_clients() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_get_clients_sync (
    OrgGnomeSessionManager *proxy,
    gchar ***out_clients,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetClients",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(^ao)",
                 out_clients);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_get_inhibitors:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.GetInhibitors">GetInhibitors()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_get_inhibitors_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_get_inhibitors_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_get_inhibitors (
    OrgGnomeSessionManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetInhibitors",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_get_inhibitors_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @out_inhibitors: (out) (optional) (array zero-terminated=1): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_get_inhibitors().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_get_inhibitors().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_get_inhibitors_finish (
    OrgGnomeSessionManager *proxy,
    gchar ***out_inhibitors,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(^ao)",
                 out_inhibitors);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_get_inhibitors_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @out_inhibitors: (out) (optional) (array zero-terminated=1): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.GetInhibitors">GetInhibitors()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_get_inhibitors() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_get_inhibitors_sync (
    OrgGnomeSessionManager *proxy,
    gchar ***out_inhibitors,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetInhibitors",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(^ao)",
                 out_inhibitors);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_is_autostart_condition_handled:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_condition: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.IsAutostartConditionHandled">IsAutostartConditionHandled()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_is_autostart_condition_handled_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_is_autostart_condition_handled_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_is_autostart_condition_handled (
    OrgGnomeSessionManager *proxy,
    const gchar *arg_condition,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "IsAutostartConditionHandled",
    g_variant_new ("(s)",
                   arg_condition),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_is_autostart_condition_handled_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @out_handled: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_is_autostart_condition_handled().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_is_autostart_condition_handled().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_is_autostart_condition_handled_finish (
    OrgGnomeSessionManager *proxy,
    gboolean *out_handled,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_handled);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_is_autostart_condition_handled_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_condition: Argument to pass with the method invocation.
 * @out_handled: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.IsAutostartConditionHandled">IsAutostartConditionHandled()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_is_autostart_condition_handled() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_is_autostart_condition_handled_sync (
    OrgGnomeSessionManager *proxy,
    const gchar *arg_condition,
    gboolean *out_handled,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "IsAutostartConditionHandled",
    g_variant_new ("(s)",
                   arg_condition),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_handled);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_shutdown:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.Shutdown">Shutdown()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_shutdown_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_shutdown_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_shutdown (
    OrgGnomeSessionManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "Shutdown",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_shutdown_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_shutdown().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_shutdown().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_shutdown_finish (
    OrgGnomeSessionManager *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_shutdown_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.Shutdown">Shutdown()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_shutdown() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_shutdown_sync (
    OrgGnomeSessionManager *proxy,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "Shutdown",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_reboot:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.Reboot">Reboot()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_reboot_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_reboot_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_reboot (
    OrgGnomeSessionManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "Reboot",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_reboot_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_reboot().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_reboot().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_reboot_finish (
    OrgGnomeSessionManager *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_reboot_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.Reboot">Reboot()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_reboot() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_reboot_sync (
    OrgGnomeSessionManager *proxy,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "Reboot",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_can_shutdown:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.CanShutdown">CanShutdown()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_can_shutdown_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_can_shutdown_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_can_shutdown (
    OrgGnomeSessionManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "CanShutdown",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_can_shutdown_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @out_is_available: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_can_shutdown().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_can_shutdown().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_can_shutdown_finish (
    OrgGnomeSessionManager *proxy,
    gboolean *out_is_available,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_is_available);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_can_shutdown_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @out_is_available: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.CanShutdown">CanShutdown()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_can_shutdown() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_can_shutdown_sync (
    OrgGnomeSessionManager *proxy,
    gboolean *out_is_available,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "CanShutdown",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_is_available);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_logout:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_mode: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.Logout">Logout()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_logout_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_logout_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_logout (
    OrgGnomeSessionManager *proxy,
    guint arg_mode,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "Logout",
    g_variant_new ("(u)",
                   arg_mode),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_logout_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_logout().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_logout().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_logout_finish (
    OrgGnomeSessionManager *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_logout_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @arg_mode: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.Logout">Logout()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_logout() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_logout_sync (
    OrgGnomeSessionManager *proxy,
    guint arg_mode,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "Logout",
    g_variant_new ("(u)",
                   arg_mode),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_is_session_running:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.IsSessionRunning">IsSessionRunning()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_call_is_session_running_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_call_is_session_running_sync() for the synchronous, blocking version of this method.
 */
void
org_gnome_session_manager_call_is_session_running (
    OrgGnomeSessionManager *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "IsSessionRunning",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * org_gnome_session_manager_call_is_session_running_finish:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @out_running: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_call_is_session_running().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with org_gnome_session_manager_call_is_session_running().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_is_session_running_finish (
    OrgGnomeSessionManager *proxy,
    gboolean *out_running,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_running);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_call_is_session_running_sync:
 * @proxy: A #OrgGnomeSessionManagerProxy.
 * @out_running: (out) (optional): Return location for return parameter or %NULL to ignore.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager.IsSessionRunning">IsSessionRunning()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_call_is_session_running() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
org_gnome_session_manager_call_is_session_running_sync (
    OrgGnomeSessionManager *proxy,
    gboolean *out_running,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "IsSessionRunning",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(b)",
                 out_running);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * org_gnome_session_manager_complete_setenv:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.Setenv">Setenv()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_setenv (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_gnome_session_manager_complete_get_locale:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @value: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.GetLocale">GetLocale()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_get_locale (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *value)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   value));
}

/**
 * org_gnome_session_manager_complete_initialization_error:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.InitializationError">InitializationError()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_initialization_error (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_gnome_session_manager_complete_register_client:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @client_id: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.RegisterClient">RegisterClient()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_register_client (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *client_id)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(o)",
                   client_id));
}

/**
 * org_gnome_session_manager_complete_unregister_client:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.UnregisterClient">UnregisterClient()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_unregister_client (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_gnome_session_manager_complete_inhibit:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @inhibit_cookie: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.Inhibit">Inhibit()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_inhibit (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    guint inhibit_cookie)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(u)",
                   inhibit_cookie));
}

/**
 * org_gnome_session_manager_complete_uninhibit:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.Uninhibit">Uninhibit()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_uninhibit (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_gnome_session_manager_complete_is_inhibited:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @is_inhibited: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.IsInhibited">IsInhibited()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_is_inhibited (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    gboolean is_inhibited)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(b)",
                   is_inhibited));
}

/**
 * org_gnome_session_manager_complete_get_clients:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @clients: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.GetClients">GetClients()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_get_clients (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *const *clients)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(^ao)",
                   clients));
}

/**
 * org_gnome_session_manager_complete_get_inhibitors:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @inhibitors: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.GetInhibitors">GetInhibitors()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_get_inhibitors (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    const gchar *const *inhibitors)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(^ao)",
                   inhibitors));
}

/**
 * org_gnome_session_manager_complete_is_autostart_condition_handled:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @handled: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.IsAutostartConditionHandled">IsAutostartConditionHandled()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_is_autostart_condition_handled (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    gboolean handled)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(b)",
                   handled));
}

/**
 * org_gnome_session_manager_complete_shutdown:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.Shutdown">Shutdown()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_shutdown (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_gnome_session_manager_complete_reboot:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.Reboot">Reboot()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_reboot (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_gnome_session_manager_complete_can_shutdown:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @is_available: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.CanShutdown">CanShutdown()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_can_shutdown (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    gboolean is_available)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(b)",
                   is_available));
}

/**
 * org_gnome_session_manager_complete_logout:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.Logout">Logout()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_logout (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/**
 * org_gnome_session_manager_complete_is_session_running:
 * @object: A #OrgGnomeSessionManager.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @running: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager.IsSessionRunning">IsSessionRunning()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
org_gnome_session_manager_complete_is_session_running (
    OrgGnomeSessionManager *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation,
    gboolean running)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(b)",
                   running));
}

/* ------------------------------------------------------------------------ */

/**
 * OrgGnomeSessionManagerProxy:
 *
 * The #OrgGnomeSessionManagerProxy structure contains only private data and should only be accessed using the provided API.
 */

/**
 * OrgGnomeSessionManagerProxyClass:
 * @parent_class: The parent class.
 *
 * Class structure for #OrgGnomeSessionManagerProxy.
 */

struct _OrgGnomeSessionManagerProxyPrivate
{
  GData *qdata;
};

static void org_gnome_session_manager_proxy_iface_init (OrgGnomeSessionManagerIface *iface);

#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (OrgGnomeSessionManagerProxy, org_gnome_session_manager_proxy, G_TYPE_DBUS_PROXY,
                         G_ADD_PRIVATE (OrgGnomeSessionManagerProxy)
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_GNOME_SESSION_MANAGER, org_gnome_session_manager_proxy_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (OrgGnomeSessionManagerProxy, org_gnome_session_manager_proxy, G_TYPE_DBUS_PROXY,
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_GNOME_SESSION_MANAGER, org_gnome_session_manager_proxy_iface_init))

#endif
static void
org_gnome_session_manager_proxy_finalize (GObject *object)
{
  OrgGnomeSessionManagerProxy *proxy = ORG_GNOME_SESSION_MANAGER_PROXY (object);
  g_datalist_clear (&proxy->priv->qdata);
  G_OBJECT_CLASS (org_gnome_session_manager_proxy_parent_class)->finalize (object);
}

static void
org_gnome_session_manager_proxy_get_property (GObject      *object,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  const _ExtendedGDBusPropertyInfo *info;
  GVariant *variant;
  g_assert (prop_id != 0 && prop_id - 1 < 3);
  info = (const _ExtendedGDBusPropertyInfo *) _org_gnome_session_manager_property_info_pointers[prop_id - 1];
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (object), info->parent_struct.name);
  if (info->use_gvariant)
    {
      g_value_set_variant (value, variant);
    }
  else
    {
      if (variant != NULL)
        g_dbus_gvariant_to_gvalue (variant, value);
    }
  if (variant != NULL)
    g_variant_unref (variant);
}

static void
org_gnome_session_manager_proxy_set_property_cb (GDBusProxy *proxy,
  GAsyncResult *res,
  gpointer      user_data)
{
  const _ExtendedGDBusPropertyInfo *info = user_data;
  GError *error;
  GVariant *_ret;
  error = NULL;
  _ret = g_dbus_proxy_call_finish (proxy, res, &error);
  if (!_ret)
    {
      g_warning ("Error setting property '%s' on interface org.gnome.SessionManager: %s (%s, %d)",
                 info->parent_struct.name, 
                 error->message, g_quark_to_string (error->domain), error->code);
      g_error_free (error);
    }
  else
    {
      g_variant_unref (_ret);
    }
}

static void
org_gnome_session_manager_proxy_set_property (GObject      *object,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  const _ExtendedGDBusPropertyInfo *info;
  GVariant *variant;
  g_assert (prop_id != 0 && prop_id - 1 < 3);
  info = (const _ExtendedGDBusPropertyInfo *) _org_gnome_session_manager_property_info_pointers[prop_id - 1];
  variant = g_dbus_gvalue_to_gvariant (value, G_VARIANT_TYPE (info->parent_struct.signature));
  g_dbus_proxy_call (G_DBUS_PROXY (object),
    "org.freedesktop.DBus.Properties.Set",
    g_variant_new ("(ssv)", "org.gnome.SessionManager", info->parent_struct.name, variant),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    NULL, (GAsyncReadyCallback) org_gnome_session_manager_proxy_set_property_cb, (GDBusPropertyInfo *) &info->parent_struct);
  g_variant_unref (variant);
}

static void
org_gnome_session_manager_proxy_g_signal (GDBusProxy *proxy,
  const gchar *sender_name G_GNUC_UNUSED,
  const gchar *signal_name,
  GVariant *parameters)
{
  _ExtendedGDBusSignalInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  gsize n;
  guint signal_id;
  info = (_ExtendedGDBusSignalInfo *) g_dbus_interface_info_lookup_signal ((GDBusInterfaceInfo *) &_org_gnome_session_manager_interface_info.parent_struct, signal_name);
  if (info == NULL)
    return;
  num_params = g_variant_n_children (parameters);
  paramv = g_new0 (GValue, num_params + 1);
  g_value_init (&paramv[0], TYPE_ORG_GNOME_SESSION_MANAGER);
  g_value_set_object (&paramv[0], proxy);
  g_variant_iter_init (&iter, parameters);
  n = 1;
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.args[n - 1];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_ORG_GNOME_SESSION_MANAGER);
  g_signal_emitv (paramv, signal_id, 0, NULL);
  for (n = 0; n < num_params + 1; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static void
org_gnome_session_manager_proxy_g_properties_changed (GDBusProxy *_proxy,
  GVariant *changed_properties,
  const gchar *const *invalidated_properties)
{
  OrgGnomeSessionManagerProxy *proxy = ORG_GNOME_SESSION_MANAGER_PROXY (_proxy);
  guint n;
  const gchar *key;
  GVariantIter *iter;
  _ExtendedGDBusPropertyInfo *info;
  g_variant_get (changed_properties, "a{sv}", &iter);
  while (g_variant_iter_next (iter, "{&sv}", &key, NULL))
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_gnome_session_manager_interface_info.parent_struct, key);
      g_datalist_remove_data (&proxy->priv->qdata, key);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
  g_variant_iter_free (iter);
  for (n = 0; invalidated_properties[n] != NULL; n++)
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_gnome_session_manager_interface_info.parent_struct, invalidated_properties[n]);
      g_datalist_remove_data (&proxy->priv->qdata, invalidated_properties[n]);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
}

static const gchar *
org_gnome_session_manager_proxy_get_session_name (OrgGnomeSessionManager *object)
{
  OrgGnomeSessionManagerProxy *proxy = ORG_GNOME_SESSION_MANAGER_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "SessionName");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static gboolean 
org_gnome_session_manager_proxy_get_session_is_active (OrgGnomeSessionManager *object)
{
  OrgGnomeSessionManagerProxy *proxy = ORG_GNOME_SESSION_MANAGER_PROXY (object);
  GVariant *variant;
  gboolean value = FALSE;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "SessionIsActive");
  if (variant != NULL)
    {
      value = g_variant_get_boolean (variant);
      g_variant_unref (variant);
    }
  return value;
}

static guint 
org_gnome_session_manager_proxy_get_inhibited_actions (OrgGnomeSessionManager *object)
{
  OrgGnomeSessionManagerProxy *proxy = ORG_GNOME_SESSION_MANAGER_PROXY (object);
  GVariant *variant;
  guint value = 0;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "InhibitedActions");
  if (variant != NULL)
    {
      value = g_variant_get_uint32 (variant);
      g_variant_unref (variant);
    }
  return value;
}

static void
org_gnome_session_manager_proxy_init (OrgGnomeSessionManagerProxy *proxy)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  proxy->priv = org_gnome_session_manager_proxy_get_instance_private (proxy);
#else
  proxy->priv = G_TYPE_INSTANCE_GET_PRIVATE (proxy, TYPE_ORG_GNOME_SESSION_MANAGER_PROXY, OrgGnomeSessionManagerProxyPrivate);
#endif

  g_dbus_proxy_set_interface_info (G_DBUS_PROXY (proxy), org_gnome_session_manager_interface_info ());
}

static void
org_gnome_session_manager_proxy_class_init (OrgGnomeSessionManagerProxyClass *klass)
{
  GObjectClass *gobject_class;
  GDBusProxyClass *proxy_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize     = org_gnome_session_manager_proxy_finalize;
  gobject_class->get_property = org_gnome_session_manager_proxy_get_property;
  gobject_class->set_property = org_gnome_session_manager_proxy_set_property;

  proxy_class = G_DBUS_PROXY_CLASS (klass);
  proxy_class->g_signal = org_gnome_session_manager_proxy_g_signal;
  proxy_class->g_properties_changed = org_gnome_session_manager_proxy_g_properties_changed;

  org_gnome_session_manager_override_properties (gobject_class, 1);

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (OrgGnomeSessionManagerProxyPrivate));
#endif
}

static void
org_gnome_session_manager_proxy_iface_init (OrgGnomeSessionManagerIface *iface)
{
  iface->get_session_name = org_gnome_session_manager_proxy_get_session_name;
  iface->get_session_is_active = org_gnome_session_manager_proxy_get_session_is_active;
  iface->get_inhibited_actions = org_gnome_session_manager_proxy_get_inhibited_actions;
}

/**
 * org_gnome_session_manager_proxy_new:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-gnome-SessionManager.top_of_page">org.gnome.SessionManager</link>. See g_dbus_proxy_new() for more details.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_proxy_new_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_proxy_new_sync() for the synchronous, blocking version of this constructor.
 */
void
org_gnome_session_manager_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_ORG_GNOME_SESSION_MANAGER_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.gnome.SessionManager", NULL);
}

/**
 * org_gnome_session_manager_proxy_new_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_proxy_new().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with org_gnome_session_manager_proxy_new().
 *
 * Returns: (transfer full) (type OrgGnomeSessionManagerProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgGnomeSessionManager *
org_gnome_session_manager_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return ORG_GNOME_SESSION_MANAGER (ret);
  else
    return NULL;
}

/**
 * org_gnome_session_manager_proxy_new_sync:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Synchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-gnome-SessionManager.top_of_page">org.gnome.SessionManager</link>. See g_dbus_proxy_new_sync() for more details.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_proxy_new() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type OrgGnomeSessionManagerProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgGnomeSessionManager *
org_gnome_session_manager_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_ORG_GNOME_SESSION_MANAGER_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.gnome.SessionManager", NULL);
  if (ret != NULL)
    return ORG_GNOME_SESSION_MANAGER (ret);
  else
    return NULL;
}


/**
 * org_gnome_session_manager_proxy_new_for_bus:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Like org_gnome_session_manager_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call org_gnome_session_manager_proxy_new_for_bus_finish() to get the result of the operation.
 *
 * See org_gnome_session_manager_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
 */
void
org_gnome_session_manager_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_ORG_GNOME_SESSION_MANAGER_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.gnome.SessionManager", NULL);
}

/**
 * org_gnome_session_manager_proxy_new_for_bus_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to org_gnome_session_manager_proxy_new_for_bus().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with org_gnome_session_manager_proxy_new_for_bus().
 *
 * Returns: (transfer full) (type OrgGnomeSessionManagerProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgGnomeSessionManager *
org_gnome_session_manager_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return ORG_GNOME_SESSION_MANAGER (ret);
  else
    return NULL;
}

/**
 * org_gnome_session_manager_proxy_new_for_bus_sync:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Like org_gnome_session_manager_proxy_new_sync() but takes a #GBusType instead of a #GDBusConnection.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See org_gnome_session_manager_proxy_new_for_bus() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type OrgGnomeSessionManagerProxy): The constructed proxy object or %NULL if @error is set.
 */
OrgGnomeSessionManager *
org_gnome_session_manager_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_ORG_GNOME_SESSION_MANAGER_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.gnome.SessionManager", NULL);
  if (ret != NULL)
    return ORG_GNOME_SESSION_MANAGER (ret);
  else
    return NULL;
}


/* ------------------------------------------------------------------------ */

/**
 * OrgGnomeSessionManagerSkeleton:
 *
 * The #OrgGnomeSessionManagerSkeleton structure contains only private data and should only be accessed using the provided API.
 */

/**
 * OrgGnomeSessionManagerSkeletonClass:
 * @parent_class: The parent class.
 *
 * Class structure for #OrgGnomeSessionManagerSkeleton.
 */

struct _OrgGnomeSessionManagerSkeletonPrivate
{
  GValue *properties;
  GList *changed_properties;
  GSource *changed_properties_idle_source;
  GMainContext *context;
  GMutex lock;
};

static void
_org_gnome_session_manager_skeleton_handle_method_call (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name,
  const gchar *method_name,
  GVariant *parameters,
  GDBusMethodInvocation *invocation,
  gpointer user_data)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (user_data);
  _ExtendedGDBusMethodInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  guint num_extra;
  gsize n;
  guint signal_id;
  GValue return_value = G_VALUE_INIT;
  info = (_ExtendedGDBusMethodInfo *) g_dbus_method_invocation_get_method_info (invocation);
  g_assert (info != NULL);
  num_params = g_variant_n_children (parameters);
  num_extra = info->pass_fdlist ? 3 : 2;  paramv = g_new0 (GValue, num_params + num_extra);
  n = 0;
  g_value_init (&paramv[n], TYPE_ORG_GNOME_SESSION_MANAGER);
  g_value_set_object (&paramv[n++], skeleton);
  g_value_init (&paramv[n], G_TYPE_DBUS_METHOD_INVOCATION);
  g_value_set_object (&paramv[n++], invocation);
  if (info->pass_fdlist)
    {
#ifdef G_OS_UNIX
      g_value_init (&paramv[n], G_TYPE_UNIX_FD_LIST);
      g_value_set_object (&paramv[n++], g_dbus_message_get_unix_fd_list (g_dbus_method_invocation_get_message (invocation)));
#else
      g_assert_not_reached ();
#endif
    }
  g_variant_iter_init (&iter, parameters);
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.in_args[n - num_extra];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_ORG_GNOME_SESSION_MANAGER);
  g_value_init (&return_value, G_TYPE_BOOLEAN);
  g_signal_emitv (paramv, signal_id, 0, &return_value);
  if (!g_value_get_boolean (&return_value))
    g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD, "Method %s is not implemented on interface %s", method_name, interface_name);
  g_value_unset (&return_value);
  for (n = 0; n < num_params + num_extra; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static GVariant *
_org_gnome_session_manager_skeleton_handle_get_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GError **error,
  gpointer user_data)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  GVariant *ret;
  ret = NULL;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_gnome_session_manager_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      g_value_init (&value, pspec->value_type);
      g_object_get_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      ret = g_dbus_gvalue_to_gvariant (&value, G_VARIANT_TYPE (info->parent_struct.signature));
      g_value_unset (&value);
    }
  return ret;
}

static gboolean
_org_gnome_session_manager_skeleton_handle_set_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GVariant *variant,
  GError **error,
  gpointer user_data)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  gboolean ret;
  ret = FALSE;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_org_gnome_session_manager_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      if (info->use_gvariant)
        g_value_set_variant (&value, variant);
      else
        g_dbus_gvariant_to_gvalue (variant, &value);
      g_object_set_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      g_value_unset (&value);
      ret = TRUE;
    }
  return ret;
}

static const GDBusInterfaceVTable _org_gnome_session_manager_skeleton_vtable =
{
  _org_gnome_session_manager_skeleton_handle_method_call,
  _org_gnome_session_manager_skeleton_handle_get_property,
  _org_gnome_session_manager_skeleton_handle_set_property,
  {NULL}
};

static GDBusInterfaceInfo *
org_gnome_session_manager_skeleton_dbus_interface_get_info (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return org_gnome_session_manager_interface_info ();
}

static GDBusInterfaceVTable *
org_gnome_session_manager_skeleton_dbus_interface_get_vtable (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return (GDBusInterfaceVTable *) &_org_gnome_session_manager_skeleton_vtable;
}

static GVariant *
org_gnome_session_manager_skeleton_dbus_interface_get_properties (GDBusInterfaceSkeleton *_skeleton)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (_skeleton);

  GVariantBuilder builder;
  guint n;
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
#else
  g_variant_builder_init(&builder, G_VARIANT_TYPE ("a{sv}"));
#endif
  if (_org_gnome_session_manager_interface_info.parent_struct.properties == NULL)
    goto out;
  for (n = 0; _org_gnome_session_manager_interface_info.parent_struct.properties[n] != NULL; n++)
    {
      GDBusPropertyInfo *info = _org_gnome_session_manager_interface_info.parent_struct.properties[n];
      if (info->flags & G_DBUS_PROPERTY_INFO_FLAGS_READABLE)
        {
          GVariant *value;
          value = _org_gnome_session_manager_skeleton_handle_get_property (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)), NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.gnome.SessionManager", info->name, NULL, skeleton);
          if (value != NULL)
            {
              g_variant_take_ref (value);
              g_variant_builder_add (&builder, "{sv}", info->name, value);
              g_variant_unref (value);
            }
        }
    }
out:
  return g_variant_builder_end (&builder);
}

static gboolean _org_gnome_session_manager_emit_changed (gpointer user_data);

static void
org_gnome_session_manager_skeleton_dbus_interface_flush (GDBusInterfaceSkeleton *_skeleton)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (_skeleton);
  gboolean emit_changed = FALSE;

  g_mutex_lock (&skeleton->priv->lock);
  if (skeleton->priv->changed_properties_idle_source != NULL)
    {
      g_source_destroy (skeleton->priv->changed_properties_idle_source);
      skeleton->priv->changed_properties_idle_source = NULL;
      emit_changed = TRUE;
    }
  g_mutex_unlock (&skeleton->priv->lock);

  if (emit_changed)
    _org_gnome_session_manager_emit_changed (skeleton);
}

static void
_org_gnome_session_manager_on_signal_client_added (
    OrgGnomeSessionManager *object,
    const gchar *arg_id)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(o)",
                   arg_id));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.gnome.SessionManager", "ClientAdded",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void
_org_gnome_session_manager_on_signal_client_removed (
    OrgGnomeSessionManager *object,
    const gchar *arg_id)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(o)",
                   arg_id));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.gnome.SessionManager", "ClientRemoved",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void
_org_gnome_session_manager_on_signal_inhibitor_added (
    OrgGnomeSessionManager *object,
    const gchar *arg_id)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(o)",
                   arg_id));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.gnome.SessionManager", "InhibitorAdded",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void
_org_gnome_session_manager_on_signal_inhibitor_removed (
    OrgGnomeSessionManager *object,
    const gchar *arg_id)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(o)",
                   arg_id));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.gnome.SessionManager", "InhibitorRemoved",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void
_org_gnome_session_manager_on_signal_session_running (
    OrgGnomeSessionManager *object)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("()"));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.gnome.SessionManager", "SessionRunning",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void
_org_gnome_session_manager_on_signal_session_over (
    OrgGnomeSessionManager *object)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("()"));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.gnome.SessionManager", "SessionOver",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void org_gnome_session_manager_skeleton_iface_init (OrgGnomeSessionManagerIface *iface);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (OrgGnomeSessionManagerSkeleton, org_gnome_session_manager_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_ADD_PRIVATE (OrgGnomeSessionManagerSkeleton)
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_GNOME_SESSION_MANAGER, org_gnome_session_manager_skeleton_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (OrgGnomeSessionManagerSkeleton, org_gnome_session_manager_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_IMPLEMENT_INTERFACE (TYPE_ORG_GNOME_SESSION_MANAGER, org_gnome_session_manager_skeleton_iface_init))

#endif
static void
org_gnome_session_manager_skeleton_finalize (GObject *object)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (object);
  guint n;
  for (n = 0; n < 3; n++)
    g_value_unset (&skeleton->priv->properties[n]);
  g_free (skeleton->priv->properties);
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
  if (skeleton->priv->changed_properties_idle_source != NULL)
    g_source_destroy (skeleton->priv->changed_properties_idle_source);
  g_main_context_unref (skeleton->priv->context);
  g_mutex_clear (&skeleton->priv->lock);
  G_OBJECT_CLASS (org_gnome_session_manager_skeleton_parent_class)->finalize (object);
}

static void
org_gnome_session_manager_skeleton_get_property (GObject      *object,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (object);
  g_assert (prop_id != 0 && prop_id - 1 < 3);
  g_mutex_lock (&skeleton->priv->lock);
  g_value_copy (&skeleton->priv->properties[prop_id - 1], value);
  g_mutex_unlock (&skeleton->priv->lock);
}

static gboolean
_org_gnome_session_manager_emit_changed (gpointer user_data)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (user_data);
  GList *l;
  GVariantBuilder builder;
  GVariantBuilder invalidated_builder;
  guint num_changes;

  g_mutex_lock (&skeleton->priv->lock);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
  g_variant_builder_init_static (&invalidated_builder, G_VARIANT_TYPE ("as"));
#else
  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
  g_variant_builder_init (&invalidated_builder, G_VARIANT_TYPE ("as"));
#endif
  for (l = skeleton->priv->changed_properties, num_changes = 0; l != NULL; l = l->next)
    {
      ChangedProperty *cp = l->data;
      GVariant *variant;
      const GValue *cur_value;

      cur_value = &skeleton->priv->properties[cp->prop_id - 1];
      if (!_g_value_equal (cur_value, &cp->orig_value))
        {
          variant = g_dbus_gvalue_to_gvariant (cur_value, G_VARIANT_TYPE (cp->info->parent_struct.signature));
          g_variant_builder_add (&builder, "{sv}", cp->info->parent_struct.name, variant);
          g_variant_unref (variant);
          num_changes++;
        }
    }
  if (num_changes > 0)
    {
      GList *connections, *ll;
      GVariant *signal_variant;
      signal_variant = g_variant_ref_sink (g_variant_new ("(sa{sv}as)", "org.gnome.SessionManager",
                                           &builder, &invalidated_builder));
      connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));
      for (ll = connections; ll != NULL; ll = ll->next)
        {
          GDBusConnection *connection = ll->data;

          g_dbus_connection_emit_signal (connection,
                                         NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)),
                                         "org.freedesktop.DBus.Properties",
                                         "PropertiesChanged",
                                         signal_variant,
                                         NULL);
        }
      g_variant_unref (signal_variant);
      g_list_free_full (connections, g_object_unref);
    }
  else
    {
      g_variant_builder_clear (&builder);
      g_variant_builder_clear (&invalidated_builder);
    }
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
  skeleton->priv->changed_properties = NULL;
  skeleton->priv->changed_properties_idle_source = NULL;
  g_mutex_unlock (&skeleton->priv->lock);
  return FALSE;
}

static void
_org_gnome_session_manager_schedule_emit_changed (OrgGnomeSessionManagerSkeleton *skeleton, const _ExtendedGDBusPropertyInfo *info, guint prop_id, const GValue *orig_value)
{
  ChangedProperty *cp;
  GList *l;
  cp = NULL;
  for (l = skeleton->priv->changed_properties; l != NULL; l = l->next)
    {
      ChangedProperty *i_cp = l->data;
      if (i_cp->info == info)
        {
          cp = i_cp;
          break;
        }
    }
  if (cp == NULL)
    {
      cp = g_new0 (ChangedProperty, 1);
      cp->prop_id = prop_id;
      cp->info = info;
      skeleton->priv->changed_properties = g_list_prepend (skeleton->priv->changed_properties, cp);
      g_value_init (&cp->orig_value, G_VALUE_TYPE (orig_value));
      g_value_copy (orig_value, &cp->orig_value);
    }
}

static void
org_gnome_session_manager_skeleton_notify (GObject      *object,
  GParamSpec *pspec G_GNUC_UNUSED)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (object);
  g_mutex_lock (&skeleton->priv->lock);
  if (skeleton->priv->changed_properties != NULL &&
      skeleton->priv->changed_properties_idle_source == NULL)
    {
      skeleton->priv->changed_properties_idle_source = g_idle_source_new ();
      g_source_set_priority (skeleton->priv->changed_properties_idle_source, G_PRIORITY_DEFAULT);
      g_source_set_callback (skeleton->priv->changed_properties_idle_source, _org_gnome_session_manager_emit_changed, g_object_ref (skeleton), (GDestroyNotify) g_object_unref);
      g_source_set_name (skeleton->priv->changed_properties_idle_source, "[generated] _org_gnome_session_manager_emit_changed");
      g_source_attach (skeleton->priv->changed_properties_idle_source, skeleton->priv->context);
      g_source_unref (skeleton->priv->changed_properties_idle_source);
    }
  g_mutex_unlock (&skeleton->priv->lock);
}

static void
org_gnome_session_manager_skeleton_set_property (GObject      *object,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec)
{
  const _ExtendedGDBusPropertyInfo *info;
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (object);
  g_assert (prop_id != 0 && prop_id - 1 < 3);
  info = (const _ExtendedGDBusPropertyInfo *) _org_gnome_session_manager_property_info_pointers[prop_id - 1];
  g_mutex_lock (&skeleton->priv->lock);
  g_object_freeze_notify (object);
  if (!_g_value_equal (value, &skeleton->priv->properties[prop_id - 1]))
    {
      if (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)) != NULL &&
          info->emits_changed_signal)
        _org_gnome_session_manager_schedule_emit_changed (skeleton, info, prop_id, &skeleton->priv->properties[prop_id - 1]);
      g_value_copy (value, &skeleton->priv->properties[prop_id - 1]);
      g_object_notify_by_pspec (object, pspec);
    }
  g_mutex_unlock (&skeleton->priv->lock);
  g_object_thaw_notify (object);
}

static void
org_gnome_session_manager_skeleton_init (OrgGnomeSessionManagerSkeleton *skeleton)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  skeleton->priv = org_gnome_session_manager_skeleton_get_instance_private (skeleton);
#else
  skeleton->priv = G_TYPE_INSTANCE_GET_PRIVATE (skeleton, TYPE_ORG_GNOME_SESSION_MANAGER_SKELETON, OrgGnomeSessionManagerSkeletonPrivate);
#endif

  g_mutex_init (&skeleton->priv->lock);
  skeleton->priv->context = g_main_context_ref_thread_default ();
  skeleton->priv->properties = g_new0 (GValue, 3);
  g_value_init (&skeleton->priv->properties[0], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[1], G_TYPE_BOOLEAN);
  g_value_init (&skeleton->priv->properties[2], G_TYPE_UINT);
}

static const gchar *
org_gnome_session_manager_skeleton_get_session_name (OrgGnomeSessionManager *object)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[0]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static gboolean 
org_gnome_session_manager_skeleton_get_session_is_active (OrgGnomeSessionManager *object)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (object);
  gboolean value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_boolean (&(skeleton->priv->properties[1]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static guint 
org_gnome_session_manager_skeleton_get_inhibited_actions (OrgGnomeSessionManager *object)
{
  OrgGnomeSessionManagerSkeleton *skeleton = ORG_GNOME_SESSION_MANAGER_SKELETON (object);
  guint value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_uint (&(skeleton->priv->properties[2]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static void
org_gnome_session_manager_skeleton_class_init (OrgGnomeSessionManagerSkeletonClass *klass)
{
  GObjectClass *gobject_class;
  GDBusInterfaceSkeletonClass *skeleton_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize = org_gnome_session_manager_skeleton_finalize;
  gobject_class->get_property = org_gnome_session_manager_skeleton_get_property;
  gobject_class->set_property = org_gnome_session_manager_skeleton_set_property;
  gobject_class->notify       = org_gnome_session_manager_skeleton_notify;


  org_gnome_session_manager_override_properties (gobject_class, 1);

  skeleton_class = G_DBUS_INTERFACE_SKELETON_CLASS (klass);
  skeleton_class->get_info = org_gnome_session_manager_skeleton_dbus_interface_get_info;
  skeleton_class->get_properties = org_gnome_session_manager_skeleton_dbus_interface_get_properties;
  skeleton_class->flush = org_gnome_session_manager_skeleton_dbus_interface_flush;
  skeleton_class->get_vtable = org_gnome_session_manager_skeleton_dbus_interface_get_vtable;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (OrgGnomeSessionManagerSkeletonPrivate));
#endif
}

static void
org_gnome_session_manager_skeleton_iface_init (OrgGnomeSessionManagerIface *iface)
{
  iface->client_added = _org_gnome_session_manager_on_signal_client_added;
  iface->client_removed = _org_gnome_session_manager_on_signal_client_removed;
  iface->inhibitor_added = _org_gnome_session_manager_on_signal_inhibitor_added;
  iface->inhibitor_removed = _org_gnome_session_manager_on_signal_inhibitor_removed;
  iface->session_running = _org_gnome_session_manager_on_signal_session_running;
  iface->session_over = _org_gnome_session_manager_on_signal_session_over;
  iface->get_session_name = org_gnome_session_manager_skeleton_get_session_name;
  iface->get_session_is_active = org_gnome_session_manager_skeleton_get_session_is_active;
  iface->get_inhibited_actions = org_gnome_session_manager_skeleton_get_inhibited_actions;
}

/**
 * org_gnome_session_manager_skeleton_new:
 *
 * Creates a skeleton object for the D-Bus interface <link linkend="gdbus-interface-org-gnome-SessionManager.top_of_page">org.gnome.SessionManager</link>.
 *
 * Returns: (transfer full) (type OrgGnomeSessionManagerSkeleton): The skeleton object.
 */
OrgGnomeSessionManager *
org_gnome_session_manager_skeleton_new (void)
{
  return ORG_GNOME_SESSION_MANAGER (g_object_new (TYPE_ORG_GNOME_SESSION_MANAGER_SKELETON, NULL));
}

